Package: praatpicture
Title: 'Praat Picture' Style Plots of Acoustic Data
Version: 1.8.0
Authors@R: 
    person("Rasmus", "Puggaard-Rode", 
    email = "rasmus.puggaard-rode@ling-phil.ox.ac.uk", 
    role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-4522-9987"))
Description: Quickly and easily generate plots of acoustic data aligned with transcriptions similar to those made in 'Praat' using either derived signals generated directly in R with 'wrassp' or imported derived signals from 'Praat'. Provides easy and fast out-of-the-box solutions but also a high extent of  flexibility. Also provides options for embedding audio in figures and animating figures.
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: av (>= 0.9.0), bslib (>= 0.6.1), crayon (>= 1.5.2), emuR (>=
        2.4.2), gifski (>= 1.12.0.2), graphics (>= 4.3.2), grDevices
        (>= 4.3.2), gsignal (>= 0.3.5), ipa (>= 0.1.0), multitaper (>=
        1.0-17), phonTools (>= 0.2.2.2), rPraat (>= 1.3.2.1),
        rstudioapi (>= 0.15.0), shiny (>= 1.8.1.1), shinyjs (>= 2.1.0),
        soundgen (>= 2.6.2), tuneR (>= 1.4.6), utils (>= 4.3.2), wrassp
        (>= 1.0.4), zoo (>= 1.8-12)
URL: https://github.com/rpuggaardrode/praatpicture
BugReports: https://github.com/rpuggaardrode/praatpicture/issues
NeedsCompilation: no
Packaged: 2026-04-22 15:08:50 UTC; cpgl0183
Author: Rasmus Puggaard-Rode [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-4522-9987>)
Maintainer: Rasmus Puggaard-Rode <rasmus.puggaard-rode@ling-phil.ox.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-22 15:40:02 UTC
Built: R 4.6.0; ; 2026-04-30 15:16:39 UTC; unix

Package: praatpicture
Title: 'Praat Picture' Style Plots of Acoustic Data
Version: 1.9.0
Authors@R: 
    person("Rasmus", "Puggaard-Rode", 
    email = "rasmus.puggaard-rode@ling-phil.ox.ac.uk", 
    role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-4522-9987"))
Description: Quickly and easily generate plots of acoustic data aligned with transcriptions similar to those made in 'Praat' using either derived signals generated directly in R with 'wrassp' or imported derived signals from 'Praat'. Provides easy and fast out-of-the-box solutions but also a high extent of  flexibility. Also provides options for embedding audio in figures and animating figures.
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: av (>= 0.9.0), bslib (>= 0.6.1), crayon (>= 1.5.2), emuR (>=
        2.4.2), gifski (>= 1.12.0.2), graphics (>= 4.3.2), grDevices
        (>= 4.3.2), gsignal (>= 0.3.5), ipa (>= 0.1.0), multitaper (>=
        1.0-17), phonTools (>= 0.2.2.2), rPraat (>= 1.3.2.1),
        rstudioapi (>= 0.15.0), shiny (>= 1.8.1.1), shinyjs (>= 2.1.0),
        soundgen (>= 2.6.2), tuneR (>= 1.4.6), utils (>= 4.3.2), wrassp
        (>= 1.0.4), zoo (>= 1.8-12)
URL: https://github.com/rpuggaardrode/praatpicture
BugReports: https://github.com/rpuggaardrode/praatpicture/issues
NeedsCompilation: no
Packaged: 2026-07-20 15:04:41 UTC; cpgl0183
Author: Rasmus Puggaard-Rode [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-4522-9987>)
Maintainer: Rasmus Puggaard-Rode <rasmus.puggaard-rode@ling-phil.ox.ac.uk>
Repository: CRAN
Date/Publication: 2026-07-20 15:30:07 UTC

# Generated by roxygen2: do not edit by hand

export(conv2sc)
export(draw_arrow)
export(draw_lines)
export(draw_rectangle)
export(draw_spectralslice)
export(emupicture)
export(formantplot)
export(intensity_overlay)
export(intensityplot)
export(make_TextGrid)
export(make_annot)
export(pitch_overlay)
export(pitchplot)
export(praatanimation)
export(praatpicture)
export(shiny_praatpicture)
export(specplot)
export(talking_praatpicture)
export(tg_createTier)
export(tg_stylize)
export(tgplot)
export(waveplot)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# praatpicture

If you’ve ever had to fiddle around with getting a Praat Picture to look
and export just right in order to insert it in an R markdown file or
slide deck, then this package is for you! Even if this isn’t the case,
the package offers some very useful flexibility. It’s a simple set of
functions designed to emulate some common plots made in Praat Picture
with base R plotting. It allows for flexibly combining waveforms,
spectrograms, TextGrids, pitch tracks, formant tracks, and intensity
tracks in a single plot. The derived acoustic plots can all be generated
directly in R, or (except in the case of spectrograms) generated in
Praat.

TextGrids can be plotted as long as there is a file with the `.TextGrid`
extension in the same directory and with the same base name as the sound
file. If there are files with the `.PitchTier`, `.Formant`, and
`.IntensityTier` extensions generated in Praat, these are loaded in
using the `rPraat` package and used for plotting. Otherwise, signal
processing functions from the `wrassp` package are used in a way that
emulates Praat default settings as closely as possible. Spectrograms are
generated with the `phonTools` package.

If you are an `emuR` user, the `emupicture` function will help make
Praat Picture style plots of sound files, annotations, and possibly SSFF
files from an EMU-database.

For more information on how to use the package, see the
[manual](https://rpuggaardrode.github.io/praatpicture_manual/).

If you run into bugs or have suggested changes, please let me know!

## Installation

`praatpicture` is on CRAN and can be installed in the usual way:

``` r
install.packages('praatpicture')
```

You can install the latest development version of `praatpicture` like
so:

``` r
#install.packages('devtools')
devtools::install_github('rpuggaardrode/praatpicture')
```

New features will be available from Github before they are available on
CRAN.

ui <- bslib::page_sidebar(
  shinyjs::useShinyjs(),
  title = 'praatpicture: Praat-style plots of acoustic data in R',
  sidebar = bslib::sidebar(
    bslib::accordion(
      bslib::accordion_panel(
        'File uploads',
        fileInput('sound', 'Upload a sound file'),
        shinyjs::hidden(fileInput('tg_file', 'Upload a TextGrid file')),
      ),
      bslib::accordion_panel(
        'General appearance',
        numericInput('start', 'Start time (s)', 0, min = 0),
        numericInput('end', 'End time (s; 0 = entire file)', 0, min = 0),
        checkboxInput('tfrom0', 'Should time run from 0 on the x-axis?', TRUE),
        selectInput('tUnit', 'What should be the unit of time on the x-axis?',
                    choices = c('seconds' = 's', 'milliseconds' = 'ms')),
        selectInput('frames', 'Which frames should be plotted?',
                    choices = c('sound', 'spectrogram', 'TextGrid', 'pitch',
                                'formant', 'intensity'),
                    selected = c('sound', 'spectrogram'),
                    multiple = TRUE),
        textInput('proportion',
                  'Relative proportion of frames (comma-delimited)',
                  '0'),
        textInput('mainTitle', 'Title'),
        numericInput('mainTitleAlignment', 'Title alignment',
                     0, min = 0, max = 1),
        checkboxInput('start_end_only',
                      'Should only start and end times be shown on the x-axis?',
                      TRUE),
        checkboxInput('min_max_only',
                      'Should only minima and maxima be shown on the y-axis?',
                      TRUE),
        numericInput('speckleSize', 'Point size of speckled plot components',
                     1, min = 0),
        numericInput('drawSize', 'Line width of drawn plot components',
                     1, min = 0),
        textInput('time_axisLabel', 'x-axis label', 'Time (s)'),
        numericInput('fontSize', 'Font size', 12),
        selectInput('fontFace', 'Font face',
                    choices = c('Default' = 1,
                                'Bold' = 2,
                                'Italic' = 3,
                                'Bold italic' = 4)),
        textInput('fontFamily', 'Font family', 'sans'),
        textInput('globalColor', 'Global color scheme', 'black'),
        textInput('bgColor', 'Background color', 'white'),
        numericInput('figWidth', 'Width (cm)', 15),
        numericInput('figHeight', 'Height (cm)', 10),
        numericInput('figDPI', 'Resolution (DPI)', 300)
      ),
      bslib::accordion_panel('Waveform',
                             shinyjs::hidden(textInput('wave_color',
                                                       'Color',
                                                       'black')),
                             shinyjs::hidden(textInput('wave_channels',
                                                       paste('Which channels should be plotted?',
                                                             '(comma-delimited)'),
                                                       'all')),
                             shinyjs::hidden(checkboxInput('wave_channelNames',
                                                           paste('Should channel names be writted next',
                                                                 'to waveform channels?'),
                                                           FALSE)),
                             shinyjs::hidden(numericInput('wave_axisDigits',
                                                          paste('How many digits should be used for',
                                                                'the waveform y-axis?'),
                                                          3)),
                             shinyjs::hidden(textInput('wave_channelNamesStr',
                                                       'Custom channel names (comma-delimited)',
                                                       NA)),
                             shinyjs::hidden(numericInput('wave_lineWidth',
                                                          'Line width',
                                                          1)),
                             shinyjs::hidden(checkboxInput('pitch_plotOnWave',
                                                           'Pitch overlay',
                                                           FALSE)),
                             shinyjs::hidden(checkboxInput('intensity_plotOnWave',
                                                           'Intensity overlay',
                                                           FALSE))
      ),
      bslib::accordion_panel(
        'Spectrogram',
        shinyjs::hidden(numericInput('spec_freqRangeMin',
                                     'Minimum frequency (Hz)',
                                     0)),
        shinyjs::hidden(numericInput('spec_freqRangeMax',
                                     'Maximum frequency (Hz)',
                                     5000)),
        shinyjs::hidden(numericInput('spec_channel',
                                     'Channel used to derive spectrogram',
                                     NA)),
        shinyjs::hidden(numericInput('spec_windowLength',
                                     'Length of analysis window (s)',
                                     0.005)),
        shinyjs::hidden(numericInput('spec_dynamicRange',
                                     'Dynamic range (dB)',
                                     50)),
        shinyjs::hidden(numericInput('spec_timeStep',
                                     'Number of time steps / frames',
                                     1000)),
        shinyjs::hidden(selectInput('spec_windowShape', 'Window shape',
                                    choices = c('Gaussian', 'Hamming', 'Hanning',
                                                'Bartlett', 'square', 'Blackman'),
                                    selected = 'Gaussian')),
        shinyjs::hidden(textInput('spec_colors', 'Colors (comma-delimited)',
                                  'white,black'),
                        textInput('spec_axisLabel', 'y-axis label',
                                  'Frequency (Hz)')),
        shinyjs::hidden(checkboxInput('pitch_plotOnSpec',
                                      'Pitch overlay',
                                      FALSE)),
        shinyjs::hidden(checkboxInput('formant_plotOnSpec',
                                      'Formant overlay',
                                      FALSE)),
        shinyjs::hidden(checkboxInput('intensity_plotOnSpec',
                                      'Intensity overlay',
                                      FALSE))
      ),
      bslib::accordion_panel(
        'TextGrid',
        shinyjs::hidden(textInput('tg_tiers',
                                  paste('Which tiers should be shown?',
                                        '(comma-delimited)'),
                                  'all')),
        shinyjs::hidden(textInput('tg_focusTier',
                                  paste('Which tier(s) are in focus?',
                                        '(comma-delimited)'),
                                  'all')),
        shinyjs::hidden(textInput('tg_focusTierColor',
                                  paste('Which color(s) should be used for the',
                                        'focus tier(s)? (comma-delimited)'),
                                  'black')),
        shinyjs::hidden(textInput('tg_focusTierLineType',
                                  paste('Which line type(s) should be used for the',
                                        'focus tier(s)? (comma-delimited)'),
                                  'dotted')),
        shinyjs::hidden(checkboxInput('tg_tierNames',
                                      'Should tier names be printed along the x-axis?',
                                      TRUE)),
        shinyjs::hidden(textInput('tg_alignment',
                                  paste('How should text in the tier(s) be aligned?',
                                        '(comma-delimited)'),
                                  'central')),
        shinyjs::hidden(checkboxInput('tg_specialChar',
                                      'Should Praat-style special typesetting be used?',
                                      FALSE)),
        shinyjs::hidden(textInput('tg_color',
                                  paste('Which color(s) should be used for text in the',
                                        'tier(s)?'),
                                  'black'))
      ),
      bslib::accordion_panel(
        'Pitch',
        shinyjs::hidden(selectizeInput('pitch_plotType',
                                       'Plot type',
                                       choices = c('draw', 'speckle'),
                                       selected = 'draw',
                                       multiple = TRUE,
                                       options = list(create = TRUE))),
        shinyjs::hidden(selectInput('pitch_scale',
                                    'Frequency scale',
                                    choices = c('hz', 'logarithmic', 'semitones',
                                                'erb', 'mel'),
                                    selected = 'hz')),
        shinyjs::hidden(numericInput('pitch_semitonesRe',
                                     'Reference frequency for semitone conversion (Hz)',
                                     100)),
        shinyjs::hidden(numericInput('pitch_freqRangeMin',
                                     'Minimum frequency to show (scale-dependent)',
                                     50)),
        shinyjs::hidden(numericInput('pitch_freqRangeMax',
                                     'Maximum frequency to show (scale-dependent)',
                                     500)),
        shinyjs::hidden(numericInput('pitch_timeStep',
                                     'Time step / frame shift (s)',
                                     0.01)),
        shinyjs::hidden(numericInput('pitch_floor',
                                     'Floor frequency (Hz)',
                                     75)),
        shinyjs::hidden(numericInput('pitch_ceiling',
                                     'Ceiling frequency (Hz)',
                                     600)),
        shinyjs::hidden(textInput('pitch_color',
                                  'Color',
                                  'black')),
        shinyjs::hidden(textInput('pitch_axisLabel',
                                  'y-axis label',
                                  'Frequency (Hz)'))
      ),
      bslib::accordion_panel(
        'Formants',
        shinyjs::hidden(selectizeInput('formant_plotType',
                                       'Plot type',
                                       choices = c('draw', 'speckle'),
                                       selected = 'speckle',
                                       multiple = TRUE,
                                       options = list(create = TRUE))),
        shinyjs::hidden(numericInput('formant_freqRangeMin',
                                     'Minimum frequency to show (Hz)',
                                     0)),
        shinyjs::hidden(numericInput('formant_freqRangeMax',
                                     'Maximum frequency to show (Hz)',
                                     5500)),
        shinyjs::hidden(numericInput('formant_dynamicRange',
                                     'Dynamic range (dB)',
                                     30)),
        shinyjs::hidden(numericInput('formant_timeStep',
                                     'Time step / window shift (s)',
                                     0.00625)),
        shinyjs::hidden(numericInput('formant_windowLength',
                                     'Length of analysis window (s)',
                                     0.025)),
        shinyjs::hidden(numericInput('formant_maxN',
                                     'Maximum number of formants to estimate',
                                     5)),
        shinyjs::hidden(numericInput('formant_number',
                                     'Number of formants to plot',
                                     5)),
        shinyjs::hidden(textInput('formant_color',
                                  'Colors (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('formant_axisLabel',
                                  'y-axis label',
                                  'Frequency (Hz)'))
      ),
      bslib::accordion_panel(
        'Intensity',
        shinyjs::hidden(numericInput('intensity_rangeMin',
                                     'Minimum amplitude (dB)',
                                     NA)),
        shinyjs::hidden(numericInput('intensity_rangeMax',
                                     'Maximum amplitude (dB)',
                                     NA)),
        shinyjs::hidden(numericInput('intensity_minPitch',
                                     'Minimum pitch (Hz)',
                                     100)),
        shinyjs::hidden(textInput('intensity_color',
                                  'Color',
                                  'black')),
        shinyjs::hidden(textInput('intensity_axisLabel',
                                  'Axis label',
                                  'Intensity (dB)'))
      ),
      bslib::accordion_panel(
        'Highlighting',
        checkboxInput('highlightBool',
                      'Differential highlighting of parts of signals?',
                      FALSE),
        shinyjs::hidden(selectInput('highlightComp',
                                    'Which signal to highlight',
                                    choices = c('all', 'wave', 'spectrogram',
                                                'pitch', 'formant', 'intensity'),
                                    selected = 'all')),
        shinyjs::hidden(textInput('highlightStart',
                                  'Start time(s) (comma-separated)',
                                  '0')),
        shinyjs::hidden(textInput('highlightEnd',
                                  'End time(s) (comma-separated)',
                                  '0.1')),
        shinyjs::hidden(checkboxInput('highlightTGbool',
                                      'Use TextGrid information?',
                                      FALSE)),
        shinyjs::hidden(textInput('highlightTier',
                                  'TextGrid tier')),
        shinyjs::hidden(textInput('highlightLabel',
                                  'Label(s) to match in TextGrid tier')),
        shinyjs::hidden(textInput('highlightCol',
                                  'Color(s) (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('highlightBG',
                                  'Background color',
                                  'white')),
        shinyjs::hidden(textInput('highlightDrawSize',
                                  'Draw size')),
        shinyjs::hidden(textInput('highlightSpeckleSize',
                                  'Speckle size')),
        checkboxInput('arrowBool', 'Add arrow(s)?'),
        shinyjs::hidden(textInput('arrowComp',
                                  paste('Which plot component(s) should arrow(s) be',
                                        'drawn on? (comma-separated)'),
                                  'sound')),
        shinyjs::hidden(textInput('arrowx0',
                                  paste('Where should the arrow(s) start on the',
                                        'x-axis? (comma-separated)'),
                                  '0')),
        shinyjs::hidden(textInput('arrowx1',
                                  paste('Where should the arrow(s) end on the',
                                        'x-axis? (comma-separated)'),
                                  '0.5')),
        shinyjs::hidden(textInput('arrowy0',
                                  paste('Where should the arrow(s) start on the',
                                        'y-axis? (comma-separated)'),
                                  '0')),
        shinyjs::hidden(textInput('arrowy1',
                                  paste('Where should the arrow(s) end on the',
                                        'y-axis? (comma-separated)'),
                                  '0.1')),
        shinyjs::hidden(textInput('arrowLength',
                                  paste('What should be the length of the arrow',
                                        'head(s) (in inches)?'),
                                  '0.25')),
        shinyjs::hidden(textInput('arrowAngle',
                                  paste('What should be the angle of the arrow',
                                        'head(s) (in degrees)?'),
                                  '30')),
        shinyjs::hidden(textInput('arrowColor',
                                  'Color(s) (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('arrowLineType',
                                  'Line type(s) (comma-separated)',
                                  'solid')),
        shinyjs::hidden(textInput('arrowLineWidth',
                                  'Line width(s) (comma-separated)',
                                  '1')),
        checkboxInput('rectBool', 'Add rectangle(s)?', FALSE),
        shinyjs::hidden(textInput('rectComp',
                                  paste('Which plot component(s) should rectangle(s) be',
                                        'drawn on? (comma-separated)'),
                                  'sound')),
        shinyjs::hidden(textInput('rectxleft',
                                  paste('Where should the left side be on the x-axis?',
                                        '(comma-separated)'),
                                  '0')),
        shinyjs::hidden(textInput('rectybottom',
                                  paste('Where should the lower side be on the y-axis?',
                                        '(comma-separated)'),
                                  '-0.05')),
        shinyjs::hidden(textInput('rectxright',
                                  paste('Where should the right side be on the x-axis?',
                                        'y-axis? (comma-separated)'),
                                  '0.5')),
        shinyjs::hidden(textInput('rectytop',
                                  paste('Where should the top side be on the y-axis',
                                        '(comma-separated)'),
                                  '0.05')),
        shinyjs::hidden(textInput('rectFill',
                                  'Fill color(s) (comma-separated)',
                                  NA)),
        shinyjs::hidden(textInput('rectColor',
                                  'Line color(s) (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('rectLineType',
                                  'Line type(s) (comma-separated)',
                                  'solid')),
        shinyjs::hidden(textInput('rectLineWidth',
                                  'Line width(s) (comma-separated)',
                                  '1')),
        checkboxInput('annotBool', 'Add annotation(s)?', FALSE),
        shinyjs::hidden(textInput('annotComp',
                                  paste('Which plot component(s) should be annotated?',
                                        '(comma-separated)'),
                                  'sound')),
        shinyjs::hidden(textInput('annotLabel',
                                  'Label(s) (semicolon-separated)',
                                  'TEXT')),
        shinyjs::hidden(textInput('annotx',
                                  'Position(s) on the x-axis (comma-separated)',
                                  0)),
        shinyjs::hidden(textInput('annoty',
                                  'Position(s) on the y-axis (comma-separated)',
                                  0.1)),
        shinyjs::hidden(textInput('annotColor',
                                  'Color(s) (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('annotFace',
                                  'Font face(s) (comma-separated)',
                                  '1')),
        shinyjs::hidden(textInput('annotSize',
                                  'Font size(s) (comma-separated)',
                                  '1'))
      )
    )
  ),
  downloadButton('download'),
  imageOutput('plot')
)

options(shiny.maxRequestSize=50*1024^2)

server <- function(input, output, session) {

  curDev <- grDevices::dev.cur()
  onSessionEnded(function() grDevices::dev.set(curDev))

  observeEvent(input$frames, {
    if ('sound' %in% input$frames) {
      shinyjs::show('wave_color')
      shinyjs::show('wave_channels')
      shinyjs::show('wave_channelNames')
      shinyjs::show('wave_lineWidth')
      shinyjs::show('wave_axisDigits')
      shinyjs::show('pitch_plotOnWave')
      shinyjs::show('intensity_plotOnWave')
    } else {
      shinyjs::hide('wave_color')
      shinyjs::hide('wave_channels')
      shinyjs::hide('wave_channelNames')
      shinyjs::hide('wave_lineWidth')
      shinyjs::hide('wave_axisDigits')
      shinyjs::hide('pitch_plotOnWave')
      shinyjs::hide('intensity_plotOnWave')
    }

    if ('TextGrid' %in% input$frames) {
      shinyjs::show('tg_file')
    } else {
      shinyjs::hide('tg_file')
    }

    if ('spectrogram' %in% input$frames) {
      shinyjs::show('spec_freqRangeMin')
      shinyjs::show('spec_freqRangeMax')
      shinyjs::show('spec_windowLength')
      shinyjs::show('spec_dynamicRange')
      shinyjs::show('spec_timeStep')
      shinyjs::show('spec_windowShape')
      shinyjs::show('spec_colors')
      shinyjs::show('spec_axisLabel')
      shinyjs::show('spec_channel')
      shinyjs::show('pitch_plotOnSpec')
      shinyjs::show('formant_plotOnSpec')
      shinyjs::show('intensity_plotOnSpec')
    } else {
      shinyjs::hide('spec_freqRangeMin')
      shinyjs::hide('spec_freqRangeMax')
      shinyjs::hide('spec_windowLength')
      shinyjs::hide('spec_dynamicRange')
      shinyjs::hide('spec_timeStep')
      shinyjs::hide('spec_windowShape')
      shinyjs::hide('spec_colors')
      shinyjs::hide('spec_axisLabel')
      shinyjs::hide('spec_channel')
      shinyjs::hide('pitch_plotOnSpec')
      shinyjs::hide('formant_plotOnSpec')
      shinyjs::hide('intensity_plotOnSpec')
    }

    if ('TextGrid' %in% input$frames) {
      shinyjs::show('tg_tiers')
      shinyjs::show('tg_focusTier')
      shinyjs::show('tg_focusTierColor')
      shinyjs::show('tg_focusTierLineType')
      shinyjs::show('tg_tierNames')
      shinyjs::show('tg_alignment')
      shinyjs::show('tg_specialChar')
      shinyjs::show('tg_color')
    } else {
      shinyjs::hide('tg_tiers')
      shinyjs::hide('tg_focusTier')
      shinyjs::hide('tg_focusTierColor')
      shinyjs::hide('tg_focusTierLineType')
      shinyjs::hide('tg_tierNames')
      shinyjs::hide('tg_alignment')
      shinyjs::hide('tg_specialChar')
      shinyjs::hide('tg_color')
    }

    if ('pitch' %in% input$frames) {
      shinyjs::show('pitch_plotType')
      shinyjs::show('pitch_scale')
      shinyjs::show('pitch_freqRangeMin')
      shinyjs::show('pitch_freqRangeMax')
      shinyjs::show('pitch_timeStep')
      shinyjs::show('pitch_floor')
      shinyjs::show('pitch_ceiling')
      shinyjs::show('pitch_color')
      shinyjs::show('pitch_axisLabel')
    } else {
      shinyjs::hide('pitch_plotType')
      shinyjs::hide('pitch_scale')
      shinyjs::hide('pitch_freqRangeMin')
      shinyjs::hide('pitch_freqRangeMax')
      shinyjs::hide('pitch_timeStep')
      shinyjs::hide('pitch_floor')
      shinyjs::hide('pitch_ceiling')
      shinyjs::hide('pitch_color')
      shinyjs::hide('pitch_axisLabel')
    }

    if ('formant' %in% input$frames) {
      shinyjs::show('formant_plotType')
      shinyjs::show('formant_freqRangeMin')
      shinyjs::show('formant_freqRangeMax')
      shinyjs::show('formant_dynamicRange')
      shinyjs::show('formant_timeStep')
      shinyjs::show('formant_windowLength')
      shinyjs::show('formant_maxN')
      shinyjs::show('formant_color')
      shinyjs::show('formant_axisLabel')
      shinyjs::show('formant_number')
    } else {
      shinyjs::hide('formant_plotType')
      shinyjs::hide('formant_freqRangeMin')
      shinyjs::hide('formant_freqRangeMax')
      shinyjs::hide('formant_dynamicRange')
      shinyjs::hide('formant_timeStep')
      shinyjs::hide('formant_windowLength')
      shinyjs::hide('formant_maxN')
      shinyjs::hide('formant_color')
      shinyjs::hide('formant_axisLabel')
      shinyjs::hide('formant_number')
    }

    if ('intensity' %in% input$frames) {
      shinyjs::show('intensity_rangeMin')
      shinyjs::show('intensity_rangeMax')
      shinyjs::show('intensity_minPitch')
      shinyjs::show('intensity_color')
      shinyjs::show('intensity_axisLabel')
    } else {
      shinyjs::hide('intensity_rangeMin')
      shinyjs::hide('intensity_rangeMax')
      shinyjs::hide('intensity_minPitch')
      shinyjs::hide('intensity_color')
      shinyjs::hide('intensity_axisLabel')
    }

    updateTextInput(inputId = 'proportion',
                    value = paste(rep(50, length(input$frames)),
                                  collapse=','))
  })

  observeEvent(input$pitch_plotOnSpec, {
    if (input$pitch_plotOnSpec) {
      shinyjs::show('pitch_plotType')
      shinyjs::show('pitch_scale')
      shinyjs::show('pitch_freqRangeMin')
      shinyjs::show('pitch_freqRangeMax')
      shinyjs::show('pitch_timeStep')
      shinyjs::show('pitch_floor')
      shinyjs::show('pitch_ceiling')
      shinyjs::show('pitch_color')
      shinyjs::show('pitch_axisLabel')
    } else {
      shinyjs::hide('pitch_plotType')
      shinyjs::hide('pitch_scale')
      shinyjs::hide('pitch_freqRangeMin')
      shinyjs::hide('pitch_freqRangeMax')
      shinyjs::hide('pitch_timeStep')
      shinyjs::hide('pitch_floor')
      shinyjs::hide('pitch_ceiling')
      shinyjs::hide('pitch_color')
      shinyjs::hide('pitch_axisLabel')
    }
  })

  observeEvent(input$pitch_plotOnWave, {
    if (input$pitch_plotOnWave) {
      shinyjs::show('pitch_plotType')
      shinyjs::show('pitch_scale')
      shinyjs::show('pitch_freqRangeMin')
      shinyjs::show('pitch_freqRangeMax')
      shinyjs::show('pitch_timeStep')
      shinyjs::show('pitch_floor')
      shinyjs::show('pitch_ceiling')
      shinyjs::show('pitch_color')
      shinyjs::show('pitch_axisLabel')
    } else {
      shinyjs::hide('pitch_plotType')
      shinyjs::hide('pitch_scale')
      shinyjs::hide('pitch_freqRangeMin')
      shinyjs::hide('pitch_freqRangeMax')
      shinyjs::hide('pitch_timeStep')
      shinyjs::hide('pitch_floor')
      shinyjs::hide('pitch_ceiling')
      shinyjs::hide('pitch_color')
      shinyjs::hide('pitch_axisLabel')
    }
  })


  observeEvent(input$formant_plotOnSpec, {
    if (input$formant_plotOnSpec) {
      shinyjs::show('formant_plotType')
      shinyjs::show('formant_freqRangeMin')
      shinyjs::show('formant_freqRangeMax')
      shinyjs::show('formant_dynamicRange')
      shinyjs::show('formant_timeStep')
      shinyjs::show('formant_windowLength')
      shinyjs::show('formant_maxN')
      shinyjs::show('formant_color')
      shinyjs::show('formant_axisLabel')
      shinyjs::show('formant_number')
    } else {
      shinyjs::hide('formant_plotType')
      shinyjs::hide('formant_freqRangeMin')
      shinyjs::hide('formant_freqRangeMax')
      shinyjs::hide('formant_dynamicRange')
      shinyjs::hide('formant_timeStep')
      shinyjs::hide('formant_windowLength')
      shinyjs::hide('formant_maxN')
      shinyjs::hide('formant_color')
      shinyjs::hide('formant_axisLabel')
      shinyjs::hide('formant_number')
    }
  })

  observeEvent(input$intensity_plotOnSpec, {
    if (input$intensity_plotOnSpec) {
      shinyjs::show('intensity_rangeMin')
      shinyjs::show('intensity_rangeMax')
      shinyjs::show('intensity_minPitch')
      shinyjs::show('intensity_color')
      shinyjs::show('intensity_axisLabel')
    } else {
      shinyjs::hide('intensity_rangeMin')
      shinyjs::hide('intensity_rangeMax')
      shinyjs::hide('intensity_minPitch')
      shinyjs::hide('intensity_color')
      shinyjs::hide('intensity_axisLabel')
    }
  })

  observeEvent(input$intensity_plotOnWave, {
    if (input$intensity_plotOnWave) {
      shinyjs::show('intensity_rangeMin')
      shinyjs::show('intensity_rangeMax')
      shinyjs::show('intensity_minPitch')
      shinyjs::show('intensity_color')
      shinyjs::show('intensity_axisLabel')
    } else {
      shinyjs::hide('intensity_rangeMin')
      shinyjs::hide('intensity_rangeMax')
      shinyjs::hide('intensity_minPitch')
      shinyjs::hide('intensity_color')
      shinyjs::hide('intensity_axisLabel')
    }
  })

  wave_channels <- reactive({
    if (input$wave_channels != 'all') {
      as.numeric(unlist(strsplit(input$wave_channels, ',')))
    } else {
      'all'
    }
  })

  tg_tiers <- reactive({
    if (input$tg_tiers == 'all') {
      'all'
    } else if (!grepl(paste(0:9, collapse='|'), input$tg_tiers)) {
      unlist(strsplit(input$tg_tiers, ','))
    } else {
      as.numeric(unlist(strsplit(input$tg_tiers, ',')))
    }
  })

  tg_focusTier <- reactive({
    if (input$tg_focusTier == 'all') {
      'all'
    } else if (input$tg_focusTier == 'none') {
      'none'
    } else if (!grepl(paste(0:9, collapse='|'), input$tg_focusTier)) {
      unlist(strsplit(input$tg_focusTier, ','))
    } else {
      as.numeric(unlist(strsplit(input$tg_focusTier, ',')))
    }
  })

  spec_channel <- reactive({
    if (is.na(input$spec_channel)) {
      NULL
    } else {
      input$spec_channel
    }
  })

  observeEvent(input$wave_channelNames, {
    if (input$wave_channelNames) {
      shinyjs::show('wave_channelNamesStr')
    } else {
      shinyjs::hide('wave_ChannelNamesStr')
    }
  })

  channelNames <- reactive({
    if (input$wave_channelNames) {
      if (!is.na(input$wave_channelNamesStr)) {
        unlist(strsplit(input$wave_channelNamesStr, ','))
      } else {
        TRUE
      }
    } else {
      FALSE
    }
  })

  observeEvent(input$tUnit, {
    if (input$tUnit == 'ms') {
      updateTextInput(inputId = 'time_axisLabel',
                      value = 'Time (ms)')
    } else if (input$tUnit == 's') {
      updateTextInput(inputId = 'time_axisLabel',
                      value = 'Time (s)')
    }
  })

  observeEvent(input$pitch_scale, {
    if (input$pitch_scale == 'semitones') {
      shinyjs::show('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = -12)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 30)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (semitones')
    } else if (input$pitch_scale == 'erb') {
      shinyjs::hide('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = 0)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 10)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (ERB)')
    } else if (input$pitch_scale == 'mel') {
      shinyjs::hide('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = 50)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 500)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (mel)')
    } else if (input$pitch_scale == 'logarithmic') {
      shinyjs::hide('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = 50)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 500)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (log Hz)')
    } else if (input$pitch_scale == 'hz') {
      shinyjs::hide('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = 50)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 500)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (Hz)')
    }
  })

  observeEvent(input$pitch_floor, {
    updateNumericInput(inputId = 'pitch_timeStep',
                       value = 0.75 / input$pitch_floor)
  })

  observeEvent(input$formant_windowLength, {
    updateNumericInput(inputId = 'formant_timeStep',
                       value = 0.25 * input$formant_windowLength)
  })

  observeEvent(input$formant_maxN, {
    updateNumericInput(inputId = 'formant_number',
                       value = input$formant_maxN,
                       max = input$formant_maxN)
  })

  intensity_range <- reactive({
    if (is.na(input$intensity_rangeMin) & is.na(input$intensity_rangeMax)) {
      NULL
    } else {
      c(as.numeric(input$intensity_rangeMin),
        as.numeric(input$intensity_rangeMax))
    }
  })

  observeEvent(input$globalColor, {
    updateTextInput(inputId = 'wave_color', value = input$globalColor)
    updateTextInput(inputId = 'pitch_color', value = input$globalColor)
    updateTextInput(inputId = 'formant_color', value = input$globalColor)
    updateTextInput(inputId = 'intensity_color', value = input$globalColor)
    updateTextInput(inputId = 'tg_color', value = input$globalColor)
    updateTextInput(inputId = 'tg_focusTierColor', value = input$globalColor)
    updateTextInput(inputId = 'spec_colors',
                    value = paste(input$bgColor, input$globalColor, sep=','))
  })

  observeEvent(input$bgColor, {
    updateTextInput(inputId = 'spec_colors',
                    value = paste(input$bgColor, input$globalColor, sep=','))
  })

  observeEvent(input$highlightBool, {
    if(input$highlightBool) {
      shinyjs::show('highlightComp')
      shinyjs::show('highlightStart')
      shinyjs::show('highlightEnd')
      shinyjs::show('highlightCol')
      shinyjs::show('highlightBG')
      shinyjs::show('highlightTGbool')
      shinyjs::show('highlightDrawSize')
      shinyjs::show('highlightSpeckleSize')
    } else {
      shinyjs::hide('highlightComp')
      shinyjs::hide('highlightStart')
      shinyjs::hide('highlightEnd')
      shinyjs::hide('highlightCol')
      shinyjs::hide('highlightBG')
      shinyjs::hide('highlightTGbool')
      shinyjs::hide('highlightDrawSize')
      shinyjs::hide('highlightSpeckleSize')
    }
  })

  observeEvent(input$highlightTGbool, {
    if (input$highlightTGbool) {
      shinyjs::show('highlightTier')
      shinyjs::show('highlightLabel')
    } else {
      shinyjs::hide('highlightTier')
      shinyjs::hide('highlightLabel')
    }
  })

  observeEvent(input$arrowBool, {
    if(input$arrowBool) {
      shinyjs::show('arrowComp')
      shinyjs::show('arrowx0')
      shinyjs::show('arrowx1')
      shinyjs::show('arrowy0')
      shinyjs::show('arrowy1')
      shinyjs::show('arrowLength')
      shinyjs::show('arrowAngle')
      shinyjs::show('arrowColor')
      shinyjs::show('arrowLineType')
      shinyjs::show('arrowLineWidth')
    } else {
      shinyjs::hide('arrowComp')
      shinyjs::hide('arrowx0')
      shinyjs::hide('arrowx1')
      shinyjs::hide('arrowy0')
      shinyjs::hide('arrowy1')
      shinyjs::hide('arrowLength')
      shinyjs::hide('arrowAngle')
      shinyjs::hide('arrowColor')
      shinyjs::hide('arrowLineType')
      shinyjs::hide('arrowLineWidth')
    }
  })

  observeEvent(input$rectBool, {
    if (input$rectBool) {
      shinyjs::show('rectComp')
      shinyjs::show('rectxleft')
      shinyjs::show('rectybottom')
      shinyjs::show('rectxright')
      shinyjs::show('rectytop')
      shinyjs::show('rectFill')
      shinyjs::show('rectColor')
      shinyjs::show('rectLineType')
      shinyjs::show('rectLineWidth')
    } else {
      shinyjs::hide('rectComp')
      shinyjs::hide('rectxleft')
      shinyjs::hide('rectybottom')
      shinyjs::hide('rectxright')
      shinyjs::hide('rectytop')
      shinyjs::hide('rectFill')
      shinyjs::hide('rectColor')
      shinyjs::hide('rectLineType')
      shinyjs::hide('rectLineWidth')
    }
  })

  observeEvent(input$annotBool, {
    if (input$annotBool) {
      shinyjs::show('annotComp')
      shinyjs::show('annotx')
      shinyjs::show('annoty')
      shinyjs::show('annotLabel')
      shinyjs::show('annotColor')
      shinyjs::show('annotFace')
      shinyjs::show('annotSize')
    } else {
      shinyjs::hide('annotComp')
      shinyjs::hide('annotx')
      shinyjs::hide('annoty')
      shinyjs::hide('annotLabel')
      shinyjs::hide('annotColor')
      shinyjs::hide('annotFace')
      shinyjs::hide('annotSize')
    }
  })

  highlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'all') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      tmp$drawSize <- as.numeric(
        unlist(strsplit(input$highlightDrawSize, ',')))
      tmp$speckleSize <- as.numeric(
        unlist(strsplit(input$highlightSpeckleSize, ',')))
      return(tmp)
    }
  })

  waveHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'wave') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      return(tmp)
    }
  })

  specHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'spectrogram') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$colors <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      return(tmp)
    }
  })

  pitchHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'pitch') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      tmp$drawSize <- as.numeric(
        unlist(strsplit(input$highlightDrawSize, ',')))
      tmp$speckleSize <- as.numeric(
        unlist(strsplit(input$highlightSpeckleSize, ',')))
      return(tmp)
    }
  })

  formantHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'formant') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      tmp$drawSize <- as.numeric(
        unlist(strsplit(input$highlightDrawSize, ',')))
      tmp$speckleSize <- as.numeric(
        unlist(strsplit(input$highlightSpeckleSize, ',')))
      return(tmp)
    }
  })

  intensityHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'intensity') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      tmp$drawSize <- as.numeric(
        unlist(strsplit(input$highlightDrawSize, ',')))
      return(tmp)
    }
  })

  arrow_args <- reactive({
    if (!input$arrowBool) {
      return(NULL)
    } else {
      n_args <- length(unlist(strsplit(input$arrowComp, ',')))
      tmp <- list()
      arrowLength <- as.numeric(unlist(strsplit(input$arrowLength, ',')))
      if (length(arrowLength) != n_args) arrowLength <-
        rep(arrowLength[1], n_args)
      arrowAngle <- as.numeric(unlist(strsplit(input$arrowAngle, ',')))
      if (length(arrowAngle) != n_args) arrowAngle <-
        rep(arrowAngle[1], n_args)
      arrowColor <- unlist(strsplit(input$arrowColor, ','))
      if (length(arrowColor) != n_args) arrowColor <-
        rep(arrowColor[1], n_args)
      arrowLineType <- unlist(strsplit(input$arrowLineType, ','))
      if (length(arrowLineType) != n_args) arrowLineType <-
        rep(arrowLineType[1], n_args)
      arrowLineWidth <- as.numeric(unlist(strsplit(input$arrowLineWidth, ',')))
      if (length(arrowLineWidth) != n_args) arrowLineWidth <-
        rep(arrowLineWidth[1], n_args)
      for (i in 1:n_args) {
        tmp[[i]] <- c(
          unlist(strsplit(input$arrowComp, ','))[i],
          x0 = as.numeric(unlist(strsplit(input$arrowx0, ',')))[i],
          x1 = as.numeric(unlist(strsplit(input$arrowx1, ',')))[i],
          y0 = as.numeric(unlist(strsplit(input$arrowy0, ',')))[i],
          y1 = as.numeric(unlist(strsplit(input$arrowy1, ',')))[i],
          length = arrowLength[i],
          angle = arrowAngle[i],
          col = arrowColor[i],
          lty = arrowLineType[i],
          lwd = arrowLineWidth[i]
        )
      }
      return(tmp)
    }
  })

  rect_args <- reactive({
    if (!input$rectBool) {
      return(NULL)
    } else {
      n_args <- length(unlist(strsplit(input$rectComp, ',')))
      tmp <- list()
      rectLength <- as.numeric(unlist(strsplit(input$arrowLength, ',')))
      rectFill <- unlist(strsplit(input$rectFill, ','))
      if (length(rectFill) != n_args) rectFill <-
        rep(rectFill[1], n_args)
      rectColor <- unlist(strsplit(input$rectColor, ','))
      if (length(rectColor) != n_args) rectColor <-
        rep(rectColor[1], n_args)
      rectLineType <- unlist(strsplit(input$rectLineType, ','))
      if (length(rectLineType) != n_args) rectLineType <-
        rep(rectLineType[1], n_args)
      rectLineWidth <- as.numeric(unlist(strsplit(input$rectLineWidth, ',')))
      if (length(rectLineWidth) != n_args) rectLineWidth <-
        rep(rectLineWidth[1], n_args)
      for (i in 1:n_args) {
        tmp[[i]] <- c(
          unlist(strsplit(input$rectComp, ','))[i],
          xleft = as.numeric(unlist(strsplit(input$rectxleft, ',')))[i],
          ybottom = as.numeric(unlist(strsplit(input$rectybottom, ',')))[i],
          xright = as.numeric(unlist(strsplit(input$rectxright, ',')))[i],
          ytop = as.numeric(unlist(strsplit(input$rectytop, ',')))[i],
          col = rectFill[i],
          border = rectColor[i],
          lwd = rectLineWidth[i],
          lty = rectLineType[i]
        )
      }
      return(tmp)
    }
  })

  annot_args <- reactive({
    if (!input$annotBool) {
      return(NULL)
    } else {
      n_args <- length(unlist(strsplit(input$annotComp, ',')))
      tmp <- list()
      annotLabel <- unlist(strsplit(input$annotLabel, ';'))
      annotColor <- unlist(strsplit(input$annotColor, ','))
      if (length(annotColor) != n_args) annotColor <-
        rep(annotColor, n_args)
      annotFace <- as.numeric(unlist(strsplit(input$annotFace, ',')))
      if (length(annotFace) != n_args) annotFace <-
        rep(annotFace, n_args)
      annotSize <- as.numeric(unlist(strsplit(input$annotSize, ',')))
      if (length(annotSize) != n_args) annotSize <-
        rep(annotSize, n_args)
      for (i in 1:n_args) {
        tmp[[i]] <- c(
          unlist(strsplit(input$annotComp, ','))[i],
          x = as.numeric(unlist(strsplit(input$annotx, ',')))[i],
          y = as.numeric(unlist(strsplit(input$annoty, ',')))[i],
          label = annotLabel[i],
          col = annotColor[i],
          font = annotFace[i],
          cex = annotSize[i]
        )
      }
      return(tmp)
    }
  })

  make_fig <- function(outputFile) {
    req(input$sound)
    if ('TextGrid' %in% input$frames) req(input$tg_file)
    grDevices::png(outputFile,
                   width=as.numeric(input$figWidth),
                   height=as.numeric(input$figHeight),
                   res=as.numeric(input$figDPI),
                   pointsize=input$fontSize, units='cm')
    praatpicture::praatpicture(input$sound$datapath,
                               start = as.numeric(input$start),
                               end = as.numeric(input$end),
                               tfrom0 = input$tfrom0,
                               tUnit = input$tUnit,
                               frames = input$frames,
                               proportion = as.numeric(unlist(strsplit(
                                 input$proportion, ','
                               ))),
                               mainTitle = input$mainTitle,
                               mainTitleAlignment = input$mainTitleAlignment,
                               start_end_only = input$start_end_only,
                               min_max_only = input$min_max_only,
                               time_axisLabel = input$time_axisLabel,
                               speckleSize = input$speckleSize,
                               drawSize = input$drawSize,
                               wave_color = input$wave_color,
                               wave_channels = wave_channels(),
                               wave_channelNames = channelNames(),
                               wave_axisDigits = input$wave_axisDigits,
                               wave_lineWidth = input$wave_lineWidth,
                               wave_highlight = waveHighlight_args(),
                               tg_file = input$tg_file$datapath,
                               tg_tiers = tg_tiers(),
                               tg_focusTier = tg_focusTier(),
                               tg_focusTierColor = unlist(strsplit(
                                 input$tg_focusTierColor, ',')),
                               tg_focusTierLineType = unlist(strsplit(
                                 input$tg_focusTierLineType, ',')),
                               tg_tierNames = input$tg_tierNames,
                               tg_alignment = unlist(strsplit(
                                 input$tg_alignment, ',')),
                               tg_specialChar = input$tg_specialChar,
                               tg_color = unlist(strsplit(
                                 input$tg_color, ',')),
                               spec_freqRange = c(
                                 as.numeric(input$spec_freqRangeMin),
                                 as.numeric(input$spec_freqRangeMax)),
                               spec_windowLength =
                                 as.numeric(input$spec_windowLength),
                               spec_dynamicRange =
                                 as.numeric(input$spec_dynamicRange),
                               spec_timeStep =
                                 as.numeric(input$spec_timeStep),
                               spec_windowShape = input$spec_windowShape,
                               spec_colors = unlist(strsplit(
                                 input$spec_colors, ',')),
                               spec_axisLabel = input$spec_axisLabel,
                               spec_channel = spec_channel(),
                               spec_highlight = specHighlight_args(),
                               pitch_plotType = input$pitch_plotType,
                               pitch_scale = input$pitch_scale,
                               pitch_semitonesRe = input$pitch_semitonesRe,
                               pitch_freqRange = c(
                                 as.numeric(input$pitch_freqRangeMin),
                                 as.numeric(input$pitch_freqRangeMax)),
                               pitch_timeStep = as.numeric(input$pitch_timeStep),
                               pitch_floor = as.numeric(input$pitch_floor),
                               pitch_ceiling = as.numeric(input$pitch_ceiling),
                               pitch_color = unlist(strsplit(
                                 input$pitch_color, ',')),
                               pitch_axisLabel = input$pitch_axisLabel,
                               pitch_plotOnSpec = input$pitch_plotOnSpec,
                               pitch_plotOnWave = input$pitch_plotOnWave,
                               pitch_highlight = pitchHighlight_args(),
                               formant_plotType = input$formant_plotType,
                               formant_freqRange = c(
                                 as.numeric(input$formant_freqRangeMin),
                                 as.numeric(input$formant_freqRangeMax)),
                               formant_dynamicRange = as.numeric(
                                 input$formant_dynamicRange),
                               formant_windowLength = as.numeric(
                                 input$formant_windowLength),
                               formant_timeStep = as.numeric(
                                 input$formant_timeStep),
                               formant_maxN = input$formant_maxN,
                               formant_color = unlist(strsplit(
                                 input$formant_color, ',')),
                               formant_number = input$formant_number,
                               formant_axisLabel = input$formant_axisLabel,
                               formant_plotOnSpec = input$formant_plotOnSpec,
                               formant_highlight = formantHighlight_args(),
                               intensity_range = intensity_range(),
                               intensity_minPitch = as.numeric(
                                 input$intensity_minPitch),
                               intensity_color = unlist(strsplit(
                                 input$intensity_color, ',')),
                               intensity_axisLabel = input$intensity_axisLabel,
                               intensity_plotOnSpec = input$intensity_plotOnSpec,
                               intensity_plotOnWave = input$intensity_plotOnWave,
                               intensity_highlight = intensityHighlight_args(),
                               font = as.numeric(input$fontFace),
                               font.axis = as.numeric(input$fontFace),
                               family = input$fontFamily,
                               col = input$globalColor,
                               col.axis = input$globalColor,
                               bg = input$bgColor,
                               highlight = highlight_args(),
                               draw_arrow = arrow_args(),
                               draw_rectangle = rect_args(),
                               annotate = annot_args()
    )
    dev.off()
    list(src = outputFile,
         width = as.numeric(input$figWidth) * 40,
         height = as.numeric(input$figHeight) * 40)
  }

  output$plot <- renderImage({make_fig(tempfile(fileext='.png'))}, deleteFile = TRUE)
  output$download <- downloadHandler(
    filename = function() {'output.png'},
    content = function(file) {make_fig(file)}
  )

}

#' Interactively create a TextGrid object
#'
#' Annotate a sound file by interacting with waveform or spectrogram plots,
#' resulting in a TextGrid object which can be used for creating various
#' acoustic plots with time-aligned annotations with [praatpicture()].
#'
#' @param sound String giving the file name of a sound file with the .wav
#' extension.
#' @param tierNames String or vector of strings giving the name(s) of tiers in
#' the new TextGrid object.
#' @param start Start time (in seconds) of desired plotted area. Default is `0`.
#' @param end End time (in seconds) of desired plotted area. Default is `0`
#' (= the entire file).
#' @param show String giving the type of plot to show. Default is `wave`,
#' another option is `spectrogram`. Note that spectrogram plotting is relatively
#' slow within this function.
#' @param channel Number indicating which audio channel to show. Default is `1`.
#' @param sampa2ipa Logical; should SAMPA transcriptions be converted to IPA?
#' Default is `FALSE`.
#' @param audioInViewer Logical; should audio be playable from the Viewer pane
#' in RStudio?
#'
#' @details Running this function will show either a waveform or a spectrogram
#' in a separate `X11` graphics device window. Click on this figure in the
#' locations where you want to add boundaries to your TextGrid objects. This
#' should be done sequentially, starting with the first boundary along the
#' time axis and ending with the last. It does not matter where on the y-axis
#' you click.
#'
#' Once you have indicated all the desired boundaries, you will be prompted
#' in the R console to say whether the tier is an interval tier or a point tier
#' by typing `y` (for interval tier) or `n` (for point tier). Subsequently
#' you will be prompted in the console to write labels corresponding to each
#' interval or point.
#'
#' If you are creating a TextGrid with multiple tiers (i.e., if `tierNames`
#' is longer than 1), this process will be repeated for all tiers.
#'
#' @return A list object identical to those created by [rPraat::tg.read()] when
#' loading TextGrid objects into R. This object can be passed to the `tg_obj`
#' argument when using `praatpicture`.
#' @seealso `make_TextGrid()` is largely a wrapper around the function
#' [tg_createTier()] which does most of the work.
#' @export
#'
#' @examples
#' \dontrun{
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/2.wav')
#' tg <- make_TextGrid(soundFile, tierNames=c('Mary', 'John', 'Bell'))
#' # Follow the steps shown in the console
#'
#' praatpicture(soundFile, tg_obj=tg)
#' }
make_TextGrid <- function(sound, tierNames, start=0, end=0, audioInViewer=TRUE,
                          show='wave', channel=1, sampa2ipa=FALSE) {

  if (audioInViewer) {
    if (end == 0) end <- Inf
    snd <- tuneR::readWave(sound, from=start, to=end, units='seconds')
    tempDir <- tempfile()
    dir.create(tempDir)
    htmlFile <- file.path(tempDir, 'index.html')
    wavFile <- file.path(tempDir, 'tmp.wav')
    tuneR::writeWave(snd, filename=wavFile, extensible=FALSE)
    writeLines(
      text='<audio controls>\n<source src="tmp.wav" type="audio/wav">\n</audio>',
      con=htmlFile)
    rstudioapi::viewer(htmlFile)
  }

  tg <- list()
  for (t in tierNames) {
    tg[[t]] <- tg_createTier(sound, t, start, end, show, channel, sampa2ipa)
  }
  return(tg)
}

#' Overlay pitch on plot frame
#'
#' Function for overlaying pitch contour on another plot frame, viz. the
#' waveform or spectrogram. Instead of using this function directly, just use
#' `praatpicture('my_sound_file')` with `pitch_plotOnSpec` or `pitch_plotOnWave`
#' set to `TRUE`.
#'
#' @param pt PitchTier object loaded using [rPraat::pt.read] or other object
#' formatted in a similar way, i.e. a `list` object containing the elements
#' `t` (a vector of time values) and `f` (a vector of frequency values) of
#' identical length.
#' @param bottomRange Bottom y-axis range of the plot frame that pitch
#' is plotted on.
#' @param topRange Top y-axis range of the plot frame that pitch is plotted on.
#' @param start Start time (in seconds) of desired plotted area.
#' @param org_start Start time (in seconds) of desired plotted area in the
#' original sound file.
#' @param tfrom0 Logical; should time on the x-axis run from 0 or from the
#' original time? Default is `TRUE`.
#' @param freqRange Vector of two integers giving the frequency range to be
#' used for producing pitch plots. Default is `NULL`, in which case the pitch
#' range is automatically reset to `c(-12,30)` for the `semitones` scale,
#' `c(0,10)` for the `erb` scale, and `c(50,500)` for the Hz-based scales,
#' following Praat defaults.
#' @param plotType String giving the type of pitch plot to produce; default
#' is `draw` (a line plot), the only other option is `speckle` (a point plot).
#' Alternatively a vector `c('draw','speckle')` can be passed, in which case
#' both are used.
#' @param scale String giving the frequency scale to use when producing
#' pitch plots. Default is `hz`; other options are `logarithmic` (also in Hz),
#' `semitones`, `erb`, and `mel`.
#' @param color String giving the name of the color to be used for
#' plotting pitch. Default is `'black'`.
#' @param ind Integer indexing current plot frame relative to other plot
#' components. Default is `NULL`.
#' @param drawSize Number indicating the line width if
#' `plotType` is `'draw'`. Default is `1`. Controls the `lwd` argument of
#' [graphics::lines].
#' @param speckleSize Number indicating the point size of if `_plotType` is
#' `'speckle'`. Default is `1`. Controls the `cex` arguments of
#' [graphics::points].
#' @param axisLabel String giving the name of the label to print along the
#' y-axis when printing a pitch track. Default is `NULL`, in which case the
#' axis label will depend on the scale.
#' @param min_max_only Logical; should only minimum and maximum values be given
#' on the y-axis? Default is `TRUE`. Can also be a logical vector if some but
#' not all plot components should have minimum and maximum values on the y-axis.
#' Ignored for TextGrid component.
#' @param highlight Named list giving parameters for differential
#' highlighting of pitch based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `color`),
#' `drawSize` or `speckleSize` (both numeric), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param intensity_overlay Logical; is intensity also overlaid on the same
#' plot frame? Default is `FALSE`.
#' @param suppressAxes Should all axes and the box around plots be suppressed?
#' Default is `FALSE`. Generally not recommended.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, frames = 'spectrogram', pitch_plotOnSpec = TRUE)
pitch_overlay <- function(pt, bottomRange, topRange, start, org_start=0,
                          tfrom0=TRUE, freqRange=NULL, plotType='draw',
                          scale='hz', color='black', ind=NULL, drawSize=1,
                          speckleSize=1, axisLabel=NULL, min_max_only=TRUE,
                          highlight=NULL, intensity_overlay=FALSE,
                          suppressAxes=FALSE) {

  pfr <- freqRange
  if (tfrom0) pt$t <- pt$t - org_start
  if (scale == 'logarithmic') {
    pt$f <- log(pt$f)
    pfr <- log(freqRange)
  }
  sRan <- topRange - bottomRange
  pRan <- pfr[2] - pfr[1]
  multiplier <- sRan / pRan
  pt$f <- bottomRange + (pt$f - pfr[1]) * multiplier

  create_highlighting <- function(highlight, pt) {
    tSpeckle <- c()
    fSpeckle <- c()
    tDraw <- c()
    fDraw <- c()
    for (int in 1:length(highlight$start)) {
      times <- which(pt$t > highlight$start[int] & pt$t < highlight$end[int])
      tSpeckle <- c(tSpeckle, pt$t[times], max(pt$t[times] + 0.0001))
      fSpeckle <- c(fSpeckle, pt$f[times], NA)
      if (highlight$start[int] < start) {
        extrema <- zoo::na.approx(c(NA, pt$f),
                                  c(highlight$end[int], pt$t))
        tDraw <- c(tDraw, pt$t[times], highlight$end[int],
                   max(pt$t[times] + 0.0001))
        fDraw <- c(fDraw, pt$f[times], extrema[1], NA)
      } else {
        extrema <- zoo::na.approx(c(NA, NA, pt$f),
                                  c(highlight$start[int], highlight$end[int],
                                    pt$t))[1:2]
        tDraw <- c(tDraw, highlight$start[int], pt$t[times], highlight$end[int],
                   max(pt$t[times] + 0.0001))
        fDraw <- c(fDraw, extrema[1], pt$f[times], extrema[2], NA)
      }
      if (any(highlight$start < start)) {
        highlight$start[which(highlight$start < start)] <- start
      }
    }
    if (!'color' %in% names(highlight)) highlight$color <- color
    if (!'drawSize' %in% names(highlight)) highlight$drawSize <- drawSize
    if (!'speckleSize' %in% names(highlight)) highlight$speckleSize <-
      speckleSize

    return(list(params = highlight, tSpeckle = tSpeckle, fSpeckle = fSpeckle,
                tDraw = tDraw, fDraw = fDraw))
  }


  if ('draw' %in% plotType) {
    if (length(color) == 2) graphics::lines(pt$t, pt$f,
                                            lwd=drawSize+2, col=color[2])
    graphics::lines(pt$t, pt$f, lwd=drawSize, col=color[1])
    if (!is.null(highlight)) {
      if (any(sapply(highlight, is.list))) {
        highlightLoopDepth <- sum(sapply(highlight, is.list))
      } else {
        highlightLoopDepth <- 1
      }
      for (i in 1:highlightLoopDepth) {
        if (any(sapply(highlight, is.list))) {
          thisHL <- create_highlighting(highlight[[i]], pt)
        } else {
          thisHL <- create_highlighting(highlight, pt)
        }
        if (length(thisHL$params$color) == 2) {
          graphics::lines(thisHL$tDraw, thisHL$fDraw,
                          lwd=thisHL$params$drawSize+2,
                          col=thisHL$params$color[2])
        }
        graphics::lines(thisHL$tDraw, thisHL$fDraw,
                        col=thisHL$params$color[1],
                        lwd=thisHL$params$drawSize)
      }
    }
  }
  if ('speckle' %in% plotType) {
    if (length(color) == 2) graphics::points(pt$t, pt$f, lwd=3,
                                             pch=20, col=color[2],
                                             cex=speckleSize)
    graphics::points(pt$t, pt$f, pch=20, col=color[1],
                     cex=speckleSize)
    if (!is.null(highlight)) {
      if (any(sapply(highlight, is.list))) {
        highlightLoopDepth <- sum(sapply(highlight, is.list))
      } else {
        highlightLoopDepth <- 1
      }
      for (i in 1:highlightLoopDepth) {
        if (any(sapply(highlight, is.list))) {
          thisHL <- create_highlighting(highlight[[i]], pt)
        } else {
          thisHL <- create_highlighting(highlight, pt)
        }
        if (length(highlight$color == 2)) {
          graphics::points(thisHL$tSpeckle, thisHL$fSpeckle,
                           lwd=3, pch=20,
                           col=thisHL$params$color[2],
                           cex=thisHL$params$speckleSize)
        }
        graphics::points(thisHL$tSpeckle, thisHL$fSpeckle, pch=20,
                         col=thisHL$params$color[1],
                         cex=thisHL$params$speckleSize)
      }
    }
  }

  pline <- c(3.5,1)
  if (intensity_overlay) pline <- c(1,1)

  if (!suppressAxes) {
    graphics::mtext(axisLabel, side=4, line=pline[1], cex=0.8, las=3,
                    col=color[1])

    if (min_max_only[ind]) {
      graphics::mtext(freqRange[1], side=4, line=pline[2],
                      at=bottomRange, padj=0, las=2, cex=0.7,
                      col=color[1])
      graphics::mtext(freqRange[2], side=4, line=pline[2],
                      at=topRange, padj=1, las=2, cex=0.7,
                      col=color[1])
    } else {
      rtix <- pretty(pfr[1]:pfr[2])
      graphics::axis(4, at=(rtix-pfr[1])*multiplier, labels=rtix,
                     col.ticks=color[1], col.axis=color[1])
    }
  }

}

#' Plot waveform
#'
#' Function for plotting waveforms called by [praatpicture]. Instead of using
#' this function directly, just use
#' `praatpicture('my_sound_file', frames='sound')`.
#'
#' @param sig Numeric vector corresponding to a sound signal.
#' @param bit Numeric; will generally be grabbed from a loaded `WaveMC` object.
#' @param t Numeric vector giving times corresponding to the signal.
#' @param start Start time (in seconds) of desired plotted area.
#' @param tfrom0 Logical; should time on the x-axis run from 0 or from the
#' original time? Default is `TRUE`.
#' @param nchan Numeric; how many channels will be plotted? Default is `1`.
#' @param energyRange Numeric vector giving the desired energy
#' range (y-axis range) of waveform(s). Default is `NULL`, in which case the
#' y-axis range is set to the lowest and highest value for each wave.
#' If a vector of two values is passed for a multi-wave plot, these are
#' recycled for each wave. Separate ranges can be set for each waveform by
#' passing a vector of a length corresponding to twice the number of waves
#' plotted.
#' @param color String giving the name of the color to be used for plotting
#' the waveform. Default is `'black'`. Alternatively, a vector of colors, if
#' different channels should be plotted with different colors.
#' @param pitch_plotOnWave Boolean; should pitch be plotted on top of
#' waveform? Default is `FALSE`.
#' @param pt Pitch object loaded using [rPraat::pt.read] or similar object.
#' @param pitch_plotType String giving the type of pitch plot to produce; default
#' is `draw` (a line plot), the only other option is `speckle` (a point plot).
#' Alternatively a vector `c('draw','speckle')` can be passed, in which case
#' both are used.
#' @param pitch_scale String giving the frequency scale to use when producing
#' pitch plots. Default is `hz`; other options are `logarithmic` (also in Hz),
#' `semitones`, `erb`, and `mel`.
#' @param pitch_freqRange Vector of two integers giving the frequency range to be
#' used for producing pitch plots. Default is `NULL`, in which case the pitch
#' range is automatically reset to `c(-12,30)` for the `semitones` scale,
#' `c(0,10)` for the `erb` scale, and `c(50,500)` for the Hz-based scales,
#' following Praat defaults.
#' @param pitch_axisLabel String giving the name of the label to print along the
#' y-axis when printing a pitch track. Default is `NULL`, in which case the
#' axis label will depend on the scale.
#' @param pitch_color String or vector of strings giving the name of the color
#' to be used for plotting pitch. Default is `'black'`. If a vector of two
#' strings is passed, the second color will be used for background highlighting.
#' @param pitch_highlight Named list giving parameters for differential
#' highlighting of pitch based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `pitch_color`),
#' `drawSize` or `speckleSize` (both numeric).
#' @param intensity_plotOnWave Boolean; should intensity be plotted on top of
#' waveform? Default is `FALSE`.
#' @param it Intensity object loaded using [rPraat::it.read] or similar object.
#' @param intensity_range Vector of two integers giving the intensity range to be
#' used for producing intensity plots. Default is `NULL`, in which case the
#' range is simply the minimum and maximum levels in the curve.
#' @param intensity_axisLabel String giving the name of the label to print along
#' the y-axis when plotting intensity. Default is `Intensity (dB)`.
#' @param intensity_color String or vector of strings giving the name of the
#' color to be used for plotting intensity. Default is `'black'`. If a vector of
#' two strings is passed, the second color will be used for background
#' highlighting.
#' @param intensity_highlight Named list giving parameters for differential
#' highlighting of the intensity contour based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `intensity_color`) and
#' `drawSize` (integer).
#' @param tgbool Logical; should dotted lines be plotted corresponding to
#' locations in a TextGrid? Default is `FALSE`.
#' @param lines Numeric vector giving locations in seconds of locations from
#' a TextGrid to be plotted with dotted lines. Default is `NULL`.
#' @param focusTierColor String or vector of strings giving the color(s) to
#' use for plotting focus tier lines. If multiple tiers are focused, a vector
#' of the same length can be passed, and the nth tier will be plotted in the
#' nth color. Default is `'black'`.
#' @param focusTierLineType String or vector of strings giving the line
#' type(s) for plotting focus tier lines. If multiple tiers are focused, a
#' vector of the same length can be passed, and the nth tier will be plotted in
#' the nth line type. Default is `'dotted'`.
#' @param ind Integer indexing waveform relative to other plot components.
#' Default is `NULL`.
#' @param line_comp Vector of strings or numbers giving plot components to draw
#' straight lines on. Default is `NULL`.
#' @param rect_comp Vector of strings or numbers giving plot components to draw
#' rectangles on. Default is `NULL`.
#' @param arr_comp Vector of strings of numbers giving plot components to draw
#' arrows on. Default is `NULL`.
#' @param annot_comp Vector of strings of numbers giving plot components to
#' annotate. Default is `NULL`.
#' @param draw_lines List of arguments for drawing straight lines passed from
#' `praatpicture()`. Default is `NULL`.
#' @param draw_rectangle List of arguments for drawing rectangles passed from
#' `praatpicture()`. Default is `NULL`.
#' @param draw_arrow List of arguments for drawing arrows passed from
#' `praatpicture()`. Default is `NULL`.
#' @param annotate List of arguments for annotating passed from
#' `praatpicture()`. Default is `NULL`.
#' @param channelNames Logical; should names of audio channels be printed on
#' the y-axis? Default is `FALSE`.
#' @param axisDigits Numeric giving the number of digits to print for
#' values along the y-axis of the waveform. Default is `3`. If `0` is passed,
#' the y-axis is suppressed. Alternatively a vector of numbers, if different
#' numbers should be used for different channels. Note that this only
#' applies when `min_max_only = TRUE`, as otherwise the look of the y-axis is
#' determined entirely using `grDevices::axisTicks()`.
#' @param lineWidth Number giving the line width to use for plotting
#' the waveform. Default is `1`. Alternatively a vector of numbers, if
#' different line widths should be used for different channels.
#' @param cn Vector of strings with channel names to be printed on the y-axis
#' if `channelNames` is `TRUE`.
#' @param min_max_only Logical; should only minimum and maximum values be given
#' on the y-axis? Default is `TRUE`. Can also be a logical vector if some but
#' not all plot components should have minimum and maximum values on the y-axis.
#' Ignored for TextGrid component.
#' @param highlight Named list giving parameters for differential
#' highlighting of the waveform based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the argument
#' `color` (string, see `color`), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param drawSize Number indicating the line width of plot components where
#' the `_plotType` is `'draw'` (i.e., pitch, formants, or intensity rendered as
#' line plots). Default is `1`. Controls the `lwd` argument of
#' [graphics::lines].
#' @param speckleSize Number indicating the point size of plot components where
#' the `_plotType` is `'speckle'` (i.e. pitch or formants rendered as point
#' plots). Default is `1`. Controls the `cex` arguments of [graphics::points].
#' @param suppressAxes Should all axes and the box around plots be suppressed?
#' Default is `FALSE`. Generally not recommended.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, frames='sound')
waveplot <- function(sig, bit, t, start, tfrom0=TRUE, nchan=1,
                     energyRange=NULL, color='black',
                     pitch_plotOnWave=FALSE, pt=NULL,
                     pitch_plotType='draw', pitch_scale='hz',
                     pitch_freqRange=NULL, pitch_axisLabel=NULL,
                     pitch_color='black', pitch_highlight=NULL,
                     intensity_plotOnWave=FALSE, it=NULL, intensity_range=NULL,
                     intensity_axisLabel='Intensity (dB)',
                     intensity_color='black', intensity_highlight=NULL,
                     tgbool=FALSE,
                     lines=NULL, focusTierColor='black',
                     focusTierLineType='dotted',
                     ind=NULL, line_comp=NULL,
                     rect_comp=NULL, arr_comp=NULL, annot_comp=NULL,
                     draw_lines=NULL, draw_rectangle=NULL, draw_arrow=NULL,
                     annotate=NULL, channelNames=FALSE, axisDigits=3,
                     lineWidth=1, cn=NULL, min_max_only=TRUE, highlight=NULL,
                     drawSize=1, speckleSize=1, suppressAxes=FALSE) {

  if (suppressAxes) {
    min_max_only <- TRUE
    axisDigits <- 0
  }

  if (length(color) != nchan) color <- rep(color, nchan)
  if (length(lineWidth) != nchan) lineWidth <- rep(lineWidth, nchan)
  if (length(axisDigits) != nchan) axisDigits <- rep(axisDigits, nchan)

  if (tfrom0) {
    org_start <- start
    start <- 0
  }

  for (j in 1:nchan) {

    sig[,j] <- sig[,j] / (2^(bit - 1) - 1)

    if (is.null(energyRange)) {
      currentEnergyRange <- c(min(sig[,j]), max(sig[,j]))
    } else if (length(energyRange) > 2) {
      currentEnergyRange <- c(energyRange[j*2-1], energyRange[j*2])
    } else {
      currentEnergyRange <- energyRange
    }

    if (!min_max_only[ind]) {
      if (j == 1 & ind == 1) {
        yax <- 's'
      } else {
        ytix <- grDevices::axisTicks(currentEnergyRange, log=F)
        ytix <- ytix[-length(ytix)]
        yax <- 'n'
      }
    } else {
      yax <- 'n'
      ytix <- c(currentEnergyRange[1], 0, currentEnergyRange[2])
      if (axisDigits[j] == 0) {
        ytixLabs <- rep('', 3)
      } else {
        ytixLabs <- c(round(currentEnergyRange[1], axisDigits[j]), 0,
                      round(currentEnergyRange[2], axisDigits[j]))
      }
    }

    create_highlighting <- function(highlight, sig, t) {
      hl_t <- c()
      hl_w <- c()
      for (int in 1:length(highlight$start)) {
        times <- which(t > highlight$start[int] & t < highlight$end[int])
        hl_t <- c(hl_t, t[times[-length(times)]+1],
                  max(t[times] + 0.0001))
        hl_w <- c(hl_w, sig[times[-length(times)],j], NA)
      }
      if (any(highlight$start < t[1])) {
        highlight$start[which(highlight$start < t[1])] <- t[1]
      }
      if (any(highlight$end > max(t))) {
        highlight$end[which(highlight$end > max(t))] <- max(t)
      }
      if (!'color' %in% names(highlight)) highlight$color <- color
      return(list(params = highlight, hl_t = hl_t, hl_w = hl_w))
    }

    plot(t[-1], sig[,j], type='l', xlab='', xaxt='n', ylab='', yaxt=yax,
         col=color[j], lwd=lineWidth[j], ylim=currentEnergyRange)

    if (!is.null(highlight)) {
      if (any(sapply(highlight, is.list))) {
        highlightLoopDepth <- sum(sapply(highlight, is.list))
      } else {
        highlightLoopDepth <- 1
      }
      for (i in 1:highlightLoopDepth) {
        if (any(sapply(highlight, is.list))) {
          thisHL <- create_highlighting(highlight[[i]], sig, t)
        } else {
          thisHL <- create_highlighting(highlight, sig, t)
        }
        if ('background' %in% names(thisHL$params)) {
          graphics::rect(thisHL$params$start,
                         -abs(currentEnergyRange[1] * 2),
                         thisHL$params$end,
                         currentEnergyRange[2] * 2,
                         col = thisHL$params$background, border = NA)
        }
        graphics::lines(thisHL$hl_t,
                        thisHL$hl_w,
                        col=thisHL$params$color,
                        lwd=lineWidth[j])
      }
    }

    if (yax == 'n' & !min_max_only[ind]) graphics::axis(2, at=ytix)
    if (min_max_only[ind]) graphics::axis(2, at=ytix, las=2, padj=c(0,0.5,1),
                                          tick=F, labels=ytixLabs)
    if (channelNames) graphics::mtext(cn[j], side=2, las=2, line=3.5, cex=0.8)
    if (tgbool) {
      for (k in 1:length(lines)) {
        graphics::abline(v=lines[[k]], col=focusTierColor[k],
                       lty=focusTierLineType[k])
      }
    }

    if (as.numeric(pitch_plotOnWave) == j) {
      pitch_overlay(pt, currentEnergyRange[1], currentEnergyRange[2], start,
                    org_start, tfrom0, pitch_freqRange, pitch_plotType,
                    pitch_scale, pitch_color, ind, drawSize, speckleSize,
                    pitch_axisLabel, min_max_only, pitch_highlight,
                    intensity_plotOnWave == j, suppressAxes)
    }

    if (as.numeric(intensity_plotOnWave) == j) {
      intensity_overlay(it, currentEnergyRange[1], currentEnergyRange[2], start,
                        org_start, tfrom0, intensity_range, intensity_color,
                        ind, drawSize, intensity_axisLabel, min_max_only,
                        intensity_highlight, pitch_plotOnWave == j,
                        suppressAxes)
    }

    if ('sound' %in% rect_comp) draw_rectangle('sound', draw_rectangle)
    if ('sound' %in% arr_comp) draw_arrow('sound', draw_arrow)
    if ('sound' %in% annot_comp) make_annot('sound', annotate)
    if ('sound' %in% line_comp) draw_lines('sound', draw_lines)

    if (j %in% rect_comp) draw_rectangle(j, draw_rectangle)
    if (j %in% arr_comp) draw_arrow(j, draw_arrow)
    if (j %in% annot_comp) make_annot(j, annotate)
    if (j %in% line_comp) draw_lines(j, draw_lines)

    graphics::box()
  }
}

#' Overlay intensity on plot frame
#'
#' Function for overlaying intensity contour on another plot frame, viz. the
#' waveform or spectrogram. Instead of using this function directly, just use
#' `praatpicture('my_sound_file')` with `intensity_plotOnSpec` or
#' `pitch_plotOnWave` set to `TRUE`.
#'
#' @param it IntensityTier object loaded using [rPraat::it.read] or other object
#' formatted in a similar way, i.e. a `list` object containing the elements
#' `t` (a vector of time values) and `i` (a vector of intensity values) of
#' identical length.
#' @param bottomRange Bottom y-axis range of the plot frame that intensity
#' is plotted on.
#' @param topRange Top y-axis range of the plot frame that intensity is
#' plotted on.
#' @param start Start time (in seconds) of desired plotted area.
#' @param org_start Start time (in seconds) of desired plotted area in the
#' original sound file.
#' @param tfrom0 Logical; should time on the x-axis run from 0 or from the
#' original time? Default is `TRUE`.
#' @param range Vector of two integers giving the intensity range to be
#' used for producing intensity plots. Default is `NULL`, in which case the
#' range is simply the minimum and maximum levels in the curve.
#' @param color String giving the name of the color to be used for
#' plotting intensity. Default is `'black'`.
#' @param ind Integer indexing intensity relative to other plot components.
#' Default is `NULL`.
#' @param drawSize Number indicating the line width of the intensity contour.
#' Default is `1`. Controls the `lwd` argument of [graphics::lines].
#' @param axisLabel String giving the name of the label to print along
#' the y-axis when plotting intensity. Default is `Intensity (dB)`.
#' @param min_max_only Logical; should only minimum and maximum values be given
#' on the y-axis? Default is `TRUE`. Can also be a logical vector if some but
#' not all plot components should have minimum and maximum values on the y-axis.
#' Ignored for TextGrid component.
#' @param highlight Named list giving parameters for differential
#' highlighting of the intensity contour based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `color`) and
#' `drawSize` (integer), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param pitch_overlay Logical; is pitch also overlaid on the same
#' plot frame? Default is `FALSE`.
#' @param suppressAxes Should all axes and the box around plots be suppressed?
#' Default is `FALSE`. Generally not recommended.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, frames = 'spectrogram',
#' intensity_plotOnSpec = TRUE)
intensity_overlay <- function(it, bottomRange, topRange, start, org_start=0,
                              tfrom0=TRUE, range=NULL, color='black', ind=NULL,
                              drawSize=1, axisLabel='Intensity (dB)',
                              min_max_only=TRUE, highlight=NULL,
                              pitch_overlay=FALSE, suppressAxes=FALSE) {
  if (tfrom0) it$t <- it$t - org_start
  sRan <- topRange - bottomRange
  iRan <- range[2] - range[1]
  multiplier <- sRan / iRan
  it$i <- bottomRange + (it$i - range[1]) * multiplier

  create_highlighting <- function(highlight, it) {
    hl_t <- c()
    hl_i <- c()
    for (int in 1:length(highlight$start)) {
      times <- which(it$t > highlight$start[int] & it$t < highlight$end[int])
      if (highlight$start[int] < start) {
        extrema <- zoo::na.approx(c(NA, it$i),
                                  c(highlight$end[int], it$t))
        hl_t <- c(hl_t, it$t[times], highlight$end[int],
                  max(it$t[times] + 0.0001))
        hl_i <- c(hl_i, it$i[times], extrema[1], NA)
      } else {
        extrema <- zoo::na.approx(c(NA, NA, it$i),
                                  c(highlight$start[int], highlight$end[int],
                                    it$t))[1:2]
        hl_t <- c(hl_t, highlight$start[int], it$t[times], highlight$end[int],
                  max(it$t[times] + 0.0001))
        hl_i <- c(hl_i, extrema[1], it$i[times], extrema[2], NA)
      }
    }
    if (any(highlight$start < start)) {
      highlight$start[which(highlight$start < start)] <- start
    }
    if (!'color' %in% names(highlight)) highlight$color <- color
    if (!'drawSize' %in% names(highlight)) highlight$drawSize <- drawSize
    return(list(params = highlight, hl_t = hl_t, hl_i = hl_i))
  }

  if (length(color) == 2) graphics::lines(it$t, it$i,
                                                    lwd=drawSize+2,
                                                    col=color[2])
  graphics::lines(it$t, it$i, col=color[1], lwd=drawSize)
  if (!is.null(highlight)) {
    if (any(sapply(highlight, is.list))) {
      highlightLoopDepth <- sum(sapply(highlight, is.list))
    } else {
      highlightLoopDepth <- 1
    }
    for (i in 1:highlightLoopDepth) {
      if (any(sapply(highlight, is.list))) {
        thisHL <- create_highlighting(highlight[[i]], it)
      } else {
        thisHL <- create_highlighting(highlight, it)
      }
      if (length(thisHL$params$color) == 2) {
        graphics::lines(thisHL$hl_t, thisHL$hl_i,
                        lwd=thisHL$params$drawSize+2,
                        col=thisHL$params$color[2])
      }
      graphics::lines(thisHL$hl_t, thisHL$hl_i,
                      col=thisHL$params$color,
                      lwd=thisHL$params$drawSize)
    }
  }

  iline <- c(3.5,1)
  if (pitch_overlay) iline <- c(3.5,3.5)

  if (!suppressAxes) {
    graphics::mtext(axisLabel, side=4, line=iline[1], cex=0.8, las=3,
                    col=color[1])

    if (min_max_only[ind]) {
      graphics::mtext(round(range[1], 0),
                      side=4, line=iline[2], at=bottomRange,
                      padj=0, las=2, cex=0.7, col=color[1])
      graphics::mtext(round(range[2]),
                      side=4, line=iline[2], at=topRange,
                      padj=1, las=2, cex=0.7, col=color[1])
    } else {
      rtix <- pretty(range[1]:range[2])
      graphics::axis(4, at=(rtix-range[1])*multiplier, labels=rtix,
                     col.ticks=color[1],
                     col.axis=color[1])
    }
  }

}

#' Make Praat Picture style plots of acoustic data
#'
#' Generate plots of acoustic data aligned with transcriptions similar
#' to those made with Praat Picture. The default is to produce a plot with a
#' relatively small waveform, somewhat larger spectrogram, and the first tier
#' of a TextGrid.
#'
#' @param sound String giving the file name of a sound file with the .wav
#' extension.
#' @param start Start time (in seconds) of desired plotted area. Default is `0`.
#' @param end End time (in seconds) of desired plotted area. Default is `0`
#' (= the entire file).
#' @param tfrom0 Logical; should time on the x-axis run from 0 or from the
#' original time? Default is `TRUE`.
#' @param tUnit String giving the unit of time to print along the x-axis.
#' Possible options are `'s'` (default) for seconds and `'ms'` for milliseconds.
#' @param frames String or vector of strings giving the frames that the plot
#' should consist of. Default is `sound`, `spectrogram`, `TextGrid`. This
#' requires a file with the extension `.TextGrid` and the same base name as the
#' sound file. Other options are `pitch`, `formant`, and `intensity`. See
#' details for more information.
#' @param proportion Integer or vector of integers of the same size as `frames`
#' giving the proportion in percents of the plotting area to be taken up by the
#' individual frames. Default is `c(30,50,20)`. If more or less than three
#' frames are plotted and no proportions are given, frames will be of equal
#' size.
#' @param mainTitle String giving a title to print at the top of the plot.
#' The default is an empty string, i.e. no title.
#' @param mainTitleAlignment Number indicating the vertical alignment of the
#' plot title, where `0` (default) indicates left-alignment, `1` indicates
#' right-alignment, `0.5` indicates central alignment, etc, following the
#' conventions of the `adj` argument of [graphics::mtext].
#' @param start_end_only Logical; should there only be ticks on the x-axis
#' for start and end times? Default is `TRUE`.
#' @param min_max_only Logical; should only minimum and maximum values be given
#' on the y-axis? Default is `TRUE`. Can also be a logical vector if some but
#' not all plot components should have minimum and maximum values on the y-axis.
#' Ignored for TextGrid component.
#' @param drawSize Number indicating the line width of plot components where
#' the `_plotType` is `'draw'` (i.e., pitch, formants, or intensity rendered as
#' line plots). Default is `1`. Controls the `lwd` argument of
#' [graphics::lines].
#' @param speckleSize Number indicating the point size of plot components where
#' the `_plotType` is `'speckle'` (i.e. pitch or formants rendered as point
#' plots). Default is `1`. Controls the `cex` arguments of [graphics::points].
#' @param wave_channels Vector of numbers or strings giving either numeric
#' identifiers of audio channels to plot of the names of audio channels to plot.
#' Also understands `'all'`, which plots all channels and is the default.
#' @param wave_channelNames Should names of audio channels be printed
#' on the y-axis? If `TRUE`, names will be grabbed from the audio metadata if
#' available. Alternatively, if two channels are available, they will be named
#' `left` and `right`. If more or less than two channels are available,
#' channels are named `Cn`, where `n` is the number of the channel. Alternatvely,
#' a vector of strings can be provided with channel names. Default is `FALSE`.
#' @param wave_energyRange Numeric vector giving the desired energy
#' range (y-axis range) of waveform(s). Default is `NULL`, in which case the
#' y-axis range is set to the lowest and highest value for each wave.
#' If a vector of two values is passed for a multi-wave plot, these are
#' recycled for each wave. Separate ranges can be set for each waveform by
#' passing a vector of a length corresponding to twice the number of waves
#' plotted.
#' @param wave_axisDigits Numeric giving the number of digits to print for
#' values along the y-axis of the waveform. Default is `3`. If `0` is passed,
#' the y-axis is suppressed. Alternatively a vector of numbers, if different
#' numbers should be used for different channels. Note that this only
#' applies when `min_max_only = TRUE`, as otherwise the look of the y-axis is
#' determined entirely using `grDevices::axisTicks()`.
#' @param wave_color String giving the name of the color to be used for plotting
#' the waveform. Default is `'black'`. Alternatively a vector of strings, if
#' different colors should be used for different channels.
#' @param wave_lineWidth Number giving the line width to use for plotting
#' the waveform. Default is `1`. Alternatively a vector of numbers, if
#' different line widths should be used for different channels.
#' @param wave_highlight Named list giving parameters for differential
#' highlighting of the waveform based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the argument
#' `color` (string, see `wave_color`), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param tg_file Path of file to be used for plotting TextGrid. Default is
#' `NULL`, in which case the function searches for a TextGrid sharing the same
#' base name as `sound` with the `.TextGrid` extension.
#' @param tg_obj A TextGrid object returned by the [make_TextGrid()] function.
#' @param tg_tiers Vector of numbers or strings giving either numeric identifiers
#' of TextGrid tiers to plot or the names of TextGrid tiers to plot. Also
#' understands `'all'`, which plots all tiers and is the default.
#' @param tg_focusTier For which tier(s) should lines be shown on all
#' acoustic plots giving the locations of boundaries? Vector of number or
#' strings giving either numeric identifiers
#' of TextGrid tiers or the names of TextGrid tiers to plot. Default is
#' `tg_tiers[1]`, i.e. the first tier given in the `tg_tiers` argument.
#' Additionally accepts the string `none`,
#' in which case no lines are shown on acoustic plots, and `all`, in
#' which case lines from all tiers are shown on acoustic plots.
#' @param tg_focusTierColor String or vector of strings giving the color(s) to
#' use for plotting focus tier lines. If multiple tiers are focused, a vector
#' of the same length can be passed, and the nth tier will be plotted in the
#' nth color. Default is `'black'`.
#' @param tg_focusTierLineType String or vector of strings giving the line
#' type(s) for plotting focus tier lines. If multiple tiers are focused, a
#' vector of the same length can be passed, and the nth tier will be plotted in
#' the nth line type. Default is `'dotted'`.
#' @param tg_alignment String giving the desired alignment of text in the
#' TextGrids. Default is `central`; other options are `left` and `right`.
#' Alternatively, a vector of strings if different alignments are needed.
#' @param tg_edgeLabels String specifying how to handle TextGrid labels in
#' interval tiers that fall partially before `start` or partially after `end`.
#' Default is `'keep'`, where labels are kept at the center of the interval.
#' Other options are `'center'`, where labels are recentered to the visible
#' part of the interval, or `'discard'`, where such labels are ignored.
#' @param tg_specialChar Logical; should Praat typesetting for special font types
#' such as italic, bold, and small caps be converted into corresponding
#' R-readable special font types. Default is `FALSE`, since special characters
#' can create unfortunate text alignment artefacts.
#' See https://www.fon.hum.uva.nl/praat/manual/Text_styles.html.
#' @param tg_tierNames Logical; should TextGrid tier names be printed along the
#' y-axis? Default is `TRUE`.
#' @param tg_color String or vector of strings giving the name of the color(s)
#' to be used for the text in TextGrids. Default is `'black'`. If a vector is
#' provided, different colors are used for different tiers.
#' @param tg_highlight Named list giving parameters for differential
#' highlighting of TextGrid intervals. This list
#' should contain information about which intervals to highlight, using the
#' `tier` and `label`. Further contains the argument
#' `color`, and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param spec_channel Numeric giving the channel that should be used to
#' generate the spectrogram. Default is `1`. Generating spectrograms from
#' multiple channels is not currently possible with `praatpicture`.
#' @param spec_scale String giving the frequency scale to use for plotting
#' spectrograms. Default is `hz`. Alternatives are `erb` (for the equivalent
#' rectangular bandwidth scale), `mel` (for the Mel scale, using 1000 Hz
#' as the corner frequency, following Fant 1968), or `logarithmic` (for
#' log Hz).
#' @param spec_freqRange Vector of two integers giving the frequency range to be
#' used for plotting spectrograms. Default is `NULL`, in which case suitable
#' ranges are chosen based on the frequency scale. For `hz`, the default is
#' `c(0,5000)`; for `erb`, the default is `c(0.8,42)`; for `mel`, the default is
#' `c(30,4400)`. The latter two correspond roughly to the human auditory
#' frequency response.
#' @param spec_windowLength Window length in seconds for generating spectrograms.
#' Default is `0.005`.
#' @param spec_dynamicRange Dynamic range in dB for generating spectrograms. The
#' maximum intensity minus `spec_dynamicRange` will all be printed in white.
#' Default is `50`.
#' @param spec_timeStep How many time steps should be calculated for spectrograms?
#' Default is `1000`.
#' @param spec_windowShape String giving the name of the window shape to be
#' applied to the signal when generating spectrograms. Default is `Gaussian`;
#' other options are `square`, `Hamming`, `Bartlett`, `Hanning`, or `Blackman`.
#' Note that the Gaussian window function provided by the `phonTools` package
#' and used in `praatpicture()` does not have the same properties as the
#' Gaussian window function used for spectral estimation in Praat; plotting
#' a simple sine wave with high dynamic range will produce sidelobes in
#' `praatpicture()` but not in Praat. It's recommended to use Blackman windows
#' instead if you have this problem.
#' @param spec_colors Vector of strings giving the names of colors to be used
#' for plotting the spectrogram; default is `c('white', 'black')`. The first
#' value is used for plotting the lowest visible amplitude, and the last for
#' plotting the highest visible amplitude. Vectors with more than two color
#' names can be used for plotting values in between in different colors.
#' @param spec_axisLabel String giving the name of the label to print along the
#' y-axis when plotting a spectrogram. Default is `NULL`, in which case the
#' axis label will depend on the scale.
#' @param spec_highlight Named list giving parameters for differential
#' highlighting of the spectrogram based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the argument
#' `colors` (vector of strings, see `spec_colors`). Can be nested lists to
#' highlight different parts of a figure in different ways.
#' @param pitch_timeStep Measurement interval in seconds for tracking pitch.
#' Default is `NULL`, in which case the measurement interval is equal to
#' 0.75 / `pitch_floor`.
#' @param pitch_floor Frequency in Hz; no pitch candidates considered below
#' this frequency. Default is `75`.
#' @param pitch_ceiling Frequency in Hz; no pitch candidates considered above
#' this frequency. Default is `600`.
#' @param pitch_plotType String giving the type of pitch plot to produce; default
#' is `draw` (a line plot), the only other option is `speckle` (a point plot).
#' Alternatively a vector `c('draw','speckle')` can be passed, in which case
#' both are used.
#' @param pitch_scale String giving the frequency scale to use when producing
#' pitch plots. Default is `hz`; other options are `logarithmic` (also in Hz),
#' `semitones`, `erb`, and `mel`.
#' @param pitch_freqRange Vector of two integers giving the frequency range to be
#' used for producing pitch plots. Default is `NULL`, in which case the pitch
#' range is automatically reset to `c(-12,30)` for the `semitones` scale,
#' `c(0,10)` for the `erb` scale, and `c(50,500)` for the Hz-based scales,
#' following Praat defaults.
#' @param pitch_semitonesRe Frequency in Hz giving the reference level for
#' converting pitch frequency to semitones. Default is `100`.
#' @param pitch_color String giving the name of the color to be used for
#' plotting pitch. Default is `'black'`. If `pitch_plotOnSpec=TRUE`, axes will
#' follow the same color scheme. Also if `pitch_plotOnSpec=TRUE`, a vector of
#' two strings can be passed, in which case the second color is used for
#' background highlighting.
#' @param pitch_plotOnSpec Boolean; should pitch be plotted on top of
#' spectrogram? Default is `FALSE`.
#' @param pitch_plotOnWave Boolean; should pitch be plotted on top of
#' waveform? Default is `FALSE`.
#' @param pitch_ssff An object of class `AsspDataObj` containing a pitch track.
#' Default is `NULL`.
#' @param pitch_axisLabel String giving the name of the label to print along the
#' y-axis when printing a pitch track. Default is `NULL`, in which case the
#' axis label will depend on the scale. If `pitch_plotOnSpec=TRUE`, this label
#' will be printed on the right-hand y-axis label.
#' @param pitch_highlight Named list giving parameters for differential
#' highlighting of pitch based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `pitch_color`),
#' `drawSize` or `speckleSize` (both numeric), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param formant_timeStep Measurement interval in seconds for tracking formants.
#' Default is `NULL`, in which case the measurement interval is equal to
#' `formant_windowLength` / 4.
#' @param formant_maxN Integer giving the maximum number of formants to track.
#' Default is `5`.
#' @param formant_windowLength The effective duration of the analysis window
#' used for tracking formants in seconds; the actual duration of the
#' analysis window is twice this value.
#' @param formant_dynamicRange Dynamic range in dB for producing formant plots.
#' When a formant plot of `formant_plotType='speckle'` is drawn, no formants are
#' shown in frames with intensity level `formant_dynamicRange` below the maximum
#' intensity. Default is `30`. If set to `0`, all formants are shown.
#' @param formant_freqRange Vector of two integers giving the frequency range to
#' be used for producing formant plots. Default is `c(0,5500)`.
#' @param formant_number Number of formants to plot. Default is `NULL`, in
#' which case all available formants are plotted.
#' @param formant_plotType String giving the type of formant plot to produce;
#' default is `speckle` (a point plot), the only other option is `draw` (a line
#' plot). Alternatively a vector `c('draw','speckle')` can be passed, in which
#' case both are used.
#' @param formant_color String or vector of strings giving the name(s) of
#' colors to be used for plotting formants. If one color is provided, all
#' formants will be plotted in this color. If multiple colors are provided,
#' different formants will be shown in different colors. Default is `'black'`.
#' If `formant_plotOnSpec=TRUE` and the length of this vector twice the number
#' of formants plotted, the first
#' half of strings will be used for the formants' primary colors and the second
#' half will be used for background highlighting. If the length of this vector
#' is one more than the number of formants plotted, the last string will
#' be used for background highlighting.
#' @param formant_plotOnSpec Boolean; should formants be plotted on top of
#' spectrogram? Default is `FALSE`.
#' @param formant_ssff An object of class `AsspDataObj` containing formant tracks.
#' Default is `NULL`.
#' @param formant_axisLabel String giving the name of the label to print along the
#' y-axis when plotting formants. Default is `Frequency (Hz)`.
#' @param formant_highlight Named list giving parameters for differential
#' highlighting of formants based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `formant_color`),
#' `drawSize` or `speckleSize` (both numeric), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param intensity_timeStep Measurement interval in seconds for tracking
#' intensity. Default is `NULL`, in which case the measurement interval is
#' equal to 0.8 * `intensity_minPitch`.
#' @param intensity_minPitch Lowest pitch in Hz used when calculating
#' intensity; default is `100`
#' @param intensity_range Vector of two integers giving the intensity range to be
#' used for producing intensity plots. Default is `NULL`, in which case the
#' range is simply the minimum and maximum levels in the curve.
#' @param intensity_color String giving the name of the color to be used for
#' plotting intensity. Default is `'black'`. If `intensity_plotOnSpec=TRUE`,
#' axes will follow the same color scheme. Also if `intensity_plotOnSpec=TRUE`,
#' a vector of two strings can be passed, in which case the second color is used
#' for background highlighting.
#' @param intensity_plotOnSpec Boolean; should intensity be plotted on top of
#' spectrogram? Default is `FALSE`.
#' @param intensity_plotOnWave Boolean; should intensity be plotted on top of
#' waveform? Default is `FALSE`.
#' @param intensity_ssff An object of class `AsspDataObj` containing intensity
#' tracks. Default is `NULL`.
#' @param intensity_axisLabel String giving the name of the label to print along
#' the y-axis when plotting intensity. Default is `Intensity (dB)`.
#' If `intensity_plotOnSpec=TRUE`, this label
#' will be printed on the right-hand y-axis label.
#' @param intensity_highlight Named list giving parameters for differential
#' highlighting of the intensity contour based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `intensity_color`) and
#' `drawSize` (integer), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param time_axisLabel String giving the name of the label to print along
#' the x-axis. Default is `NULL`, in which case `Time (s)` is printed if
#' `tUnit = 's'` and `Time (ms)` is printed if `tUnit = 'ms'`.
#' @param highlight Named list giving parameters for differential highlighting
#' of part of the plot based on the time domain. This list should contain
#' information about which parts of the plot to highlight, either done with the
#' `start` and `end` arguments which must be numbers or numeric vectors, or
#' using the `tier` and `label` arguments to highlight based on information in
#' a plotted TextGrid. Further contains the optional arguments `color`
#' (a string), `drawSize` and `speckleSize` (both numeric), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways. This argument is
#' used to highlight all plot components, use the `*_highlight` arguments for
#' highlighting individuals plot components.
#' @param suppressAxes Should all axes and the box around plots be suppressed?
#' Default is `FALSE`. Generally not recommended.
#' @param draw_lines Use for drawing straight lines on plot components. Takes
#' an argument of type `list` which should contain a) a string giving the plot
#' component to draw straight lines on, and b) arguments to pass on to
#' [graphics::abline]. Should have a named argument `h` for horizontal lines,
#' or `v` for vertical lines, or `a`,`b` for the intercept and slope of the
#' line otherwise. Alternatively a nested list can be passed if more (sets of)
#' lines should be drawn. If multiple audio
#' channels are plotted and lines should be added to one of these,
#' use the channel identifier instead of a string giving the frame to draw on.
#' The default value is `list('formant', h=seq(0,10000,by=1000), lty='dotted')`.
#' According to Praat defaults, this means that if formants are plotted in a
#' separate frame, horizontal dotted lines (`lty`) are shown at 1000 Hz
#' intervals. To override this behavior, simply pass `draw_lines=NULL`.
#' @param draw_rectangle Use for drawing rectangles on plot components. A
#' vector containing a) a string giving the plot component to draw a rectangle
#' on, and b) arguments to pass on to [graphics::rect]. Alternatively a list
#' of such vectors, if more rectangles should be drawn. If multiple audio
#' channels are plotted and a rectangle should be added to one of these,
#' use the channel identifier instead of a string giving the frame to draw on.
#' @param draw_arrow Use for drawing arrows on plot components. A vector
#' containing a) a string giving the plot component to draw an arrow on, and
#' b) arguments to pass on to [graphics::arrows]. Alternatively a list of
#' such vectors, if more arrows should be drawn. If multiple audio
#' channels are plotted and an arrow should be added to one of these,
#' use the channel identifier instead of a string giving the frame to draw on.
#' @param annotate Use for annotating plot components. A vector containing
#' a) a string giving the plot component to annotate, and
#' b) arguments to pass on to [graphics::text]. Alternatively a list of
#' such vectors, if more annotations should be made. If multiple audio
#' channels are plotted and annotations should be added to one of these,
#' use the channel identifier instead of a string giving the frame to draw on.
#' @param gender String indicating the gender of the speaker; default is
#' `u` for unknown, other legal values are `m` and `f`. Used to tweak pitch
#' and formant tracking parameters.
#' @param ... Further global plotting arguments passed on to `par()`.
#'
#' @details When available, pitch, formant, and intensity tracks are loaded
#' from Praat files with the same base name as `sound`; i.e., if your sound
#' file is called `1.wav` and there is a Praat file called `1.Formant` in the
#' same directory, this file is used for plotting formants. Pitch files should
#' have either the `PitchTier` or `Pitch` extension, and intensity files should
#' have the `IntensityTier` extension.
#'
#' If no such files are available, the signal processing tools in the `wrassp`
#' package are used; pitch is tracked with the function [wrassp::ksvF0],
#' formants are tracked with [wrassp::forest], and intensity is tracked with
#' [wrassp::rmsana]. Parameters are set to mimic Praat as closely as possible,
#' e.g. using a Gaussian-like window shape `KAISER2_0`, but results will differ
#' from Praat simply because the tracking algorithms differ; as far as I know,
#' the Burg algorithm used by Praat for tracking formants isn't implemented in R,
#' nor is the autocorrelation method for tracking pitch.
#'
#' Spectrograms are generated with the function [phonTools::spectrogram]. The
#' code portion that actually adds the spectrogram to a plot is based on
#' [phonTools::plot.spectrogram] but rewritten to use a bitmap raster for
#' rendering the image if the graphics device allows for it, which
#' significantly speeds up rendering the spectrogram.
#'
#' @return No return value, produces a figure.
#' @export
#'
#' @examples
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile)
praatpicture <- function(sound, start=0, end=0, tfrom0=TRUE, tUnit='s',
                         frames=c('sound', 'spectrogram', 'TextGrid'),
                         proportion=c(30,50,20), mainTitle='',
                         mainTitleAlignment = 0,
                         start_end_only=TRUE, min_max_only=TRUE,
                         drawSize = 1, speckleSize = 1,
                         wave_channels='all', wave_channelNames=FALSE,
                         wave_energyRange=NULL, wave_axisDigits=3,
                         wave_color='black', wave_lineWidth=1,
                         wave_highlight=NULL,
                         tg_obj=NULL, tg_file=NULL, tg_tiers='all',
                         tg_focusTier=tg_tiers[1], tg_focusTierColor='black',
                         tg_focusTierLineType='dotted', tg_tierNames=TRUE,
                         tg_alignment='central', tg_edgeLabels='keep',
                         tg_specialChar=FALSE,
                         tg_color='black', tg_highlight=NULL,
                         spec_channel=NULL, spec_scale='hz',
                         spec_freqRange=NULL,
                         spec_windowLength=0.005, spec_dynamicRange=50,
                         spec_timeStep=1000, spec_windowShape='Gaussian',
                         spec_colors=c('white', 'black'),
                         spec_axisLabel=NULL,
                         spec_highlight=NULL,
                         pitch_timeStep=NULL, pitch_floor=75, pitch_ceiling=600,
                         pitch_plotType='draw', pitch_scale='hz',
                         pitch_freqRange=NULL, pitch_semitonesRe=100,
                         pitch_color='black', pitch_plotOnSpec=FALSE,
                         pitch_plotOnWave=FALSE, pitch_ssff=NULL,
                         pitch_axisLabel=NULL, pitch_highlight=NULL,
                         formant_timeStep=NULL, formant_maxN=5,
                         formant_windowLength=0.025, formant_dynamicRange=30,
                         formant_freqRange=c(50, 5500),
                         formant_number=NULL,
                         formant_plotType='speckle', formant_color='black',
                         formant_plotOnSpec=FALSE,
                         formant_ssff=NULL, formant_axisLabel='Frequency (Hz)',
                         formant_highlight=NULL,
                         intensity_timeStep=NULL, intensity_minPitch=100,
                         intensity_range=NULL, intensity_color='black',
                         intensity_plotOnSpec=FALSE, intensity_plotOnWave=FALSE,
                         intensity_ssff=NULL,
                         intensity_axisLabel='Intensity (dB)',
                         intensity_highlight=NULL,
                         time_axisLabel=NULL,
                         highlight=NULL,
                         suppressAxes=FALSE,
                         draw_lines=list('formant', h=seq(0,10000,by=1000),
                                         lty='dotted'),
                         draw_rectangle=NULL, draw_arrow=NULL, annotate=NULL,
                         gender='u', ...) {

  p <- graphics::par(no.readonly=TRUE)
  on.exit(graphics::par(p))

  legal_frames <- c('sound', 'TextGrid', 'spectrogram', 'pitch', 'formant',
                    'intensity')
  if (any(!frames %in% legal_frames)) {
    stop('Currently available frames are sound, TextGrid, spectrogram, formant, ',
         'pitch, and intensity')
  }
  if (spec_scale == 'log') spec_scale <- 'logarithmic'
  legal_spec_scales <- c('hz', 'logarithmic', 'erb', 'mel')
  if (!spec_scale %in% legal_spec_scales) {
    stop('Possible spectrogram scales are hz, logarithmic, erb, and mel')
  }
  if (pitch_scale == 'log') pitch_scale <- 'logarithmic'
  legal_pitch_scales <- c('hz', 'logarithmic', 'semitones', 'erb', 'mel')
  if (!pitch_scale %in% legal_pitch_scales) {
    stop('Possible pitch scales are hz, logarithmic, semitones, erb, and mel')
  }
  if (!any(pitch_plotType %in% c('draw', 'speckle'))) {
    stop('Please select either draw or speckle as the pitch plot type')
  }
  if (!tUnit %in% c('s', 'ms')) {
    stop('Please select either s or ms as the time unit')
  }
  if (!tg_edgeLabels %in% c('keep', 'center', 'discard')) {
    stop('tg_edgeLabels should be either keep, center, or discard')
  }

  if (is.null(time_axisLabel)) {
    time_axisLabel <- ifelse(tUnit == 's', 'Time (s)', 'Time (ms)')
  }
  if (is.null(formant_number)) {
    formant_number <- formant_maxN
  }

  proportion <- round((proportion / sum(proportion)) * 100)

  nframe <- length(frames)
  if (nframe != 3 & length(proportion) != nframe) {
    proportion <- rep(round(100/nframe), nframe)
  }
  if (nframe != length(proportion)) stop(paste('The proportion argument must',
                                               'be the same length as the',
                                               'frames argument'))
  if (nframe > length(min_max_only)) min_max_only <- rep(min_max_only, nframe)

  if (is.character(draw_lines[[1]])) draw_lines <- list(draw_lines)
  line_comp <- sapply(draw_lines, '[[', 1)

  if (!is.list(draw_rectangle)) draw_rectangle <- list(draw_rectangle)
  rect_comp <- sapply(draw_rectangle, '[[', 1)

  if (!is.list(draw_arrow)) draw_arrow <- list(draw_arrow)
  arr_comp <- sapply(draw_arrow, '[[', 1)

  if (!is.list(annotate)) annotate <- list(annotate)
  annot_comp <- sapply(annotate, '[[', 1)

  if (end == 0) {
    tend <- Inf
  } else {
    tend <- end
  }
  snd <- tuneR::readWave(sound, from=start, to=tend, units='seconds', toWaveMC=T)

  sr <- snd@samp.rate
  bit <- snd@bit
  nsamp <- snd@dim[1]
  nchan <- snd@dim[2]
  if (any(wave_channels == 'all')) {
    wave_channels <- 1:nchan
  }

  if (any(is.null(spec_channel))) {
    if (any(wave_channels == 'all')) {
      spec_channel <- 1
    } else {
      spec_channel <- wave_channels[1]
    }
  }

  sig <- snd@.Data[,wave_channels]
  if (any(class(sig) %in% c('integer', 'numeric'))) sig <- as.matrix(sig)
  nchan <- dim(sig)[2]

  if (is.logical(wave_channelNames) & isTRUE(wave_channelNames)) {
    if (length(colnames(sig)) > 0) {
      cn <- colnames(sig)
    } else if (nchan == 2) {
      cn <- c('left', 'right')
    } else {
      cn <- paste0('C', 1:nchan)
    }
  } else if (is.character(wave_channelNames)) {
    cn <- wave_channelNames
    wave_channelNames <- TRUE
  } else {
    cn <- NULL
  }

  if (start == 0) {
    tstart <- 0 + (1/sr)
  } else {
    tstart <- start
  }
  tseq <- seq(tstart, tstart+nsamp/sr, by=1/sr)

  spl <- unlist(strsplit(sound, '[.]'))
  fn <- paste(spl[1:(length(spl)-1)], collapse='.')

  if ('TextGrid' %in% frames) {
    if (!is.null(tg_file)) {
      tgfn <- tg_file
    } else {
      tgfn <- paste0(fn, '.TextGrid')
    }
    if (!is.null(tg_obj)) {
      tg <- tg_obj
    } else if (file.exists(tgfn)) {
      tg <- rPraat::tg.read(tgfn, encoding='auto')
    } else {
      stop(paste('There is no TextGrid file available for this sound file.',
                 'Either create the plot without a TextGrid by using the',
                 'frames argument, create a TextGrid object using the',
                 'make_TextGrid() function and pass it to the tg_obj argument,',
                 'or create a TextGrid using Praat',
                 'or the functionality available in the rPraat package.'))
    }

    tgind <- which(frames == 'TextGrid')
    if (any(tg_tiers == 'all')) tg_tiers <- 1:length(tg)
    if (length(tg_tiers) > 1) {
      ntiers <- length(tg_tiers)
      tgprop <- proportion[tgind]
      proportion[tgind] <- round(tgprop / ntiers)
      if (tgind == nframe) {
        proportion[(nframe+1):(nframe+ntiers-1)] <-
          round(tgprop / ntiers)
      } else {
        proportion[(tgind+ntiers):(nframe+ntiers-1)] <-
          proportion[(tgind+1):nframe]
        proportion[(tgind+1):(tgind+ntiers-1)] <-
          round(tgprop / ntiers)
      }
    }

    if (any(tg_focusTier == 'all')) tg_focusTier <- tg_tiers
    if (length(tg_focusTierColor) == 1) {
      tg_focusTierColor <- rep(tg_focusTierColor, length(tg_focusTier))
    }
    if (length(tg_focusTierLineType) == 1) {
      tg_focusTierLineType <- rep(tg_focusTierLineType, length(tg_focusTier))
    }
    if (any(tg_focusTier != 'none')) {
      tgbool <- TRUE
      focus_linevec <- list()
      j <- 1
      for (i in tg_focusTier) {
        if (tfrom0) {
          if (tg[[i]]$type == 'point') focus_linevec[[j]] <- tg[[i]]$t - start
          if (tg[[i]]$type == 'interval') focus_linevec[[j]] <-
              tg[[i]]$t1[-1] - start
        } else {
          if (tg[[i]]$type == 'point') focus_linevec[[j]] <- tg[[i]]$t
          if (tg[[i]]$type == 'interval') focus_linevec[[j]] <- tg[[i]]$t1[-1]
        }
        j <- j + 1
      }
    } else {
      tgbool <- FALSE
      focus_linevec <- NULL
    }
  } else {
    tgbool <- FALSE
    focus_linevec <- NULL
  }

  findStartEndVals <- function(listobj) {
    if ('tier' %in% names(listobj)) {
      tierName <- listobj$tier
      matches <- which(grepl(listobj$label, tg[[tierName]]$label, perl=T))
      if (length(matches) == 0) stop(paste('There is no match for the',
                                            'highlighting conditions'))
      listobj$start <- tg[[tierName]]$t1[matches]
      listobj$end <- tg[[tierName]]$t2[matches]
      if (end != 0) {
        discard <- which(listobj$end < start | listobj$start > end)
      } else {
        discard <- which(listobj$end < start)
      }
      if (length(discard) > 0) {
        listobj$start <- listobj$start[-discard]
        listobj$end <- listobj$end[-discard]
      }
      if (start > 0 & tfrom0) {
        listobj$start <- listobj$start - start
        listobj$end <- listobj$end - start
      }
    }
    return(listobj)
  }

  if (!is.null(highlight)) {
    if (is.null(wave_highlight)) wave_highlight <- highlight
    if (is.null(pitch_highlight)) pitch_highlight <- highlight
    if (is.null(formant_highlight)) formant_highlight <- highlight
    if (is.null(intensity_highlight)) intensity_highlight <- highlight
    if (is.null(spec_highlight)) {
      if (any(sapply(highlight, is.list))) {
        spec_highlight <- list()
        for (i in 1:length(highlight)) {
          spec_highlight[[i]] <- highlight[[i]]
          spec_highlight[[i]]$colors <- spec_colors
          if ('color' %in% names(highlight[[i]])) {
            spec_highlight[[i]]$colors <-
              c(spec_highlight[[i]]$colors[1], highlight[[i]]$color)
          }
          if ('background' %in% names(highlight[[i]])) {
            spec_highlight[[i]]$colors <-
              c(highlight[[i]]$background, spec_highlight[[i]]$colors[2])
          }
        }
      } else {
        spec_highlight <- highlight
        spec_highlight$colors <- spec_colors
        if ('color' %in% names(highlight)) spec_highlight$colors <-
          c(spec_highlight$colors[1], highlight$color)
        if ('background' %in% names(highlight)) spec_highlight$colors <-
          c(highlight$background, spec_highlight$colors[2])
      }
    }
    if (is.null(tg_highlight)) tg_highlight <- highlight
  }

  if (any(sapply(wave_highlight, is.list))) {
    for (i in 1:length(wave_highlight)) {
      wave_highlight[[i]] <- findStartEndVals(wave_highlight[[i]])
    }
  } else {
    wave_highlight <- findStartEndVals(wave_highlight)
  }

  if (any(sapply(pitch_highlight, is.list))) {
    for (i in 1:length(pitch_highlight)) {
      pitch_highlight[[i]] <- findStartEndVals(pitch_highlight[[i]])
    }
  } else {
    pitch_highlight <- findStartEndVals(pitch_highlight)
  }

  if (any(sapply(formant_highlight, is.list))) {
    for (i in 1:length(formant_highlight)) {
      formant_highlight[[i]] <- findStartEndVals(formant_highlight[[i]])
    }
  } else {
    formant_highlight <- findStartEndVals(formant_highlight)
  }

  if (any(sapply(intensity_highlight, is.list))) {
    for (i in 1:length(intensity_highlight)) {
      intensity_highlight[[i]] <- findStartEndVals(intensity_highlight[[i]])
    }
  } else {
    intensity_highlight <- findStartEndVals(intensity_highlight)
  }

  if (any(sapply(spec_highlight, is.list))) {
    for (i in 1:length(spec_highlight)) {
      spec_highlight[[i]] <- findStartEndVals(spec_highlight[[i]])
    }
  } else {
    spec_highlight <- findStartEndVals(spec_highlight)
  }

  if ('spectrogram' %in% frames) {
    if (is.null(spec_freqRange)) {
      if (spec_scale == 'erb') {
        spec_freqRange <- c(0.8,42)
      } else if (spec_scale == 'mel') {
        spec_freqRange <- c(30,4400)
      } else if (spec_scale == 'logarithmic') {
        spec_freqRange <- c(50,5000)
      } else {
        spec_freqRange <- c(0,5000)
      }
    }
    if (is.null(spec_axisLabel)) {
      if (spec_scale == 'hz') spec_axisLabel <- 'Frequency (Hz)'
      if (spec_scale == 'logarithmic') spec_axisLabel <- 'Frequency (log Hz)'
      if (spec_scale == 'mel') spec_axisLabel <- 'Frequency (mel)'
      if (spec_scale == 'erb') spec_axisLabel <- 'Frequency (ERB)'
    }
  }

  if ('pitch' %in% frames | pitch_plotOnSpec | pitch_plotOnWave) {
    if (is.null(pitch_freqRange)) {
      if (pitch_scale == 'erb') {
        pitch_freqRange <- c(0,10)
      } else if (pitch_scale == 'semitones') {
        pitch_freqRange <- c(-12,30)
      } else {
        pitch_freqRange <- c(50,500)
      }
    }
    if (is.null(pitch_axisLabel)) {
      if (pitch_scale == 'hz') pitch_axisLabel <- 'Frequency (Hz)'
      if (pitch_scale == 'logarithmic') pitch_axisLabel <- 'Frequency (log Hz)'
      if (pitch_scale == 'mel') pitch_axisLabel <- 'Frequency (mel)'
      if (pitch_scale == 'erb') pitch_axisLabel <- 'Frequency (ERB)'
      if (pitch_scale == 'semitones') pitch_axisLabel <-
          paste0('Frequency (semitones re ', pitch_semitonesRe, ')')
    }
    if (file.exists(paste0(fn, '.PitchTier'))) {
      ptfn <- paste0(fn, '.PitchTier')
      pt <- rPraat::pt.read(ptfn, encoding = 'auto')
    } else if (file.exists(paste0(fn, '.Pitch'))) {
      ptfn <- paste0(fn, '.Pitch')
      pt <- rPraat::pitch.read(ptfn, encoding = 'auto')
      f <- c()
      for (i in 1:length(pt$frame)) f[i] <- pt$frame[[i]]$frequency[1]
      t <- pt$t
      f[which(f==0)] <- NA
      pt <- list(t = t, f = f)
    } else {
      if (!is.null(pitch_ssff)) {
        wpt <- pitch_ssff
      } else {
        if (is.null(pitch_timeStep)) {
          pitch_timeStep <- (0.75 / pitch_floor) * 1000
        } else {
          pitch_timeStep <- pitch_timeStep * 1000
        }
        wpt <- wrassp::ksvF0(paste0(fn, '.wav'), toFile=F,
                             beginTime=start, endTime=end,
                             windowShift=pitch_timeStep,
                             maxF=pitch_ceiling, minF=pitch_floor,
                             gender=gender)
      }
      f <- wpt[[1]][,1]
      a <- attributes(wpt)
      t <- seq(a$startRecord/a$sampleRate, a$endRecord/a$sampleRate,
               by=1/a$sampleRate)
      f[which(f==0)] <- NA
      pt <- list(t = t, f = f)
    }
    if (pitch_scale == 'semitones') pt <-
        rPraat::pt.Hz2ST(pt, ref=pitch_semitonesRe)
    if (pitch_scale == 'mel') pt$f <- emuR::mel(pt$f)
    if (pitch_scale == 'erb') pt$f <- soundgen::HzToOther(pt$f, scale = 'ERB')
  }

  if ('formant' %in% frames | formant_plotOnSpec) {
    if (file.exists(paste0(fn, '.Formant'))) {
      fmfn <- paste0(fn, '.Formant')
      fm <- rPraat::formant.read(fmfn, encoding = 'auto')
      fm <- rPraat::formant.toArray(fm)
      fm$conv2db <- TRUE
      mnf <- dim(fm$frequencyArray)[1]
      if (formant_number < mnf) {
        fm$frequencyArray <- fm$frequencyArray[1:formant_number,]
        fm$maxnFormants <- formant_number
      }
      if (formant_number > mnf) stop(paste('formant_number is greater than the',
                                           'number of formants available'))
    } else {
      if (!is.null(formant_ssff)) {
        wfm <- formant_ssff
        a <- attributes(wfm)
        it_time <- (1 / a$sampleRate) * 1000
        wit <- wrassp::rmsana(paste0(fn, '.wav'), toFile=F,
                              windowShift=it_time,
                              windowSize=(3.2/intensity_minPitch)*1000,
                              window='KAISER2_0')
      } else {
        if (is.null(formant_timeStep)) {
          formant_timeStep <- (0.25 * formant_windowLength) * 1000
        } else {
          formant_timeStep <- formant_timeStep * 1000
        }
        wfm <- wrassp::forest(paste0(fn, '.wav'), toFile=F,
                              beginTime=start, endTime=end,
                              windowShift = formant_timeStep,
                              windowSize = formant_windowLength*2000,
                              numFormants = formant_maxN, estimate=TRUE,
                              window='KAISER2_0', gender=gender)
        wit <- wrassp::rmsana(paste0(fn, '.wav'), toFile=F,
                              beginTime=start, endTime=end,
                              windowShift=formant_timeStep,
                              windowSize=(3.2/intensity_minPitch)*1000,
                              window='KAISER2_0')
      }
      wfm$fm[which(wfm$fm==0)] <- NA
      a <- attributes(wfm)
      t <- seq(a$startTime, a$endRecord/a$sampleRate, by=1/a$sampleRate)
      fArray <- t(wfm[[1]])
      mnf <- dim(fArray)[1]
      if (formant_number < mnf) fArray <- fArray[1:formant_number,]
      if (formant_number > mnf) stop(paste('formant_number is greater than the',
                                           'number of formants available'))
      fm <- list(t = t, frequencyArray = fArray, maxnFormants = formant_number,
                 intensityVector = wit$rms, conv2db = FALSE)
    }
  } else {
    fm <- NULL
  }

  if ('intensity' %in% frames | intensity_plotOnSpec | intensity_plotOnWave) {
    if (file.exists(paste0(fn, '.IntensityTier'))) {
      itfn <- paste0(fn, '.IntensityTier')
      it <- rPraat::it.read(itfn, encoding = 'auto')
    } else {
      if (!is.null(intensity_ssff)) {
        wit <- intensity_ssff
      } else {
        if (is.null(intensity_timeStep)) {
          intensity_timeStep <- (0.8 / intensity_minPitch) * 1000
        } else {
          intensity_timeStep <- intensity_timeStep * 1000
        }
        wit <- wrassp::rmsana(paste0(fn, '.wav'), toFile=F,
                              beginTime=start, endTime=end,
                              windowShift=intensity_timeStep,
                              windowSize=(3.2/intensity_minPitch)*1000,
                              window='KAISER2_0')
      }
      db <- wit[[1]][,1]
      a <- attributes(wit)
      t <- seq(a$startTime, a$endRecord/a$sampleRate, by=1/a$sampleRate)
      it <- list(t = t, i = db)
    }
    if (is.null(intensity_range)) intensity_range <-
        c(min(it$i), max(it$i))
  }

  if (tfrom0) {
    t <- tseq - start
  } else {
    t <- tseq
  }

  if (nchan > 1 & 'sound' %in% frames) {
    nf <- nframe + length(tg_tiers) - 1
    wavind <- which(frames=='sound')
    if ('TextGrid' %in% frames) {
      if (length(tg_tiers) > 1 & wavind > tgind) wavind <-
          wavind + length(tg_tiers) - 1
    }
    wavprop <- proportion[wavind]
    proportion[wavind] <- round(wavprop / nchan)
    if (wavind == nf) {
      proportion[(nf+1):(nf+nchan-1)] <-
        round(wavprop / nchan)
    } else {
      proportion[(wavind+nchan):(nf+nchan-1)] <-
        proportion[(wavind+1):nf]
      proportion[(wavind+1):(wavind+nchan-1)] <-
        round(wavprop / nchan)
    }
  }

  if (suppressAxes) {
    boxType <- 'n'
  } else {
    boxType <- 'o'
  }

  plot_matrix <- rep(1:length(proportion), proportion)
  graphics::par(mar=c(0,2,0,2), oma=c(5,5,5,5), bty=boxType, ...)
  graphics::layout(matrix(plot_matrix, nrow=length(plot_matrix)))

  for (i in 1:nframe) {
    if (frames[i] == 'sound') {
      ind <- which(frames == 'sound')
      waveplot(sig, bit, t, start, tfrom0, nchan, wave_energyRange, wave_color,
               pitch_plotOnWave, pt, pitch_plotType, pitch_scale,
               pitch_freqRange, pitch_axisLabel, pitch_color, pitch_highlight,
               intensity_plotOnWave, it,
               intensity_range, intensity_axisLabel, intensity_color,
               intensity_highlight,
               tgbool, focus_linevec,
               tg_focusTierColor, tg_focusTierLineType, ind,
               line_comp, rect_comp, arr_comp, annot_comp,
               draw_lines, draw_rectangle, draw_arrow, annotate,
               wave_channelNames, wave_axisDigits, wave_lineWidth,
               cn, min_max_only, wave_highlight, drawSize, speckleSize,
               suppressAxes)
    } else if (frames[i] == 'spectrogram') {
      ind <- which(frames == 'spectrogram')
      specplot(sig[,which(wave_channels==spec_channel)], sr, t, start,
               max(tseq)-start, tfrom0, spec_scale,
               spec_freqRange, spec_windowLength, spec_dynamicRange,
               spec_timeStep, spec_windowShape, spec_colors,
               pitch_plotOnSpec, pt, pitch_plotType, pitch_scale,
               pitch_freqRange, pitch_axisLabel, pitch_color, pitch_highlight,
               formant_plotOnSpec, fm, formant_plotType, formant_dynamicRange,
               formant_color, formant_highlight, intensity_plotOnSpec, it,
               intensity_range, intensity_axisLabel, intensity_color,
               intensity_highlight,
               tgbool, focus_linevec, tg_focusTierColor,
               tg_focusTierLineType, ind, min_max_only,
               spec_highlight, spec_axisLabel, drawSize, speckleSize,
               suppressAxes)
      if ('spectrogram' %in% rect_comp) draw_rectangle('spectrogram',
                                                       draw_rectangle)
      if ('spectrogram' %in% arr_comp) draw_arrow('spectrogram', draw_arrow)
      if ('spectrogram' %in% annot_comp) make_annot('spectrogram', annotate)
      if ('spectrogram' %in% line_comp) draw_lines('spectrogram', draw_lines)
      graphics::box()
    } else if (frames[i] == 'TextGrid') {
      ind <- which(frames == 'TextGrid')
      tgplot(tg, t, sr, start, end, tg_tiers, tfrom0, tg_tierNames,
             tg_alignment, tg_edgeLabels, tg_specialChar,
             tg_color, tg_highlight)
    } else if (frames[i] == 'pitch') {
      ind <- which(frames == 'pitch')
      pitchplot(pt, start, max(tseq)-start, tfrom0, tgbool, focus_linevec,
                tg_focusTierColor, tg_focusTierLineType,
                pitch_plotType, pitch_scale, pitch_freqRange,
                pitch_semitonesRe, pitch_color, ind,
                min_max_only, pitch_highlight, pitch_axisLabel,
                drawSize, speckleSize, suppressAxes)
      if ('pitch' %in% rect_comp) draw_rectangle('pitch', draw_rectangle)
      if ('pitch' %in% arr_comp) draw_arrow('pitch', draw_arrow)
      if ('pitch' %in% annot_comp) make_annot('pitch', annotate)
      if ('pitch' %in% line_comp) draw_lines('pitch', draw_lines)
      graphics::box()
    } else if (frames[i] == 'formant') {
      ind <- which(frames == 'formant')
      formantplot(fm, start, max(tseq)-start, tfrom0, tgbool, focus_linevec,
                  tg_focusTierColor, tg_focusTierLineType,
                  formant_dynamicRange, formant_freqRange, formant_plotType,
                  formant_color, ind, min_max_only, formant_highlight,
                  formant_axisLabel, drawSize, speckleSize, suppressAxes)
      if ('formant' %in% rect_comp) draw_rectangle('formant', draw_rectangle)
      if ('formant' %in% arr_comp) draw_arrow('formant', draw_arrow)
      if ('formant' %in% annot_comp) make_annot('formant', annotate)
      if ('formant' %in% line_comp) draw_lines('formant', draw_lines)
      graphics::box()
    } else if (frames[i] == 'intensity') {
      ind <- which(frames == 'intensity')
      intensityplot(it, start, max(tseq)-start, tfrom0, tgbool, focus_linevec,
                    tg_focusTierColor, tg_focusTierLineType,
                    intensity_range, intensity_color, ind,
                    min_max_only, intensity_highlight,
                    intensity_axisLabel, drawSize, suppressAxes)
      if ('intensity' %in% rect_comp) draw_rectangle('intensity',
                                                     draw_rectangle)
      if ('intensity' %in% arr_comp) draw_arrow('intensity', draw_arrow)
      if ('intensity' %in% annot_comp) make_annot('intensity', annotate)
      if ('intensity' %in% line_comp) draw_lines('intensity', draw_lines)
      graphics::box()
    }
  }

  if (!suppressAxes) {
    if (!start_end_only) {
      xtix <- grDevices::axisTicks(c(round(min(t), 3),
                                     round(max(t), 3)), log=F)
      if (tUnit == 's') {
        xtixLab <- xtix
      } else {
        xtixLab <- xtix*1000
      }
      graphics::axis(1, at=xtix, labels=xtixLab, ...)
    } else {
      xtix <- c(round(min(t), 3), round(max(t), 3), 0)
      if (tUnit == 's') {
        xtixLab <- xtix
      } else {
        xtixLab <- xtix*1000
      }
      graphics::axis(1, at=xtix, labels=xtixLab, ...)
    }
  }

  if (!suppressAxes) graphics::mtext(time_axisLabel, side=1, line=3, outer=T,
                                     cex=0.8)
  graphics::mtext(mainTitle, side=3, line=2, adj=mainTitleAlignment, outer=T)

  graphics::par(p)
}

#' Draw spectral slice
#'
#' Generate and plot spectral slice from window around a specified time point
#' of a sound file.
#'
#' @param sound String giving the file name of a sound file with the .wav
#' extension.
#' @param time Time (in seconds) specifying the center of the window from which
#' to estimate spectrum.
#' @param channel Numeric giving the channel that should be used to
#' generate the spectrogram. Default is `1`.
#' @param freqRange Vector of two numbers giving the frequency range to be
#' used for plotting spectrograms. Default is `NULL`, in which case the whole
#' spectrum is plotted using the Nyquist frequency for the upper limit.
#' @param energyRange Either numeric or vector of two numbers used to specify
#' the y-axis range in units of dB/Hz. If a single number is passed, this is
#' interpreted as the desired dynamic range of the y-axis. If a vector of
#' two numbers is passed, these are used to exactly delimit the y-axis.
#' Default is `60`, i.e. a dynamic range of 60 dB/Hz relative to the global
#' maximum.
#' @param scale String giving the frequency scale to use. Default is `hz`.
#' Alternatives are `erb` (for the equivalent
#' rectangular bandwidth scale), `mel` (for the Mel scale, using 1000 Hz
#' as the corner frequency, following Fant 1968), or `logarithmic` (for
#' log Hz).
#' @param method String specifying the spectral estimation. Default is `fft`
#' (for the fast discrete Fourier transform). The only other option is
#' `multitaper`, in which case [multitaper::spec.mtm] is used to generate the
#' spectrum.
#' @param multitaper_args Optional named list of arguments passed on to
#' [multitaper::spec.mtm] if `method='multitaper'`.
#' @param windowLength Window length in seconds for generating spectra.
#' Default is `0.005`.
#' @param windowShape String giving the name of the window shape to be
#' applied to the signal when generating spectrograms. Default is `Gaussian`;
#' other options are `square`, `Hamming`, `Bartlett`, `Hanning`, `Blackman`, or
#' `Kaiser`.Note that the Gaussian window function provided by the `phonTools`
#' package and used for this function does not have the same properties as the
#' Gaussian window function used for spectral estimation in Praat.
#' @param color String giving the name of the color to be used for plotting
#' the spectrum. Default is `'black'`.
#' @param lineWidth Number giving the line width to use for plotting
#' the spectrum. Default is `1`.
#' @param freq_axisLabel String giving the name of the label to print along the
#' y-axis when plotting a spectrogram. Default is `NULL`, in which case the
#' axis label will depend on the scale.
#' @param energy_axisLabel String giving the name of the label to print along
#' the y-axis. Default is `Sound pressure level (dB/Hz)`.
#' @param mainTitle String giving a title to print at the top of the plot.
#' The default is an empty string, i.e. no title.
#' @param mainTitleAlignment Number indicating the vertical alignment of the
#' plot title, where `0` (default) indicates left-alignment, `1` indicates
#' right-alignment, `0.5` indicates central alignment, etc, following the
#' conventions of the `adj` argument of [graphics::mtext].
#' @param highlight Named list giving parameters for differential highlighting
#' of part of the plot based on the frequency domain. This list should contain
#' information about which parts of the plot to highlight, done with the
#' `start` and `end` arguments which must be numbers or numeric vectors.
#' Further contains the optional arguments `color`
#' (a string), `drawSize` (numeric), and `background`
#' (a string specifying a background color).
#' @param draw_lines Use for drawing straight lines. Takes
#' an argument of type `list` which should contain arguments to pass on to
#' [graphics::abline]. Should have a named argument `h` for horizontal lines,
#' or `v` for vertical lines, or `a`,`b` for the intercept and slope of the
#' line otherwise. Alternatively a nested list can be passed if more (sets of)
#' lines should be drawn. Default is `NULL`.
#' @param draw_rectangle Use for drawing rectangles. Should be a named
#' list containing arguments to pass on to [graphics::rect]. Can also be
#' multiple nested lists, if more rectangles should be drawn.
#' @param draw_arrow Use for drawing arrows on plot components. Should be a
#' named list containing arguments to pass on to [graphics::arrows].
#' Can also be multiple nested lists, if more rectangles should be drawn.
#' @param annotate Use for annotating plot components. Should be a
#' named list containing arguments to pass on to[graphics::text].
#' Can also be multiple nested lists, if more annotations should be added.
#' @param ... Further global plotting arguments passed on to `par()`.
#'
#' @return No return value, produces a figure.
#' @export
#'
#' @examples
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' draw_spectralslice(soundFile, time = 0.75)
draw_spectralslice <- function(sound, time,
                               channel = 1, freqRange = NULL, energyRange = 60,
                               scale = 'hz', method = 'fft',
                               multitaper_args = NULL,
                               windowLength = 0.005, windowShape = 'Gaussian',
                               color = 'black', lineWidth = 1,
                               freq_axisLabel = NULL,
                               energy_axisLabel = 'Sound pressure level (dB/Hz)',
                               mainTitle = '', mainTitleAlignment = 0,
                               highlight = NULL, draw_lines = NULL,
                               draw_rectangle = NULL, draw_arrow = NULL,
                               annotate = NULL, ...) {

  p <- graphics::par(no.readonly=TRUE)
  on.exit(graphics::par(p))

  if (!method %in% c('fft', 'multitaper')) stop(
    'method should be either fft or multitaper')

  if (scale == 'log') scale <- 'logarithmic'
  legal_scales <- c('hz', 'logarithmic', 'erb', 'mel')
  if (!scale %in% legal_scales) {
    stop('Possible scales are hz, logarithmic, erb, and mel')
  }
  if (is.null(freq_axisLabel)) {
    if (scale == 'hz') freq_axisLabel <- 'Frequency (Hz)'
    if (scale == 'logarithmic') freq_axisLabel <- 'Frequency (log Hz)'
    if (scale == 'mel') freq_axisLabel <- 'Frequency (mel)'
    if (scale == 'erb') freq_axisLabel <- 'Frequency (ERB)'
  }

  legal_ws <- c('square', 'Hamming', 'Bartlett', 'Hanning', 'Gaussian',
                'Blackman', 'Kaiser')
  if (!windowShape %in% legal_ws) {
    stop('Possible window shapes are square, Hamming, Bartlett, Hanning, ',
         'Gaussian, Kaiser, or Blackman.')
  }
  if (windowShape == 'square') {
    ws <- 'rectangular'
  } else if (windowShape == 'Hanning') {
    ws <- 'hann'
  } else {
    ws <- tolower(windowShape)
  }

  if (ws == 'kaiser') {
    winparam <- 3
  } else if (ws == 'gaussian') {
    winparam <- 0.4
  } else {
    winparam <- -1
  }

  log <- ifelse(scale == 'logarithmic', 'x', '')

  start <- time - (windowLength / 2)
  end <- time + (windowLength / 2)

  snd <- tuneR::readWave(sound, from=start, to=end, units='seconds', toWaveMC=T)

  sr <- snd@samp.rate
  bit <- snd@bit
  sig <- snd@.Data[,channel]

  if (scale == 'erb') {
    maxFreq <- soundgen::HzToOther(sr/2, scale = 'ERB')
  } else if (scale == 'mel') {
    maxFreq <- 1000 / log(2) * log(1 + (sr/2) / 1000)
  } else {
    maxFreq <- sr / 2
  }

  if (is.null(freqRange)) {
    freqRange <- c(0, maxFreq)
  }

  if (method == 'fft') {
    sig <- sig / (2^(bit - 1) - 1)
    hz <- seq(0, sr, length.out = (length(sig)*2))
    hz <- hz[hz <= sr / 2]
    windowed_sig <- sig * phonTools::windowfunc(length(sig), ws, winparam)
    padded_sig <- c(rep(0, length(sig)/2), windowed_sig, rep(0, length(sig)/2))
    spec <- stats::fft(padded_sig)
    psd <- (Mod(spec)^2) / (length(sig) * sr)
    psd <- psd[1:length(sig)]
    spec <- data.frame(hz = hz,
                       dB = 10 * log10(psd / 0.00002^2))
  } else {
    multitaper_args$timeSeries <- sig
    multitaper_args$plot <- FALSE
    multitaper_args$deltat <- 1 / sr
    multitaper_args$log <- 'dB'
    spec <- do.call(multitaper::spec.mtm, multitaper_args)
    spec <- data.frame(hz = spec$freq,
                       dB = 10 * log10(spec$spec / 0.00002))
  }

  if (scale == 'erb') {
    spec$freq <- soundgen::HzToOther(spec$hz, scale = 'ERB')
  } else if (scale == 'mel') {
    spec$freq <- 1000 / log(2) * log(1 + spec$hz / 1000)
  } else {
    spec$freq <- spec$hz
  }

  spec <- spec[which(spec$freq > freqRange[1] & spec$freq < freqRange[2]),]

  graphics::par(...)

  if (!is.null(highlight)) {
    highlight_f <- c()
    highlight_p <- c()
    for (int in 1:length(highlight$start)) {
      freqs <- which(spec$freq > highlight$start[int] &
                       spec$freq < highlight$end[int])
      extrema <- zoo::na.approx(c(NA, NA, spec$dB),
                                c(highlight$start[int], highlight$end[int],
                                  spec$hz))[1:2]
      highlight_f <- c(highlight_f, highlight$start[int],
                       spec$freq[freqs], highlight$end[int],
                       max(spec$freq[freqs] + 0.0001))
      highlight_p <- c(highlight_p, extrema[1], spec$dB[freqs],
                       extrema[2], NA)
    }
    if (!'color' %in% names(highlight)) highlight$color <- color
  }

  if (length(energyRange) == 1) {
    energyRange <- c(max(spec$dB) - energyRange, max(spec$dB))
  }

  plot(spec$freq, spec$dB, type = 'l', xlab = freq_axisLabel,
       ylab = energy_axisLabel, log = log, ylim = energyRange,
       col = color, lwd = lineWidth)
  graphics::mtext(mainTitle, side=3, line=2, adj=mainTitleAlignment)

  if (!is.null(highlight)) {
    if ('background' %in% names(highlight)) {
      graphics::rect(highlight$start,
                     energyRange[1] - energyRange[2] * 2,
                     highlight$end,
                     energyRange[2] + energyRange[2] * 2,
                     col = highlight$background, border = NA)
    }
    graphics::lines(highlight_f, highlight_p, col=highlight$color,
                    lwd=highlight$drawSize)
  }

  if (!is.null(draw_arrow)) {
    if (is.list(draw_arrow[[1]])) {
      for (i in 1:length(draw_arrow)) do.call(graphics::arrows, draw_arrow[[i]])
    } else {
      do.call(graphics::arrows, as.list(draw_arrow))
    }
  }

  if (!is.null(draw_rectangle)) {
    if (is.list(draw_rectangle[[1]])) {
      for (i in 1:length(draw_rectangle)) do.call(graphics::rect,
                                                  draw_rectangle[[i]])
    } else {
      do.call(graphics::rect, as.list(draw_rectangle))
    }
  }

  if (!is.null(draw_lines)) {
    if (is.list(draw_lines[[1]])) {
      for (i in 1:length(draw_lines)) do.call(graphics::abline, draw_lines[[i]])
    } else {
      do.call(graphics::abline, as.list(draw_lines))
    }
  }

  if (!is.null(annotate)) {
    if (is.list(annotate[[1]])) {
      for (i in 1:length(annotate)) do.call(graphics::text, annotate[[i]])
    } else {
      do.call(graphics::text, as.list(annotate))
    }
  }

  graphics::box()

}

ui <- bslib::page_sidebar(
  shinyjs::useShinyjs(),
  title = 'praatpicture: Praat-style plots of acoustic data in R',
  sidebar = bslib::sidebar(
    bslib::accordion(
      bslib::accordion_panel(
        'File uploads',
        fileInput('sound', 'Upload a sound file'),
        shinyjs::hidden(fileInput('tg_file', 'Upload a TextGrid file')),
      ),
      bslib::accordion_panel(
        'General appearance',
        numericInput('start', 'Start time (s)', 0, min = 0),
        numericInput('end', 'End time (s; 0 = entire file)', 0, min = 0),
        checkboxInput('tfrom0', 'Should time run from 0 on the x-axis?', TRUE),
        selectInput('tUnit', 'What should be the unit of time on the x-axis?',
                    choices = c('seconds' = 's', 'milliseconds' = 'ms')),
        selectInput('frames', 'Which frames should be plotted?',
                    choices = c('sound', 'spectrogram', 'TextGrid', 'pitch',
                                'formant', 'intensity'),
                    selected = c('sound', 'spectrogram'),
                    multiple = TRUE),
        textInput('proportion',
                  'Relative proportion of frames (comma-delimited)',
                  '0'),
        textInput('mainTitle', 'Title'),
        numericInput('mainTitleAlignment', 'Title alignment',
                     0, min = 0, max = 1),
        checkboxInput('start_end_only',
                      'Should only start and end times be shown on the x-axis?',
                      TRUE),
        checkboxInput('min_max_only',
                      'Should only minima and maxima be shown on the y-axis?',
                      TRUE),
        numericInput('speckleSize', 'Point size of speckled plot components',
                     1, min = 0),
        numericInput('drawSize', 'Line width of drawn plot components',
                     1, min = 0),
        textInput('time_axisLabel', 'x-axis label', 'Time (s)'),
        checkboxInput('suppressAxes',
                      'Should axes be fully suppressed?',
                      FALSE),
        numericInput('fontSize', 'Font size', 12),
        selectInput('fontFace', 'Font face',
                    choices = c('Default' = 1,
                                'Bold' = 2,
                                'Italic' = 3,
                                'Bold italic' = 4)),
        textInput('fontFamily', 'Font family', 'sans'),
        textInput('globalColor', 'Global color scheme', 'black'),
        textInput('bgColor', 'Background color', 'white'),
        numericInput('figWidth', 'Width (cm)', 15),
        numericInput('figHeight', 'Height (cm)', 10),
        numericInput('figDPI', 'Resolution (DPI)', 300)
      ),
      bslib::accordion_panel('Waveform',
                             shinyjs::hidden(textInput('wave_color',
                                                       'Color',
                                                       'black')),
                             shinyjs::hidden(textInput('wave_channels',
                                                       paste('Which channels should be plotted?',
                                                             '(comma-delimited)'),
                                                       'all')),
                             shinyjs::hidden(checkboxInput('wave_channelNames',
                                                           paste('Should channel names be writted next',
                                                                 'to waveform channels?'),
                                                           FALSE)),
                             shinyjs::hidden(numericInput('wave_axisDigits',
                                                          paste('How many digits should be used for',
                                                                'the waveform y-axis?'),
                                                          3)),
                             shinyjs::hidden(textInput('wave_channelNamesStr',
                                                       'Custom channel names (comma-delimited)',
                                                       NA)),
                             shinyjs::hidden(numericInput('wave_lineWidth',
                                                          'Line width',
                                                          1)),
                             shinyjs::hidden(checkboxInput('pitch_plotOnWave',
                                                           'Pitch overlay',
                                                           FALSE)),
                             shinyjs::hidden(checkboxInput('intensity_plotOnWave',
                                                           'Intensity overlay',
                                                           FALSE))
      ),
      bslib::accordion_panel(
        'Spectrogram',
        shinyjs::hidden(numericInput('spec_freqRangeMin',
                                     'Minimum frequency (Hz)',
                                     0)),
        shinyjs::hidden(numericInput('spec_freqRangeMax',
                                     'Maximum frequency (Hz)',
                                     5000)),
        shinyjs::hidden(numericInput('spec_channel',
                                     'Channel used to derive spectrogram',
                                     NA)),
        shinyjs::hidden(numericInput('spec_windowLength',
                                     'Length of analysis window (s)',
                                     0.005)),
        shinyjs::hidden(numericInput('spec_dynamicRange',
                                     'Dynamic range (dB)',
                                     50)),
        shinyjs::hidden(numericInput('spec_timeStep',
                                     'Number of time steps / frames',
                                     1000)),
        shinyjs::hidden(selectInput('spec_windowShape', 'Window shape',
                                    choices = c('Gaussian', 'Hamming', 'Hanning',
                                                'Bartlett', 'square', 'Blackman'),
                                    selected = 'Gaussian')),
        shinyjs::hidden(textInput('spec_colors', 'Colors (comma-delimited)',
                                  'white,black'),
                        textInput('spec_axisLabel', 'y-axis label',
                                  'Frequency (Hz)')),
        shinyjs::hidden(checkboxInput('pitch_plotOnSpec',
                                      'Pitch overlay',
                                      FALSE)),
        shinyjs::hidden(checkboxInput('formant_plotOnSpec',
                                      'Formant overlay',
                                      FALSE)),
        shinyjs::hidden(checkboxInput('intensity_plotOnSpec',
                                      'Intensity overlay',
                                      FALSE))
      ),
      bslib::accordion_panel(
        'TextGrid',
        shinyjs::hidden(textInput('tg_tiers',
                                  paste('Which tiers should be shown?',
                                        '(comma-delimited)'),
                                  'all')),
        shinyjs::hidden(textInput('tg_focusTier',
                                  paste('Which tier(s) are in focus?',
                                        '(comma-delimited)'),
                                  'all')),
        shinyjs::hidden(textInput('tg_focusTierColor',
                                  paste('Which color(s) should be used for the',
                                        'focus tier(s)? (comma-delimited)'),
                                  'black')),
        shinyjs::hidden(textInput('tg_focusTierLineType',
                                  paste('Which line type(s) should be used for the',
                                        'focus tier(s)? (comma-delimited)'),
                                  'dotted')),
        shinyjs::hidden(checkboxInput('tg_tierNames',
                                      'Should tier names be printed along the x-axis?',
                                      TRUE)),
        shinyjs::hidden(textInput('tg_alignment',
                                  paste('How should text in the tier(s) be aligned?',
                                        '(comma-delimited)'),
                                  'central')),
        shinyjs::hidden(checkboxInput('tg_specialChar',
                                      'Should Praat-style special typesetting be used?',
                                      FALSE)),
        shinyjs::hidden(textInput('tg_color',
                                  paste('Which color(s) should be used for text in the',
                                        'tier(s)?'),
                                  'black'))
      ),
      bslib::accordion_panel(
        'Pitch',
        shinyjs::hidden(selectizeInput('pitch_plotType',
                                       'Plot type',
                                       choices = c('draw', 'speckle'),
                                       selected = 'draw',
                                       multiple = TRUE,
                                       options = list(create = TRUE))),
        shinyjs::hidden(selectInput('pitch_scale',
                                    'Frequency scale',
                                    choices = c('hz', 'logarithmic', 'semitones',
                                                'erb', 'mel'),
                                    selected = 'hz')),
        shinyjs::hidden(numericInput('pitch_semitonesRe',
                                     'Reference frequency for semitone conversion (Hz)',
                                     100)),
        shinyjs::hidden(numericInput('pitch_freqRangeMin',
                                     'Minimum frequency to show (scale-dependent)',
                                     50)),
        shinyjs::hidden(numericInput('pitch_freqRangeMax',
                                     'Maximum frequency to show (scale-dependent)',
                                     500)),
        shinyjs::hidden(numericInput('pitch_timeStep',
                                     'Time step / frame shift (s)',
                                     0.01)),
        shinyjs::hidden(numericInput('pitch_floor',
                                     'Floor frequency (Hz)',
                                     75)),
        shinyjs::hidden(numericInput('pitch_ceiling',
                                     'Ceiling frequency (Hz)',
                                     600)),
        shinyjs::hidden(textInput('pitch_color',
                                  'Color',
                                  'black')),
        shinyjs::hidden(textInput('pitch_axisLabel',
                                  'y-axis label',
                                  'Frequency (Hz)'))
      ),
      bslib::accordion_panel(
        'Formants',
        shinyjs::hidden(selectizeInput('formant_plotType',
                                       'Plot type',
                                       choices = c('draw', 'speckle'),
                                       selected = 'speckle',
                                       multiple = TRUE,
                                       options = list(create = TRUE))),
        shinyjs::hidden(numericInput('formant_freqRangeMin',
                                     'Minimum frequency to show (Hz)',
                                     0)),
        shinyjs::hidden(numericInput('formant_freqRangeMax',
                                     'Maximum frequency to show (Hz)',
                                     5500)),
        shinyjs::hidden(numericInput('formant_dynamicRange',
                                     'Dynamic range (dB)',
                                     30)),
        shinyjs::hidden(numericInput('formant_timeStep',
                                     'Time step / window shift (s)',
                                     0.00625)),
        shinyjs::hidden(numericInput('formant_windowLength',
                                     'Length of analysis window (s)',
                                     0.025)),
        shinyjs::hidden(numericInput('formant_maxN',
                                     'Maximum number of formants to estimate',
                                     5)),
        shinyjs::hidden(numericInput('formant_number',
                                     'Number of formants to plot',
                                     5)),
        shinyjs::hidden(textInput('formant_color',
                                  'Colors (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('formant_axisLabel',
                                  'y-axis label',
                                  'Frequency (Hz)'))
      ),
      bslib::accordion_panel(
        'Intensity',
        shinyjs::hidden(numericInput('intensity_rangeMin',
                                     'Minimum amplitude (dB)',
                                     NA)),
        shinyjs::hidden(numericInput('intensity_rangeMax',
                                     'Maximum amplitude (dB)',
                                     NA)),
        shinyjs::hidden(numericInput('intensity_minPitch',
                                     'Minimum pitch (Hz)',
                                     100)),
        shinyjs::hidden(textInput('intensity_color',
                                  'Color',
                                  'black')),
        shinyjs::hidden(textInput('intensity_axisLabel',
                                  'Axis label',
                                  'Intensity (dB)'))
      ),
      bslib::accordion_panel(
        'Highlighting',
        checkboxInput('highlightBool',
                      'Differential highlighting of parts of signals?',
                      FALSE),
        shinyjs::hidden(selectInput('highlightComp',
                                    'Which signal to highlight',
                                    choices = c('all', 'wave', 'spectrogram',
                                                'pitch', 'formant', 'intensity'),
                                    selected = 'all')),
        shinyjs::hidden(textInput('highlightStart',
                                  'Start time(s) (comma-separated)',
                                  '0')),
        shinyjs::hidden(textInput('highlightEnd',
                                  'End time(s) (comma-separated)',
                                  '0.1')),
        shinyjs::hidden(checkboxInput('highlightTGbool',
                                      'Use TextGrid information?',
                                      FALSE)),
        shinyjs::hidden(textInput('highlightTier',
                                  'TextGrid tier')),
        shinyjs::hidden(textInput('highlightLabel',
                                  'Label(s) to match in TextGrid tier')),
        shinyjs::hidden(textInput('highlightCol',
                                  'Color(s) (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('highlightBG',
                                  'Background color',
                                  'white')),
        shinyjs::hidden(textInput('highlightDrawSize',
                                  'Draw size')),
        shinyjs::hidden(textInput('highlightSpeckleSize',
                                  'Speckle size')),
        checkboxInput('arrowBool', 'Add arrow(s)?'),
        shinyjs::hidden(textInput('arrowComp',
                                  paste('Which plot component(s) should arrow(s) be',
                                        'drawn on? (comma-separated)'),
                                  'sound')),
        shinyjs::hidden(textInput('arrowx0',
                                  paste('Where should the arrow(s) start on the',
                                        'x-axis? (comma-separated)'),
                                  '0')),
        shinyjs::hidden(textInput('arrowx1',
                                  paste('Where should the arrow(s) end on the',
                                        'x-axis? (comma-separated)'),
                                  '0.5')),
        shinyjs::hidden(textInput('arrowy0',
                                  paste('Where should the arrow(s) start on the',
                                        'y-axis? (comma-separated)'),
                                  '0')),
        shinyjs::hidden(textInput('arrowy1',
                                  paste('Where should the arrow(s) end on the',
                                        'y-axis? (comma-separated)'),
                                  '0.1')),
        shinyjs::hidden(textInput('arrowLength',
                                  paste('What should be the length of the arrow',
                                        'head(s) (in inches)?'),
                                  '0.25')),
        shinyjs::hidden(textInput('arrowAngle',
                                  paste('What should be the angle of the arrow',
                                        'head(s) (in degrees)?'),
                                  '30')),
        shinyjs::hidden(textInput('arrowColor',
                                  'Color(s) (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('arrowLineType',
                                  'Line type(s) (comma-separated)',
                                  'solid')),
        shinyjs::hidden(textInput('arrowLineWidth',
                                  'Line width(s) (comma-separated)',
                                  '1')),
        checkboxInput('rectBool', 'Add rectangle(s)?', FALSE),
        shinyjs::hidden(textInput('rectComp',
                                  paste('Which plot component(s) should rectangle(s) be',
                                        'drawn on? (comma-separated)'),
                                  'sound')),
        shinyjs::hidden(textInput('rectxleft',
                                  paste('Where should the left side be on the x-axis?',
                                        '(comma-separated)'),
                                  '0')),
        shinyjs::hidden(textInput('rectybottom',
                                  paste('Where should the lower side be on the y-axis?',
                                        '(comma-separated)'),
                                  '-0.05')),
        shinyjs::hidden(textInput('rectxright',
                                  paste('Where should the right side be on the x-axis?',
                                        'y-axis? (comma-separated)'),
                                  '0.5')),
        shinyjs::hidden(textInput('rectytop',
                                  paste('Where should the top side be on the y-axis',
                                        '(comma-separated)'),
                                  '0.05')),
        shinyjs::hidden(textInput('rectFill',
                                  'Fill color(s) (comma-separated)',
                                  NA)),
        shinyjs::hidden(textInput('rectColor',
                                  'Line color(s) (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('rectLineType',
                                  'Line type(s) (comma-separated)',
                                  'solid')),
        shinyjs::hidden(textInput('rectLineWidth',
                                  'Line width(s) (comma-separated)',
                                  '1')),
        checkboxInput('annotBool', 'Add annotation(s)?', FALSE),
        shinyjs::hidden(textInput('annotComp',
                                  paste('Which plot component(s) should be annotated?',
                                        '(comma-separated)'),
                                  'sound')),
        shinyjs::hidden(textInput('annotLabel',
                                  'Label(s) (semicolon-separated)',
                                  'TEXT')),
        shinyjs::hidden(textInput('annotx',
                                  'Position(s) on the x-axis (comma-separated)',
                                  0)),
        shinyjs::hidden(textInput('annoty',
                                  'Position(s) on the y-axis (comma-separated)',
                                  0.1)),
        shinyjs::hidden(textInput('annotColor',
                                  'Color(s) (comma-separated)',
                                  'black')),
        shinyjs::hidden(textInput('annotFace',
                                  'Font face(s) (comma-separated)',
                                  '1')),
        shinyjs::hidden(textInput('annotSize',
                                  'Font size(s) (comma-separated)',
                                  '1'))
      )
    )
  ),
  downloadButton('download'),
  imageOutput('plot')
)

#' Make Praat Picture style plots in EMU
#'
#' Generate plots in the style of Praat Pictures from sound files and annotations
#' in an EMU database.
#'
#' @param db_handle The handle of an EMU database loaded into R.
#' @param session String giving the name of the session where the sound file
#' to plot is located.
#' Default is `0000`.
#' @param bundle String giving the name of the bundle with the sound file to plot.
#' @param pitch_ssffExt String giving the file extension for an SSFF track with
#' pitch data to plot. Default is `NULL`.
#' @param formant_ssffExt String giving the file extension for an SSFF track
#' with formant data to plot. Default is `NULL`.
#' @param intensity_ssffExt
#' String giving the file extension for an SSFF track with intensity data to
#' plot. Default is `NULL`.
#' @param talking Boolean; should a video be created with embedded audio, as
#' when using [talking_praatpicture]? Default is `FALSE`.
#' @param ... Further arguments passed to [praatpicture] (or
#' [talking_praatpicture] if `talking = TRUE`).
#'
#' @seealso See [praatpicture] for more details on how to customize plots and
#' [talking_praatpicture] for more details on how to customize videos.
#' @return No return value, produces a plot or a video.
#' @export
#'
#' @examples
#' \donttest{
#' # Create demo data and load demo database
#' emuR::create_emuRdemoData(tempdir())
#' db_path <- paste0(tempdir(), '/emuR_demoData/ae_emuDB')
#' db <- emuR::load_emuDB(db_path)
#'
#' emuR::list_bundles(db)
#' emupicture(db, bundle='msajc003', tg_tiers=c('Text', 'Tone'))
#'
#' # Plot SSFF track data
#'
#' emuR::list_ssffTrackDefinitions(db)
#' emupicture(db, bundle='msajc003', frames=c('sound', 'formant'),
#' proportion=c(30,70), formant_ssffExt='fms', formant_number=4)
#' }
emupicture <- function(db_handle, session='0000', bundle,
                       pitch_ssffExt=NULL, formant_ssffExt=NULL,
                       intensity_ssffExt=NULL, talking=FALSE, ...) {
  tg <- FALSE
  args <- list(...)
  if ('frames' %in% names(args) & 'TextGrid' %in% args$frames) tg <- TRUE
  if (!'frames' %in% names(args)) tg <- TRUE
  if ('tg_tiers' %in% names(args)) {
    attrNames <- args$tg_tiers
  } else {
    attrNames <- NULL
  }
  if (tg) {
    emuR::export_TextGridCollection(db_handle, targetDir='.',
                                    sessionPattern=session,
                                    bundlePattern=bundle,
                                    attributeDefinitionNames = attrNames,
                                    verbose=F)
    fn <- paste0(session, '/', bundle, '.wav')
  } else {
    fn <- paste0(db_handle$basePath, '/', session, '_ses/', bundle, '_bndl/',
                 bundle, '.wav')
  }

  if (!is.null(pitch_ssffExt)) {
    pfn <- paste0(db_handle$basePath, '/', session, '_ses/', bundle, '_bndl/',
                  bundle, '.', pitch_ssffExt)
    wpt <- wrassp::read.AsspDataObj(pfn)
  } else {
    wpt <- NULL
  }
  if (!is.null(formant_ssffExt)) {
    ffn <- paste0(db_handle$basePath, '/', session, '_ses/', bundle, '_bndl/',
                  bundle, '.', formant_ssffExt)
    wft <- wrassp::read.AsspDataObj(ffn)
  } else {
    wft <- NULL
  }
  if (!is.null(intensity_ssffExt)) {
    ifn <- paste0(db_handle$basePath, '/', session, '_ses/', bundle, '_bndl/',
                  bundle, '.', intensity_ssffExt)
    wit <- wrassp::read.AsspDataObj(ifn)
  } else {
    wit <- NULL
  }

  if (talking) {
    talking_praatpicture(fn, tg_specialChar=FALSE,
                         pitch_ssff=wpt, formant_ssff=wft, intensity_ssff=wit,
                         ...)
  } else {
    praatpicture(fn, tg_specialChar=FALSE,
                 pitch_ssff=wpt, formant_ssff=wft, intensity_ssff=wit, ...)
  }

  if (tg) unlink(session, recursive=T)
}

#' Convert Praat font styles to R font styles
#'
#' Helper function for converting Praat font styles such as italics, bold,
#' and small caps into expressions that can be read by base R plots.
#' Instead of using
#' this function directly, just use
#' `praatpicture('my_sound_file', frames='TextGrid', tg_specialChar=TRUE)`.
#'
#' @param lab A string or vector of strings with labels from a TextGrid.
#'
#' @return A list with elements of class `expression`.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#'
#' # With stylized text
#' praatpicture(soundFile, frames='TextGrid')
#'
#' # Without stylized text
#' praatpicture(soundFile, frames='TextGrid', tg_specialChar=FALSE)
tg_stylize <- function(lab) {
  x <- list()

  for (l in 1:length(lab)) {
    if (lab[l] != '') {
      if (grepl('\\\\s', lab[l])) {
        s <- c()
        splsc <- unlist(strsplit(lab[l], split='\\\\s'))
        for (i in 2:length(splsc)) {
          tmp <- unlist(strsplit(splsc[i], split='}', perl=T))
          tmp[1] <- conv2sc(tmp[1])
          tmp[1] <- sub('{', '', tmp[1], perl=T)
          s <- c(s, paste(tmp, collapse=''))
        }
        s <- paste(s, collapse='')
        lab[l] <- paste(splsc[1], s, collapse='', sep='')
      }

      spl <- unlist(strsplit(lab[l], split='(?<=.)(?=[\\\\#%^_]{2})', perl=T))
      s <- c()

      for (i in 1:length(spl)) {
        ch1 <- substr(spl[i], 1, 1)
        if (!is.na(spl[i]) & grepl('[#%^_]{2}', spl[i])) {
          if (substr(spl[i], 1, 2) != '#%') {
            tmp <- unlist(strsplit(spl[i], split=paste0('(?<=.)(?=', ch1, ')'), perl=T))
            tmp[1] <- paste0(tmp[1], tmp[2])
            tmp <- tmp[-2]
            if (length(tmp) > 1) tmp[2] <- sub(ch1, '', tmp[2])
            s <- c(s, tmp)
          } else {
            s <- c(s, spl[i])
          }
        } else if (!is.na(ch1) & ch1 == '\\') {
          tmp <- unlist(strsplit(spl[i], split='(?=.)(?=[#%^_])', perl=T))
          tmp[1] <- paste0(tmp[1], tmp[2])
          tmp <- tmp[-2]
          s <- c(s, tmp)
        } else {
          s <- c(s, spl[i])
        }
      }

      spl <- s
      s <- c()
      for (i in 1:length(spl)) {
        if (!grepl('[\\\\#%^_]{2}', spl[i]) | substr(spl[i], 1, 2) == '#%') {
          tmp <- unlist(strsplit(spl[i], split=''))
          sch <- rev(grep('[%#^_]', tmp))
          for (j in sch) {
            tmp[j] <- paste0(tmp[j], tmp[j+1])
            tmp <- tmp[-(j+1)]
          }
          s <- c(s, tmp)
        } else if (substr(spl[i], 1, 1) == '\\') {
          s <- c(s, spl[i])
        } else {
          tmp <- substr(spl[i], 2, nchar(spl[i]))
          s <- c(s, tmp)
        }
      }
      spl <- s

      print_text <- NULL
      for (i in 1:length(spl)) {
        if (!grepl('[%#^_]', spl[i])) pr <- bquote(.(spl[i]))
        clean <- sub('[%#^_]*', '', spl[i])
        if (grepl('#%', spl[i])) {
          pr <- bquote(bolditalic(.(clean)))
        } else if (grepl('%', spl[i])) {
          pr <- bquote(italic(.(clean)))
        } else if (grepl('#', spl[i])) {
          pr <- bquote(bold(.(clean)))
        }
        if (grepl('\\^', spl[i])) pr <- bquote(''^.(clean))
        if (grepl('_', spl[i])) pr <- bquote(''[.(clean)])
        if (grepl('\\\\', spl[i])) {
          pr <- bquote(.(substr(spl[i], nchar(spl[i]), nchar(spl[i]))))
        }
        print_text <- bquote(.(print_text)*.(pr))
      }

      x[[l]] <- print_text

    } else {
      x[[l]] <- ''
    }
  }


  return(x)
}

#' Run praatpicture as Shiny app
#'
#' Interactive version of [praatpicture]
#'
#' @return No return values
#' @export
#'
#' @examples
#' \dontrun{
#' shiny_praatpicture()
#' }
shiny_praatpicture <- function() {
  appDir <- system.file('shiny', 'shiny_praatpicture', package='praatpicture')
  shiny::runApp(appDir, display.mode = 'normal')
}

#' Plot formant object
#'
#' Function for plotting formant objects called by [praatpicture]. Instead of
#' using this function directly, just use
#' `praatpicture('my_sound_file', frames='formant')`.
#'
#' @param fm Formant object loaded using [rPraat::formant.read] or similar
#' object, i.e. a `list` object containing the elements `t` (time),
#' `frequencyArray` (array or formant frequencies with number of rows
#' corresponding to the length of `t`), `maxnFormants` (integer giving the
#' number of formants), `intensityVector` (vector of intensity values of
#' the same length as `t`), and `conv2db` (logical).
#' @param start Start time (in seconds) of desired plotted area.
#' @param end End time (in seconds) of desired plotted area.
#' @param tfrom0 Logical; should time on the x-axis run from 0 or from the
#' original time? Default is `TRUE`.
#' @param tgbool Logical; should dotted lines be plotted corresponding to
#' locations in a TextGrid? Default is `FALSE`.
#' @param lines Numeric vector giving locations in seconds of locations from
#' a TextGrid to be plotted with dotted lines. Default is `NULL`.
#' @param focusTierColor String or vector of strings giving the color(s) to
#' use for plotting focus tier lines. If multiple tiers are focused, a vector
#' of the same length can be passed, and the nth tier will be plotted in the
#' nth color. Default is `'black'`.
#' @param focusTierLineType String or vector of strings giving the line
#' type(s) for plotting focus tier lines. If multiple tiers are focused, a
#' vector of the same length can be passed, and the nth tier will be plotted in
#' the nth line type. Default is `'dotted'`.
#' @param dynamicRange Dynamic range in dB for producing formant plots.
#' When a formant plot of `plotType='speckle'` is drawn, no formants are
#' shown in frames with intensity level `dynamicRange` below the maximum
#' intensity. Default is `30`. If set to `0`, all formants are shown.
#' @param freqRange Vector of two integers giving the frequency range to be
#' used for producing formant plots. Default is `c(0,5500)`.
#' @param plotType String giving the type of formant plot to produce;
#' default is `speckle` (a point plot), the only other option is `draw` (a line
#' plot). Alternatively a vector `c('draw','speckle')` can be passed, in which
#' case both are used.
#' @param color String or vector of strings giving the name(s) of
#' colors to be used for plotting formants. If one color is provided, all
#' formants will be plotted in this color. If multiple colors are provided,
#' different formants will be shown in different colors. Default is `'black'`.
#' @param ind Integer indexing formants relative to other plot components.
#' Default is `NULL`.
#' @param min_max_only Logical; should only minimum and maximum values be given
#' on the y-axis? Default is `FALSE`. Can also be a logical vector if some but
#' not all plot components should have minimum and maximum values on the y-axis.
#' Ignored for TextGrid component.
#' @param highlight Named list giving parameters for differential
#' highlighting of formants based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `color`),
#' `drawSize` or `speckleSize` (both numeric), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param axisLabel String giving the name of the label to print along the
#' y-axis when plotting formants. Default is `Frequency (Hz)`.
#' @param drawSize Number indicating the line width if
#' `plotType` is `'draw'`. Default is `1`. Controls the `lwd` argument of
#' [graphics::lines].
#' @param speckleSize Number indicating the point size of if `_plotType` is
#' `'speckle'`. Default is `1`. Controls the `cex` arguments of
#' [graphics::points].
#' @param suppressAxes Should all axes and the box around plots be suppressed?
#' Default is `FALSE`. Generally not recommended.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, frames='formant')
formantplot <- function(fm, start, end, tfrom0=TRUE, tgbool=FALSE, lines=NULL,
                        focusTierColor='black', focusTierLineType='dotted',
                        dynamicRange=30, freqRange=c(0,5500),
                        plotType='speckle', color='black',
                        ind=NULL, min_max_only=FALSE, highlight=NULL,
                        axisLabel='Frequency (Hz)', drawSize=1, speckleSize=1,
                        suppressAxes=FALSE) {

  if (!min_max_only[ind]) {
    if (ind == 1) {
      yax <- 's'
    } else {
      ytix <- grDevices::axisTicks(freqRange, log=F)
      ytix <- ytix[-length(ytix)]
      yax <- 'n'
    }
  } else {
    yax <- 'n'
    ytix <- freqRange
  }

  nf <- fm$maxnFormants
  if (length(color) == 1) color <- rep(color, nf)

  if (tfrom0) {
    fm$t <- fm$t - start
    start <- 0
  }

  if (fm$conv2db) {
    db <- gsignal::pow2db(fm$intensityVector)
  } else {
    db <- fm$intensityVector
  }
  if (dynamicRange != 0) {
    subdr <- which(db < max(db)-dynamicRange)
    if (length(subdr) == 0) subdr <- 1
  } else {
    subdr <- 1
  }

  create_highlighting <- function(highlight, fm, nf, dynamicRange) {
    hl_t <- 0
    if (nf > 1) {
      hl_f <- array(rep(NA, nf), dim = c(nf,1))
    } else {
      hl_f <- NA
    }
    hl_i <- c()
    for (int in 1:length(highlight$start)) {
      times <- which(fm$t > highlight$start[int] & fm$t < highlight$end[int])
      hl_t <- c(hl_t, fm$t[times], max(fm$t[times] + 0.0001))
      if (nf > 1) {
        hl_f <- cbind(hl_f, fm$frequencyArray[,times], rep(NA, nf))
      } else {
        hl_f <- c(hl_f, fm$frequencyArray[times], NA)
      }
      hl_i <- c(hl_i, db[times], 0)
    }
    if (any(highlight$start < start)) {
      highlight$start[which(highlight$start < start)] <- start
    }
    if (any(highlight$end > end)) {
      highlight$end[which(highlight$end > end)] <- end
    }

    if (dynamicRange != 0) {
      hsubdr <- which(hl_i < max(db)-dynamicRange)
      if (length(hsubdr) == 0) hsubdr <- 1
    } else {
      hsubdr <- 1
    }

    if (!'color' %in% names(highlight)) highlight$color <- color
    if (!'drawSize' %in% names(highlight)) highlight$drawSize <- drawSize
    if (!'speckleSize' %in% names(highlight)) highlight$speckleSize <-
      speckleSize
    if (length(highlight$color) == 1) highlight$color <-
      rep(highlight$color, nf)
    return(list(params = highlight, hl_t = hl_t, hl_f = hl_f, subdr = hsubdr))
  }

  s <- freqRange[1]:freqRange[2]
  freql <- s[s %% 1000 == 0]

  if ('draw' %in% plotType) {
    if (nf > 1) {
      plot(fm$t, fm$frequencyArray[1,], xlim=c(start, end+start),
           xaxt='n', ylim=freqRange, yaxt=yax, type='l', col=color[1],
           lwd=drawSize)

      for (i in 2:nf) {
        graphics::lines(fm$t, fm$frequencyArray[i,], col=color[i], lwd=drawSize)
      }
    } else {
      plot(fm$t, fm$frequencyArray, xlim=c(start, end+start),
           xaxt='n', ylim=freqRange, yaxt=yax, type='l', col=color[1],
           lwd=drawSize)
    }
    if (!suppressAxes) {
      if (!min_max_only[ind] & ind != 1) graphics::axis(2, at=ytix)
      if (min_max_only[ind]) graphics::axis(2, at=ytix, padj=c(0,1), las=2,
                                            tick=F)
    }

    if ('speckle' %in% plotType) {
      if (nf > 1) {
        graphics::points(fm$t[-subdr], fm$frequencyArray[1,-subdr], pch=20,
                         col=color[1], cex=speckleSize)
        for (i in 2:nf) {
          graphics::points(fm$t[-subdr], fm$frequencyArray[i,-subdr], pch=20,
                           col=color[i], cex=speckleSize)
        }
      } else {
        graphics::points(fm$t[-subdr], fm$frequencyArray[-subdr], pch=20,
                         col=color[1], cex=speckleSize)
      }

    }

    if (!is.null(highlight)) {
      if (any(sapply(highlight, is.list))) {
        highlightLoopDepth <- sum(sapply(highlight, is.list))
      } else {
        highlightLoopDepth <- 1
      }
      for (i in 1:highlightLoopDepth) {
        if (any(sapply(highlight, is.list))) {
          thisHL <- create_highlighting(highlight[[i]], fm, nf, dynamicRange)
        } else {
          thisHL <- create_highlighting(highlight, fm, nf, dynamicRange)
        }
        if ('background' %in% names(thisHL$params)) {
          graphics::rect(thisHL$params$start,
                         freqRange[1] - freqRange[2] * 2,
                         thisHL$params$end,
                         freqRange[2] + freqRange[2] * 2,
                         col = thisHL$params$background, border = NA)
        }
        if (nf > 1) {
          graphics::lines(thisHL$hl_t, thisHL$hl_f[1,],
                          col=thisHL$params$color[1],
                          lwd=thisHL$params$drawSize)
          for (j in 2:nf) {
            graphics::lines(thisHL$hl_t, thisHL$hl_f[j,],
                            col=thisHL$params$color[j],
                            lwd=thisHL$params$drawSize)
          }
        } else {
          graphics::lines(thisHL$hl_t, thisHL$hl_f, col=thisHL$params$color[1],
                          lwd=thisHL$params$drawSize)
        }

        if ('speckle' %in% plotType) {
          if (nf > 1) {
            graphics::points(thisHL$hl_t[-thisHL$subdr],
                             thisHL$hl_f[1,-thisHL$subdr],
                             pch=20, col=thisHL$params$color[1],
                             cex=thisHL$params$speckleSize)
            for (j in 2:nf) {
              graphics::points(thisHL$hl_t[-thisHL$subdr],
                               thisHL$hl_f[j,-thisHL$subdr],
                               pch=20, col=thisHL$params$color[j],
                               cex=thisHL$params$speckleSize)
            }
          } else {
            graphics::points(thisHL$hl_t[-thisHL$subdr],
                             thisHL$hl_f[-thisHL$subdr], pch=20,
                             col=thisHL$params$color[1],
                             cex=thisHL$params$speckleSize)
          }
        }
      }
    }

    if (tgbool) {
      for (i in 1:length(lines)) {
        graphics::abline(v=lines[[i]], col=focusTierColor[i],
                         lty=focusTierLineType[i])
      }
    }

  }

  if (length(plotType) == 1) {
    if (plotType == 'speckle') {
      if (nf > 1) {
        plot(fm$t[-subdr], fm$frequencyArray[1,-subdr], pch=20,
             xlim=c(start, end+start), xaxt='n',
             ylim=freqRange, yaxt=yax, col=color[1], cex=speckleSize)
        for (i in 2:nf) {
          graphics::points(fm$t[-subdr], fm$frequencyArray[i,-subdr], pch=20,
                           col=color[i], cex=speckleSize)
        }
      } else {
        plot(fm$t[-subdr], fm$frequencyArray[-subdr], pch=20,
             xlim=c(start, end+start), xaxt='n',
             ylim=freqRange, yaxt=yax, col=color[1], cex=speckleSize)
      }

      if (!suppressAxes) {
        if (!min_max_only[ind] & ind != 1) graphics::axis(2, at=ytix)
        if (min_max_only[ind]) graphics::axis(2, at=ytix, padj=c(0,1), las=2,
                                              tick=F)
      }

      if (!is.null(highlight)) {
        if (any(sapply(highlight, is.list))) {
          highlightLoopDepth <- sum(sapply(highlight, is.list))
        } else {
          highlightLoopDepth <- 1
        }
        for (i in 1:highlightLoopDepth) {
          if (any(sapply(highlight, is.list))) {
            thisHL <- create_highlighting(highlight[[i]], fm, nf, dynamicRange)
          } else {
            thisHL <- create_highlighting(highlight, fm, nf, dynamicRange)
          }
          if ('background' %in% names(thisHL$params)) {
            graphics::rect(thisHL$params$start,
                           freqRange[1] - freqRange[2] * 2,
                           thisHL$params$end,
                           freqRange[2] + freqRange[2] * 2,
                           col = thisHL$params$background, border = NA)
          }
          if (nf > 1) {
            graphics::points(thisHL$hl_t[-thisHL$subdr],
                             thisHL$hl_f[1,-thisHL$subdr],
                             pch=20, col=thisHL$params$color[1],
                             cex=thisHL$params$speckleSize)
            for (j in 2:nf) {
              graphics::points(thisHL$hl_t[-thisHL$subdr],
                               thisHL$hl_f[j,-thisHL$subdr],
                               pch=20, col=thisHL$params$color[j],
                               cex=thisHL$params$speckleSize)
            }
          } else {
            graphics::points(thisHL$hl_t[-thisHL$subdr],
                             thisHL$hl_f[-thisHL$subdr], pch=20,
                             col=thisHL$params$color[1],
                             cex=thisHL$params$speckleSize)
          }
        }


      }

      if (tgbool) {
        for (i in 1:length(lines)) {
          graphics::abline(v=lines[[i]], col=focusTierColor[i],
                           lty=focusTierLineType[i])
        }
      }

    }
  }

  if (!suppressAxes) graphics::mtext(axisLabel, side=2, line=3.5, cex=0.8)

}

#' Make animations from Praat Picture-style plots of acoustic data
#'
#' Animate some aspect of a Praat Picture-style plot
#' of acoustic data, potentially aligned with transcriptions.
#'
#' @param sound String giving the file name of a sound file with the .wav
#' extension.
#' @param width Number giving the desired width of the resulting animation in
#' pixels; default is `1080`.
#' @param height Number giving the desired height of the resulting animation in
#' pixels; default is `720`.
#' @param frameRate Number giving the desired frame rate of the resulting
#' animation in Hz; default is `24`, i.e. 24 frames per second.
#' @param n_frames Number giving the desired number of frames of the resulting
#' animation; default is `50`.
#' @param loop Logical; should the animation be looped? Default is `TRUE`.
#' Ignored when `outputType` is `mp4`.
#' @param outputFile String giving the desired file name of the animation.
#' Default is `NULL`, in which case GIF files are named `praatgif.gif` and
#' MP4 files are named `praatvid.mp4`. If you choose a different name, make
#' sure that the file extension matches the selected `outputType`.
#' @param outputFormat String giving the desired file type; default is `gif`,
#' the only other option is `mp4`.S
#' @param useViewer Logical; should the animation be shown in the Viewer pane in
#' RStudio? Default is `TRUE`; if true, the animation is only saved in a
#' temporary directory, but can be downloaded from a browser.
#' @param verbose Logical; should status messages be printed in the console as
#' figures are being generated? Default is `TRUE`.
#' @param pointsize Number; which point size should be used for text in the
#' animation? Default is `25`. See [grDevices::png()] for more details.
#' @param start Start time (in seconds) of desired plotted area. Default is `0`.
#' Alternatively, a vector giving the first and last start time in the
#' animation.
#' @param end End time (in seconds) of desired plotted area. Default is `0`
#' (= the entire file). Alternatively, a vector giving the first and last
#' end time in the animation.
#' @param spec_freqRange Vector of two integers giving the frequency range to be
#' used for plotting spectrograms. Default is `c(0,5000)`. Alternatively, a
#' vector of four integers giving the first and last lowest frequency, followed
#' by the first and last highest frequency in the animation; i.e.,
#' `c(0,0,5000,10000)` will produce an animation where the upper frequency
#' boundary gradually increases from 5000 Hz to 10,000 Hz.
#' @param spec_windowLength Window length in seconds for generating spectrograms.
#' Default is `0.005`. Alternatively, a vector giving the first and last
#' window lengths in the animation.
#' @param spec_dynamicRange Dynamic range in dB for generating spectrograms. The
#' maximum intensity minus `spec_dynamicRange` will all be printed in white.
#' Default is `50`. Alternatively, a vector giving the first and last
#' dynamic range values in the animation.
#' @param spec_timeStep How many time steps should be calculated for spectrograms?
#' Default is `1000`. Alternatively, a vector giving the first and last
#' time step values in the animation.
#' @param pitch_timeStep Measurement interval in seconds for tracking pitch.
#' Default is `NULL`, in which case the measurement interval is equal to
#' 0.75 / `pitch_floor`. Alternatively, a vector giving the first and last
#' measurement intervals in the animation.
#' @param pitch_floor Frequency in Hz; no pitch candidates considered below
#' this frequency. Default is `75`. Alternatively, a vector giving the first and
#' last pitch floors to be used in the animation.
#' @param pitch_ceiling Frequency in Hz; no pitch candidates considered above
#' this frequency. Default is `600`. Alternatively, a vector giving the first and
#' last pitch ceilings to be used in the animation.
#' @param pitch_freqRange Vector of two integers giving the frequency range to be
#' used for producing pitch plots. Default is `c(50,500)`. If the frequency
#' scales `semitones` or `erb` are used, the pitch range is automatically reset
#' to the Praat defaults for these scales (`c(-12,30)` and `c(0,10)`,
#' respectively). Alternatively, a
#' vector of four integers giving the first and last lowest frequency, followed
#' by the first and last highest frequency in the animation
#' (see `spec_freqRange` for usage details).
#' @param pitch_semitonesRe Frequency in Hz giving the reference level for
#' converting pitch frequency to semitones. Default is `100`.
#' Alternatively, a vector giving the first and
#' last semitone reference levels to be used in the animation.
#' @param formant_timeStep Measurement interval in seconds for tracking formants.
#' Default is `NULL`, in which case the measurement interval is equal to
#' `formant_windowLength` / 4. Alternatively, a vector giving the first and
#' last measurement intervals to be used in the animation.
#' @param formant_windowLength The effective duration of the analysis window
#' used for tracking formants in seconds; the actual duration of the
#' analysis window is twice this value. Alternatively, a vector giving the first
#' and last window lengths to be used in the animation.
#' @param formant_dynamicRange Dynamic range in dB for producing formant plots.
#' When a formant plot of `formant_plotType='speckle'` is drawn, no formants are
#' shown in frames with intensity level `formant_dynamicRange` below the maximum
#' intensity. Default is `30`. If set to `0`, all formants are shown.
#' Alternatively, a vector giving the first
#' and last dynamic range levels to be used in the animation.
#' @param formant_freqRange Vector of two integers giving the frequency range to
#' be used for producing formant plots. Default is `c(0,5500)`. Alternatively, a
#' vector of four integers giving the first and last lowest frequency, followed
#' by the first and last highest frequency in the animation
#' (see `spec_freqRange` for usage details).
#' @param intensity_timeStep Measurement interval in seconds for tracking
#' intensity. Default is `NULL`, in which case the measurement interval is
#' equal to 0.8 * `intensity_minPitch`. Alternatively, a vector giving the first
#' and last measurement intervals to be used in the animation.
#' @param intensity_minPitch Lowest pitch in Hz used when calculating
#' intensity; default is `100`. Alternatively, a vector giving the first
#' and last minimum pitch levels to be used in the animation.
#' @param intensity_range Vector of two integers giving the intensity range to be
#' used for producing intensity plots. Default is `NULL`, in which case the
#' range is simply the minimum and maximum levels in the curve. Alternatively, a
#' vector of four integers giving the first and last lowest level, followed
#' by the first and last highest level in the animation
#' (see `spec_freqRange` for usage details).
#' @param ... Further arguments passed to `praatpicture`.
#'
#' @seealso This function is a wrapper for either `gifski::save_gif()`
#' or `av::av_capture_graphics()` used to
#' produce animations based on `praatpicture()`. For more detail on your
#' options, see the `praatpicture()` help file.
#' @return No return value, produces an animated figure.
#' @export
#'
#' @examples
#' \dontrun{
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#'
#' # Show increasing frequency range
#' praatanimation(soundFile, spec_freqRange=c(0,0,4000,12000))
#'
#' # Transition from narrowband to broadband spectrogram
#' praatanimation(soundFile, spec_windowLength=c(0.005,0.03))
#'
#' # Etc.
#' }
praatanimation <- function(sound, width=1080, height=720, frameRate=24,
                           n_frames=50, loop=TRUE, outputFile=NULL,
                           outputFormat='gif', useViewer=TRUE,
                           verbose=TRUE, pointsize=25, start=0, end=0,
                           spec_freqRange=c(0,5000), spec_windowLength=0.005,
                           spec_dynamicRange=50, spec_timeStep=1000,
                           pitch_timeStep=NULL, pitch_floor=50,
                           pitch_ceiling=600, pitch_freqRange=c(50,500),
                           pitch_semitonesRe=100, formant_timeStep=NULL,
                           formant_windowLength=0.025, formant_dynamicRange=30,
                           formant_freqRange=c(50,5500), intensity_timeStep=NULL,
                           intensity_minPitch=100, intensity_range=NULL, ...) {

  if (is.null(outputFile)) {
    if (outputFormat == 'gif') outputFile <- 'praatgif.gif'
    if (outputFormat == 'mp4') outputFile <- 'praatvid.mp4'
  }

  makeseq <- function(vec, n_frames) {
    newseq <- seq(utils::head(vec, 1), utils::tail(vec, 1), length.out=n_frames)
    return(newseq)
  }

  start <- makeseq(start,n_frames); end <- makeseq(end,n_frames)
  spec_windowLength <- makeseq(spec_windowLength,n_frames)
  spec_dynamicRange <- makeseq(spec_dynamicRange,n_frames)
  spec_timeStep <- makeseq(spec_timeStep,n_frames)
  if (!is.null(pitch_timeStep)) pitch_timeStep <-
    makeseq(pitch_timeStep,n_frames)
  pitch_floor <- makeseq(pitch_floor,n_frames)
  pitch_ceiling <- makeseq(pitch_ceiling,n_frames)
  pitch_semitonesRe <- makeseq(pitch_semitonesRe,n_frames)
  if (!is.null(formant_timeStep)) formant_timeStep <-
    makeseq(formant_timeStep,n_frames)
  formant_windowLength <- makeseq(formant_windowLength,n_frames)
  formant_dynamicRange <- makeseq(formant_dynamicRange,n_frames)
  if (!is.null(intensity_timeStep)) intensity_timeStep <-
    makeseq(intensity_timeStep,n_frames)
  intensity_minPitch <- makeseq(intensity_minPitch,n_frames)

  if (length(spec_freqRange) == 4) {
    spec_freqRangeLow <- makeseq(spec_freqRange[1:2],n_frames)
    spec_freqRangeUpp <- makeseq(spec_freqRange[3:4],n_frames)
  } else {
    spec_freqRangeLow <- makeseq(spec_freqRange[1],n_frames)
    spec_freqRangeUpp <- makeseq(spec_freqRange[2],n_frames)
  }
  if (length(pitch_freqRange) == 4) {
    pitch_freqRangeLow <- makeseq(pitch_freqRange[1:2],n_frames)
    pitch_freqRangeUpp <- makeseq(pitch_freqRange[3:4],n_frames)
  } else {
    pitch_freqRangeLow <- makeseq(pitch_freqRange[1],n_frames)
    pitch_freqRangeUpp <- makeseq(pitch_freqRange[2],n_frames)
  }
  if (length(formant_freqRange) == 4) {
    formant_freqRangeLow <- makeseq(formant_freqRange[1:2],n_frames)
    formant_freqRangeUpp <- makeseq(formant_freqRange[3:4],n_frames)
  } else {
    formant_freqRangeLow <- makeseq(formant_freqRange[1],n_frames)
    formant_freqRangeUpp <- makeseq(formant_freqRange[2],n_frames)
  }
  if (!is.null(intensity_range)) {
    if (length(intensity_range == 4)) {
      intensity_rangeLow <- makeseq(intensity_range[1:2],n_frames)
      intensity_rangeUpp <- makeseq(intensity_range[3:4],n_frames)
    } else {
      intensity_rangeLow <- makeseq(intensity_range[1],n_frames)
      intensity_rangeUpp <- makeseq(intensity_range[2],n_frames)
    }
  } else {
    intensity_rangeLow <- NULL; intensity_rangeUpp <- NULL
  }

  if (useViewer) {
    tempDir <- tempfile()
    dir.create(tempDir)
    htmlFile <- file.path(tempDir, 'index.html')
    outputFile <- file.path(tempDir, outputFile)
  }

  if (outputFormat == 'gif') {
    gifski::save_gif(
      for (i in 1:n_frames) {
        praatpicture(sound, start=start[i], end=end[i],
                     spec_freqRange=c(spec_freqRangeLow[i], spec_freqRangeUpp[i]),
                     spec_windowLength=spec_windowLength[i],
                     spec_dynamicRange=spec_dynamicRange[i],
                     spec_timeStep=spec_timeStep[i],
                     pitch_timeStep=pitch_timeStep[i],
                     pitch_floor=pitch_floor[i],
                     pitch_ceiling=pitch_ceiling[i],
                     pitch_freqRange=c(
                       pitch_freqRangeLow[i], pitch_freqRangeUpp[i]),
                     pitch_semitonesRe=pitch_semitonesRe[i],
                     formant_timeStep=formant_timeStep[i],
                     formant_windowLength=formant_windowLength[i],
                     formant_dynamicRange=formant_dynamicRange[i],
                     formant_freqRange=c(
                       formant_freqRangeLow[i], formant_freqRangeUpp[i]),
                     intensity_timeStep=intensity_timeStep[i],
                     intensity_minPitch=intensity_minPitch[i],
                     intensity_range=c(
                       intensity_rangeLow[i], intensity_rangeUpp[i]),
                     ...)
        if (verbose) print(paste('Frame', i, 'generated'))
      }, gif_file=outputFile, width=width, height=height,
      delay=1/frameRate, loop=loop, pointsize=pointsize
    )
    if (useViewer) {
      writeLines(text='<img width="600" height="400" src="praatgif.gif">',
                 con=htmlFile)
      rstudioapi::viewer(htmlFile)
    }
  }

  if (outputFormat == 'mp4') {
    av::av_capture_graphics(
      for (i in 1:n_frames) {
        praatpicture(sound, start=start[i], end=end[i],
                     spec_freqRange=c(spec_freqRangeLow[i], spec_freqRangeUpp[i]),
                     spec_windowLength=spec_windowLength[i],
                     spec_dynamicRange=spec_dynamicRange[i],
                     spec_timeStep=spec_timeStep[i],
                     pitch_timeStep=pitch_timeStep[i],
                     pitch_floor=pitch_floor[i],
                     pitch_ceiling=pitch_ceiling[i],
                     pitch_freqRange=c(
                       pitch_freqRangeLow[i], pitch_freqRangeUpp[i]),
                     pitch_semitonesRe=pitch_semitonesRe[i],
                     formant_timeStep=formant_timeStep[i],
                     formant_windowLength=formant_windowLength[i],
                     formant_dynamicRange=formant_dynamicRange[i],
                     formant_freqRange=c(
                       formant_freqRangeLow[i], formant_freqRangeUpp[i]),
                     intensity_timeStep=intensity_timeStep[i],
                     intensity_minPitch=intensity_minPitch[i],
                     intensity_range=c(
                       intensity_rangeLow[i], intensity_rangeUpp[i]),
                     ...)
        if (verbose) print(paste('Frame', i, 'generated'))
      }, output=outputFile, width=width, height=height, framerate=frameRate,
      verbose=verbose, pointsize=pointsize
    )
    if (useViewer) {
      writeLines(text=paste0('<video width="550" height="400" controls>\n',
      '<source src="praatvid.mp4">\n</video>'), con=htmlFile)
      rstudioapi::viewer(htmlFile)
    }
  }
}

#' Plot spectrogram
#'
#' Function for plotting spectrograms called by [praatpicture]. Instead of using
#' this function directly, just use
#' `praatpicture('my_sound_file', frames='spectrogram')`.
#'
#' @param sig Numeric vector corresponding to a sound signal.
#' @param sr Integer giving the sampling rate of the signal.
#' @param t Numeric vector giving times corresponding to the signal.
#' @param start Start time (in seconds) of desired plotted area.
#' @param end End time (in seconds) of desired plotted area.
#' @param tfrom0 Logical; should time on the x-axis run from 0 or from the
#' original time? Default is `TRUE`.
#' @param scale String giving the frequency scale to use for plotting
#' spectrograms. Default is `hz`. Alternatives are `erb` (for the equivalent
#' rectangular bandwidth scale), `mel` (for the Mel scale, using 1000 Hz
#' as the corner frequency, following Fant 1968), or `logarithmic` (for
#' log Hz).
#' @param freqRange Vector of two integers giving the frequency range to be
#' used for plotting spectrograms. Default is `NULL`, in which case suitable
#' ranges are chosen based on the frequency scale. For `hz`, the default is
#' `c(0,5000)`; for `erb`, the default is `c(0.8,42)`; for `mel`, the default is
#' `c(30,4400)`. The latter two correspond roughly to the human auditory
#' frequency response.
#' @param windowLength Window length in seconds for generating spectrograms.
#' Default is `0.005`.
#' @param dynamicRange Dynamic range in dB for generating spectrograms. The
#' maximum intensity minus `dynamicRange` will all be printed in white. Default
#' is `50`.
#' @param timeStep How many time steps should be calculated for spectrograms?
#' Default is `1000`. Note that this takes a while to plot, so for fiddling with
#' plotting parameters it is a good idea to choose a smaller value.
#' @param windowShape String giving the name of the window shape to be applied
#' to the signal when generating spectrograms. Default is `Gaussian`; other
#' options are `square`, `Hamming`, `Bartlett`, `Hanning`, or `Blackman`.
#' Note that the Gaussian window function provided by the `phonTools` package
#' and used in `praatpicture()` does not have the same properties as the
#' Gaussian window function used for spectral estimation in Praat; plotting
#' a simple sine wave with high dynamic range will produce sidelobes in
#' `praatpicture()` but not in Praat. It's recommended to use Blackman windows
#' instead if you have this problem.
#' @param colors Vector of strings giving the names of colors to be used
#' for plotting the spectrogram; default is `c('white', 'black')`. The first
#' value is used for plotting the lowest visible amplitude, and the last for
#' plotting the highest visible amplitude. Vectors with more than two color
#' names can be used for plotting values in between in different colors.
#' @param pitch_plotOnSpec Boolean; should pitch be plotted on top of
#' spectrogram? Default is `FALSE`.
#' @param pt Pitch object loaded using [rPraat::pt.read] or similar object.
#' @param pitch_plotType String giving the type of pitch plot to produce; default
#' is `draw` (a line plot), the only other option is `speckle` (a point plot).
#' Alternatively a vector `c('draw','speckle')` can be passed, in which case
#' both are used.
#' @param pitch_scale String giving the frequency scale to use when producing
#' pitch plots. Default is `hz`; other options are `logarithmic` (also in Hz),
#' `semitones`, `erb`, and `mel`.
#' @param pitch_freqRange Vector of two integers giving the frequency range to be
#' used for producing pitch plots. Default is `NULL`, in which case the pitch
#' range is automatically reset to `c(-12,30)` for the `semitones` scale,
#' `c(0,10)` for the `erb` scale, and `c(50,500)` for the Hz-based scales,
#' following Praat defaults.
#' @param pitch_axisLabel String giving the name of the label to print along the
#' y-axis when printing a pitch track. Default is `NULL`, in which case the
#' axis label will depend on the scale.
#' @param pitch_color String or vector of strings giving the name of the color
#' to be used for plotting pitch. Default is `'black'`. If a vector of two
#' strings is passed, the second color will be used for background highlighting.
#' @param pitch_highlight Named list giving parameters for differential
#' highlighting of pitch based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `pitch_color`),
#' `drawSize` or `speckleSize` (both numeric).
#' @param formant_plotOnSpec Boolean; should formants be plotted on top of
#' spectrogram? Default is `FALSE`.
#' @param fm Formant object loaded using [rPraat::formant.read] or similar
#' object.
#' @param formant_plotType String giving the type of formant plot to produce;
#' default is `speckle` (a point plot), the only other option is `draw` (a line
#' plot). Alternatively a vector `c('draw','speckle')` can be passed, in which
#' case both are used.
#' @param formant_dynamicRange Dynamic range in dB for producing formant plots.
#' When a formant plot of `formant_plotType='speckle'` is drawn, no formants are
#' shown in frames with intensity level `formant_dynamicRange` below the maximum
#' intensity. Default is `30`. If set to `0`, all formants are shown.
#' @param formant_color String or vector of strings giving the name(s) of
#' colors to be used for plotting formants. If one color is provided, all
#' formants will be plotted in this color. If multiple colors are provided,
#' different formants will be shown in different colors. Default is `'black'`.
#' If the length of this vector twice the number of formants plotted, the first
#' half of strings will be used for the formants' primary colors and the second
#' half will be used for background highlighting. If the length of this vector
#' is one more than the number of formants plotted, the last string will
#' be used for background highlighting.
#' @param formant_highlight Named list giving parameters for differential
#' highlighting of formants based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `formant_color`),
#' `drawSize` or `speckleSize` (both numeric).
#' @param intensity_plotOnSpec Boolean; should intensity be plotted on top of
#' spectrogram? Default is `FALSE`.
#' @param it Intensity object loaded using [rPraat::it.read] or similar object.
#' @param intensity_range Vector of two integers giving the intensity range to be
#' used for producing intensity plots. Default is `NULL`, in which case the
#' range is simply the minimum and maximum levels in the curve.
#' @param intensity_axisLabel String giving the name of the label to print along
#' the y-axis when plotting intensity. Default is `Intensity (dB)`.
#' @param intensity_color String or vector of strings giving the name of the
#' color to be used for plotting intensity. Default is `'black'`. If a vector of
#' two strings is passed, the second color will be used for background
#' highlighting.
#' @param intensity_highlight Named list giving parameters for differential
#' highlighting of the intensity contour based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `intensity_color`) and
#' `drawSize` (integer).
#' @param tgbool Logical; should dotted lines be plotted corresponding to
#' locations in a TextGrid? Default is `FALSE`.
#' @param lines Numeric vector giving locations in seconds of locations from
#' a TextGrid to be plotted with dotted lines. Default is `NULL`.
#' @param focusTierColor String or vector of strings giving the color(s) to
#' use for plotting focus tier lines. If multiple tiers are focused, a vector
#' of the same length can be passed, and the nth tier will be plotted in the
#' nth color. Default is `'black'`.
#' @param focusTierLineType String or vector of strings giving the line
#' type(s) for plotting focus tier lines. If multiple tiers are focused, a
#' vector of the same length can be passed, and the nth tier will be plotted in
#' the nth line type. Default is `'dotted'`.
#' @param ind Integer indexing waveform relative to other plot components.
#' Default is `NULL`.
#' @param min_max_only Logical; should only minimum and maximum values be given
#' on the y-axis? Default is `TRUE`. Can also be a logical vector if some but
#' not all plot components should have minimum and maximum values on the y-axis.
#' Ignored for TextGrid component.
#' @param highlight Named list giving parameters for differential
#' highlighting of the spectrogram based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the argument
#' `colors` (vector of strings, see `colors`). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param axisLabel String giving the name of the label to print along the
#' y-axis when plotting a spectrogram. Default is `NULL`, in which case the
#' axis label will depend on the scale.
#' @param drawSize Number indicating the line width of plot components where
#' the `_plotType` is `'draw'` (i.e., pitch, formants, or intensity rendered as
#' line plots). Default is `1`. Controls the `lwd` argument of
#' [graphics::lines].
#' @param speckleSize Number indicating the point size of plot components where
#' the `_plotType` is `'speckle'` (i.e. pitch or formants rendered as point
#' plots). Default is `1`. Controls the `cex` arguments of [graphics::points].
#' @param suppressAxes Should all axes and the box around plots be suppressed?
#' Default is `FALSE`. Generally not recommended.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, frames='spectrogram')
specplot <- function(sig, sr, t, start, end, tfrom0=TRUE, scale='hz',
                     freqRange=NULL, windowLength=0.005, dynamicRange=60,
                     timeStep=1000, windowShape='Gaussian',
                     colors=c('white', 'black'),
                     pitch_plotOnSpec=FALSE, pt=NULL,
                     pitch_plotType='draw', pitch_scale='hz',
                     pitch_freqRange=NULL, pitch_axisLabel=NULL,
                     pitch_color='black', pitch_highlight=NULL,
                     formant_plotOnSpec=FALSE, fm=NULL,
                     formant_plotType='speckle', formant_dynamicRange=30,
                     formant_color='black', formant_highlight=NULL,
                     intensity_plotOnSpec=FALSE, it=NULL, intensity_range=NULL,
                     intensity_axisLabel='Intensity (dB)',
                     intensity_color='black', intensity_highlight=NULL,
                     tgbool=FALSE, lines=NULL, focusTierColor='black',
                     focusTierLineType='dotted', ind=NULL,
                     min_max_only=TRUE, highlight=NULL,
                     axisLabel=NULL, drawSize=1, speckleSize=1,
                     suppressAxes=FALSE) {

  if (scale == 'logarithmic') {
    logsc <- 'y'
  } else {
    logsc <- ''
  }

  wl <- windowLength*1000
  ts <- -timeStep

  legal_ws <- c('square', 'Hamming', 'Bartlett', 'Hanning', 'Gaussian',
                'Blackman')
  if (!windowShape %in% legal_ws) {
    stop('Possible window shapes are square, Hamming, Bartlett, Hanning, ',
         'Gaussian, or Blackman.')
  }

  if (tfrom0) {
    org_start <- start
    start <- 0
  }

  if (windowShape == 'square') ws <- 'rectangular'
  if (windowShape == 'Hamming') ws <- 'hamming'
  if (windowShape == 'Bartlett') ws <- 'bartlett'
  if (windowShape == 'Hanning') ws <- 'hann'
  if (windowShape == 'Gaussian') ws <- 'gaussian'
  if (windowShape == 'Blackman') ws <- 'blackman'

  if (!min_max_only[ind]) {
    if (ind == 1) {
      yax <- 's'
    } else {
      ytix <- grDevices::axisTicks(freqRange, log=F)
      ytix <- ytix[-length(ytix)]
      yax <- 'n'
    }
  } else {
    yax <- 'n'
    ytix <- freqRange
  }

  spec <- phonTools::spectrogram(sig, sr, colors=F, show=F, timestep=ts,
                                 windowlength=wl, window=ws,
                                 dynamicrange=dynamicRange,
                                 windowparameter=0.4)

  if (scale %in% c('hz', 'log')) {
    hzFreqRange <- freqRange
  } else if (scale == 'erb') {
    hzFreqRange <- soundgen::otherToHz(freqRange, scale = 'ERB')
    if (formant_plotOnSpec) fm$frequencyArray <-
        soundgen::HzToOther(fm$frequencyArray, scale = 'ERB')
  } else {
    hzFreqRange <- 1000 * (2^(freqRange / 1000)-1)
    if (formant_plotOnSpec) fm$frequencyArray <-
        1000 / log(2) * log(1 + fm$frequencyArray / 1000)
  }

  freqdom <- as.numeric(unlist(dimnames(spec$spectrogram)[2]))
  within_freqran <- which(freqdom < hzFreqRange[2] & freqdom > hzFreqRange[1])
  spec$spectrogram <- spec$spectrogram[,within_freqran]

  time_s <- as.numeric(rownames(spec$spectrogram))/1000
  if (!tfrom0) time_s <- time_s + start
  rownames(spec$spectrogram) <- time_s

  if (grDevices::dev.capabilities()$rasterImage == 'yes' &
      scale == 'hz') {
    useRaster <- TRUE
  } else {
    useRaster <- FALSE
  }

  plot(NULL, xaxt='n', xlim=c(start, end+start),
       ylim=freqRange, yaxs='i', yaxt=yax, log = logsc)
  if (!suppressAxes) {
    if (!min_max_only[ind] & ind != 1) graphics::axis(2, at=ytix)
    if (min_max_only[ind]) graphics::axis(2, at=ytix, padj=c(0,1), las=2,
                                          tick=F)
    graphics::mtext(axisLabel, side=2, line=3.5, cex=0.8)
  }

  fillCol <- grDevices::colorRampPalette(colors)
  fillRan <- c(-dynamicRange, 0)
  nlevels <- abs(-dynamicRange - 0) * 1.2
  levels <- pretty(fillRan, nlevels)
  fillCol <- fillCol(length(levels) - 1)
  subdr <- which(spec$spectrogram < -dynamicRange)
  spec$spectrogram[subdr] <- -dynamicRange

  specDims <- dim(spec$spectrogram)
  xVals <- seq(min(time_s), max(time_s), length.out=specDims[1])
  yVals <- seq(hzFreqRange[1], hzFreqRange[2], length.out=specDims[2])

  if (scale == 'mel') yVals <- emuR::mel(yVals)
  if (scale == 'erb') yVals <- soundgen::HzToOther(yVals, scale = 'ERB')

  graphics::image(x=xVals, y=yVals, z=spec$spectrogram,
                  col=fillCol, useRaster=useRaster, add=TRUE)

  if (!is.null(highlight)) {
    if (any(sapply(highlight, is.list))) {
      highlightLoopDepth <- sum(sapply(highlight, is.list))
    } else {
      highlightLoopDepth <- 1
    }
    for (i in 1:highlightLoopDepth) {
      if (any(sapply(highlight, is.list))) {
        thisHL <- highlight[[i]]
      } else {
        thisHL <- highlight
      }
      hlFillCol <- grDevices::colorRampPalette(thisHL$colors)
      hlFillCol <- hlFillCol(length(levels) - 1)
      for (int in 1:length(thisHL$start)) {
        times <- which(xVals > thisHL$start[int] & xVals < thisHL$end[int])
        highlight_t <- xVals[times]
        highlight_spec <- spec$spectrogram[times,]
        graphics::image(x=highlight_t, y=yVals, z=highlight_spec,
                        col=hlFillCol, useRaster=useRaster, add=TRUE)
      }
    }
  }

  create_fmtHighlighting <- function(highlight, fm, nf, dynamicRange, color,
                                     drawSize, speckleSize) {
    hl_t <- 0
    if (nf > 1) {
      hl_f <- array(rep(NA, nf), dim = c(nf,1))
    } else {
      hl_f <- NA
    }
    hl_i <- c()
    for (int in 1:length(highlight$start)) {
      times <- which(fm$t > highlight$start[int] & fm$t < highlight$end[int])
      hl_t <- c(hl_t, fm$t[times], max(fm$t[times] + 0.0001))
      if (nf > 1) {
        hl_f <- cbind(hl_f, fm$frequencyArray[,times], rep(NA, nf))
      } else {
        hl_f <- c(hl_f, fm$frequencyArray[times], NA)
      }
      hl_i <- c(hl_i, db[times], 0)
    }
    if (any(highlight$start < start)) {
      highlight$start[which(highlight$start < start)] <- start
    }
    if (any(highlight$end > end)) {
      highlight$end[which(highlight$end > end)] <- end
    }
    if (dynamicRange != 0) {
      hsubdr <- which(hl_i < max(db)-dynamicRange)
      if (length(hsubdr) == 0) hsubdr <- 1
    } else {
      hsubdr <- 1
    }

    if (!'color' %in% names(highlight)) highlight$color <- color
    if (!'drawSize' %in% names(highlight)) highlight$drawSize <- drawSize
    if (!'speckleSize' %in% names(highlight)) highlight$speckleSize <-
      speckleSize
    if (length(highlight$color) == 1) highlight$color <-
        rep(highlight$color, nf)
    if (length(highlight$color) == 2 & nf > 2) highlight$color <- c(
      rep(highlight$color[1], nf),
        rep(highlight$color[2], nf))
    if (length(highlight$color) == nf+1) highlight$color <- c(
      highlight$color, rep(highlight$color[nf+1], nf-1))
    return(list(params = highlight, hl_t = hl_t, hl_f = hl_f, subdr = hsubdr))
  }

  if (formant_plotOnSpec) {
    nf <- fm$maxnFormants
    if (length(formant_color) == 1) formant_color <- rep(formant_color, nf)
    if (length(formant_color) == 2 & nf > 2) formant_color <- c(
      rep(formant_color[1], nf), rep(formant_color[2], nf))
    if (length(formant_color) == nf+1) formant_color <- c(
      formant_color, rep(formant_color[nf+1], nf-1))
    if (tfrom0) fm$t <- fm$t - org_start
    if (fm$conv2db) {
      db <- gsignal::pow2db(fm$intensityVector)
    } else {
      db <- fm$intensityVector
    }
    if (formant_dynamicRange != 0) {
      subdr <- which(db < max(db)-formant_dynamicRange)
      if (length(subdr) == 0) subdr <- 1
    } else {
      subdr <- 1
    }

    if ('draw' %in% formant_plotType) {
      if (nf > 1) {
        if (length(formant_color) == nf*2) {
          graphics::lines(fm$t, fm$frequencyArray[1,], col=formant_color[nf+1],
                          lwd=drawSize+2)
        }
        graphics::lines(fm$t, fm$frequencyArray[1,], col=formant_color[1],
                        lwd=drawSize)
        for (i in 2:nf) {
          if (length(formant_color) == nf*2) {
            graphics::lines(fm$t, fm$frequencyArray[i,], lwd=drawSize+2,
                            col=formant_color[nf+i])
          }
          graphics::lines(fm$t, fm$frequencyArray[i,], col=formant_color[i],
                          lwd=drawSize)
        }
      } else {
        if (length(formant_color) == 2) {
          graphics::lines(fm$t, fm$frequencyArray, col=formant_color[2],
                          lwd=drawSize+2)
        }
        graphics::lines(fm$t, fm$frequencyArray, col=formant_color[1],
                        lwd=drawSize)
      }

      if (!is.null(formant_highlight)) {
        if (any(sapply(formant_highlight, is.list))) {
          highlightLoopDepth <- sum(sapply(formant_highlight, is.list))
        } else {
          highlightLoopDepth <- 1
        }
        for (i in 1:highlightLoopDepth) {
          if (any(sapply(formant_highlight, is.list))) {
            thisHL <- create_fmtHighlighting(formant_highlight[[i]], fm, nf,
                                          formant_dynamicRange, formant_color,
                                          drawSize, speckleSize)
          } else {
            thisHL <- create_fmtHighlighting(formant_highlight, fm, nf,
                                          formant_dynamicRange, formant_color,
                                          drawSize, speckleSize)
          }
          if (nf > 1) {
            if (length(thisHL$params$color) == nf*2) {
              graphics::lines(thisHL$hl_t, thisHL$hl_f[1,],
                              col=thisHL$params$color[nf+1],
                              lwd=thisHL$params$drawSize+2)
            }
            graphics::lines(thisHL$hl_t, thisHL$hl_f[1,],
                            col=thisHL$params$color[1],
                            lwd=thisHL$params$drawSize)
            for (j in 2:nf) {
              if (length(thisHL$params$color) == nf*2) {
                graphics::lines(thisHL$hl_t, thisHL$hl_f[j,],
                                lwd=thisHL$params$drawSize+2,
                                col=thisHL$params$color[nf+j])
              }
              graphics::lines(thisHL$hl_t, thisHL$hl_f[j,],
                              col=thisHL$params$color[j],
                              lwd=thisHL$params$drawSize)
            }
          } else {
            if (length(thisHL$params$color) == 2) {
              graphics::lines(thisHL$hl_t, thisHL$hl_f,
                              col=thisHL$params$color[2],
                              lwd=thisHL$params$drawSize+2)
            }
            graphics::lines(thisHL$hl_t, thisHL$hl_f,
                            col=thisHL$params$color[1],
                            lwd=thisHL$params$drawSize)
          }
        }
      }
    }
    if ('speckle' %in% formant_plotType) {
      if (nf > 1) {
        if (length(formant_color) == nf*2) {
          graphics::points(fm$t[-subdr], fm$frequencyArray[1,-subdr], pch=20,
                           cex=speckleSize, lwd=3,
                           col=formant_color[nf+1])
        }
        graphics::points(fm$t[-subdr], fm$frequencyArray[1,-subdr], pch=20,
                         col=formant_color[1], cex=speckleSize)
        for (i in 2:nf) {
          if (length(formant_color) == nf*2) {
            graphics::points(fm$t[-subdr], fm$frequencyArray[i,-subdr], pch=20,
                             lwd=3, col=formant_color[nf+i], cex=speckleSize)
          }
          graphics::points(fm$t[-subdr], fm$frequencyArray[i,-subdr], pch=20,
                           col=formant_color[i], cex=speckleSize)
        }
      } else {
        if (length(formant_color) == 2) {
          graphics::points(fm$t[-subdr], fm$frequencyArray[-subdr], pch=20,
                           cex=speckleSize, lwd=3,
                           col=formant_color[2])
        }
        graphics::points(fm$t[-subdr], fm$frequencyArray[-subdr], pch=20,
                         col=formant_color[1], cex=speckleSize)
      }

      if (!is.null(formant_highlight)) {
        if (any(sapply(formant_highlight, is.list))) {
          highlightLoopDepth <- sum(sapply(formant_highlight, is.list))
        } else {
          highlightLoopDepth <- 1
        }
        for (i in 1:highlightLoopDepth) {
          if (any(sapply(formant_highlight, is.list))) {
            thisHL <- create_fmtHighlighting(formant_highlight[[i]], fm, nf,
                                          formant_dynamicRange, formant_color,
                                          drawSize, speckleSize)
          } else {
            thisHL <- create_fmtHighlighting(formant_highlight, fm, nf,
                                          formant_dynamicRange, formant_color,
                                          drawSize, speckleSize)
          }
          if (nf > 1) {
            if (length(thisHL$params$color) == nf*2) {
              graphics::points(thisHL$hl_t[-thisHL$subdr],
                               thisHL$hl_f[1,-thisHL$subdr],
                               col=thisHL$params$color[nf+1], pch=20,
                               lwd=3, cex=thisHL$params$speckleSize)
            }
            graphics::points(thisHL$hl_t[-thisHL$subdr],
                             thisHL$hl_f[1,-thisHL$subdr],
                             col=thisHL$params$color[1], pch=20,
                             cex=thisHL$params$speckleSize)
            for (j in 2:nf) {
              if (length(thisHL$params$color) == nf*2) {
                graphics::points(thisHL$hl_t[-thisHL$subdr],
                                 thisHL$hl_f[j,-thisHL$subdr],
                                 col=thisHL$params$color[nf+j], pch=20,
                                 lwd=3, cex=thisHL$params$speckleSize)
              }
              graphics::points(thisHL$hl_t[-thisHL$subdr],
                               thisHL$hl_f[j,-thisHL$subdr],
                               col=thisHL$params$color[j], pch=20,
                               cex=thisHL$params$speckleSize)
            }
          } else {
            if (length(thisHL$params$color) == 2) {
              graphics::points(thisHL$hl_t[-thisHL$subdr],
                               thisHL$hl_f[-thisHL$subdr],
                               col=thisHL$params$color[2], pch=20,
                               lwd=3, cex=thisHL$params$speckleSize)
            }
            graphics::points(thisHL$hl_t[-thisHL$subdr],
                             thisHL$hl_f[-thisHL$subdr],
                             col=thisHL$params$color[1], pch=20,
                             cex=thisHL$params$speckleSize)
          }
        }
      }

    }
  }

  if (pitch_plotOnSpec) {
    pitch_overlay(pt, freqRange[1], freqRange[2], start, org_start, tfrom0,
                  pitch_freqRange, pitch_plotType, pitch_scale, pitch_color,
                  ind, drawSize, speckleSize, pitch_axisLabel, min_max_only,
                  pitch_highlight, intensity_plotOnSpec, suppressAxes)
  }

  if (intensity_plotOnSpec) {
    intensity_overlay(it, freqRange[1], freqRange[2], start, org_start,
                      tfrom0, intensity_range, intensity_color, ind, drawSize,
                      intensity_axisLabel, min_max_only, intensity_highlight,
                      pitch_plotOnSpec, suppressAxes)
  }

  if (tgbool) {
    for (i in 1:length(lines)) {
      graphics::abline(v=lines[[i]], col=focusTierColor[i],
                       lty=focusTierLineType[i])
    }
  }

}

#' Convert capital letters to Unicode small caps
#'
#' Helper function to convert capital letters into Unicode small caps.
#' May not work for all font families. Note that there's no Unicode small cap
#' 'X', so 'X' will just be converted to 'x'.
#'
#' @param x A string where all capital letters should be converted to small
#' caps.
#'
#' @return A string where all capital letters have been converted to small caps.
#' @export
#'
#' @examples
#' my_string <- 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
#' conv2sc(my_string)
conv2sc <- function(x) {
  x <- gsub('A', '\u1d00', x)
  x <- gsub('B', '\u0299', x)
  x <- gsub('C', '\u1d04', x)
  x <- gsub('D', '\u1d05', x)
  x <- gsub('E', '\u1d07', x)
  x <- gsub('F', '\u0493', x)
  x <- gsub('G', '\u0262', x)
  x <- gsub('H', '\u029c', x)
  x <- gsub('I', '\u026a', x)
  x <- gsub('J', '\u1d0a', x)
  x <- gsub('K', '\u1d0b', x)
  x <- gsub('L', '\u029f', x)
  x <- gsub('M', '\u1d0d', x)
  x <- gsub('N', '\u0274', x)
  x <- gsub('O', '\u1d0f', x)
  x <- gsub('P', '\u1d18', x)
  x <- gsub('Q', '\u01eb', x)
  x <- gsub('R', '\u0280', x)
  x <- gsub('S', '\ua731', x)
  x <- gsub('T', '\u1d1b', x)
  x <- gsub('U', '\u1d1c', x)
  x <- gsub('V', '\u1d20', x)
  x <- gsub('W', '\u1d21', x)
  x <- gsub('X', 'x', x)
  x <- gsub('Y', '\u028f', x)
  x <- gsub('Z', '\u1d22', x)
  return(x)
}

#' Plot TextGrid
#'
#' Function for plotting TextGrids called by [praatpicture]. Instead of using
#' this function directly, just use
#' `praatpicture('my_sound_file', frames='TextGrid')`.
#'
#' @param tg TextGrid object loaded using [rPraat::tg.read]
#' @param t Numeric vector giving times corresponding to the signal.
#' @param sr Integer giving the sampling rate of the signal.
#' @param start Start time (in seconds) of desired plotted area.
#' @param end End time (in seconds) of desired plotted area.
#' @param tiers Vector of number or strings giving either numeric identifiers
#' of TextGrid tiers to plot or the names of TextGrid tiers to plot. Default is
#' `1`, which plots just the first tier.
#' @param tfrom0 Logical; should time on the x-axis run from 0 or from the
#' original time? Default is `TRUE`.
#' @param tierNames Logical; should TextGrid tier names be printed along the
#' y-axis? Default is `TRUE`.
#' @param alignment String giving the desired alignment of text in the
#' TextGrids. Default is `central`; other options are `left` and `right`.
#' Alternatively, a vector of strings if different alignments are needed.
#' @param edgeLabels String specifying how to handle TextGrid labels in
#' interval tiers that fall partially before `start` or partially after `end`.
#' Default is `'keep'`, where labels are kept at the center of the interval.
#' Other options are `'center'`, where labels are recentered to the visible
#' part of the interval, or `'discard'`, where such labels are ignored.
#' @param specialChar Logical; should Praat typesetting for special font types
#' such as italic, bold, and small caps be converted into corresponding
#' R-readable special font types. Default is `FALSE`, since special characters
#' can create unfortunate text alignment artefacts.
#' See https://www.fon.hum.uva.nl/praat/manual/Text_styles.html.
#' @param color String or vector of strings giving the name of the color(s)
#' to be used for the text in TextGrids. Default is `'black'`. If a vector is
#' provided, different colors are used for different tiers.
#' @param highlight Named list giving parameters for differential
#' highlighting of TextGrid intervals. This list
#' should contain information about which intervals to highlight, using the
#' `tier` and `label`. Further contains the argument
#' `color`, and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, frames='TextGrid')
tgplot <- function(tg, t, sr, start, end, tiers=1, tfrom0=TRUE, tierNames=TRUE,
                   alignment='central', edgeLabels='keep',
                   specialChar=FALSE, color='black', highlight=NULL) {

  if (length(alignment) != length(tiers)) {
    alignment <- rep(alignment, length(tiers))
  }

  if (length(color) == 1) color <- rep(color, length(tiers))

  i <- 0
  for (tier in tiers) {
    i <- i+1
    lab <- tg[[tier]]$label
    tname <- tg[[tier]]$name

    if (tg[[tier]]$type == 'interval') {
      t1 <- tg[[tier]]$t1
      t2 <- tg[[tier]]$t2
      if (edgeLabels != 'keep' & (start > 0 | end > 0)) {
        line_vec <- t1[-which(t1 < start | t1 > end)]
      } else {
        line_vec <- t1
      }
      if (edgeLabels == 'center') {
        discard <- which(t2 < start | t1 > end)
        lab <- lab[-discard]
        t1 <- t1[-discard]
        t2 <- t2[-discard]
        if (any(t1 < start)) t1[which(t1 < start)] <- start
        if (any(t2 > end)) t2[which(t2 > end)] <- end
      }
      if (edgeLabels == 'discard') {
        if (start != 0 & any(t1 < start)) {
          discard <- which(t1 < start)
          t1 <- t1[-discard]
          t2 <- t2[-discard]
          lab <- lab[-discard]
        }
        if (any(t2 > end)) {
          discard <- which(t2 > end)
          t1 <- t1[-discard]
          t2 <- t2[-discard]
          lab <- lab[-discard]
        }
      }
      if (tfrom0) {
        t1 <- t1 - start
        t2 <- t2 - start
        line_vec <- line_vec - start
      }
    } else {
      t0 <- tg[[tier]]$t
      if (tfrom0) t0 <- t0 - start
      line_vec <- t0
    }

    if (tfrom0) {
      x1 <- 0
      x2 <- length(t)/sr
    } else {
      x1 <- start
      x2 <- (length(t)/sr) + start
    }

    if (specialChar) lab <- as.expression(tg_stylize(lab))

    plot(1, type='n', ylab='',
         xlim=c(x1, x2), xaxt='n',
         ylim=c(0,10), yaxt='n', yaxs='i')
    if (tg[[tier]]$type == 'interval') {
      graphics::abline(v=line_vec)
      if (alignment[[i]]=='central') graphics::text(
        t1+(t2-t1)/2, 5, lab, col=color[i], adj=c(0.5,0.5))
      if (alignment[[i]]=='left') graphics::text(
        t1, 5, lab, pos=4, col=color[i], adj=c(0.5,0.5))
      if (alignment[[i]]=='right') graphics::text(
        t2, 5, lab, pos=2, col=color[i], adj=c(0.5, 0.5))
    } else {
      graphics::segments(x0=line_vec, x1=line_vec, y0=0, y1=2, col=color[i])
      graphics::segments(x0=line_vec, x1=line_vec, y0=8, y1=10, col=color[i])
      graphics::text(t0, 5, lab, adj=c(0.5,0.5), col=color[i])
    }
    if (tierNames) graphics::mtext(tname, side=2, las=2, line=0.6, cex=0.8)
    if (!is.null(highlight)) {
      if (any(sapply(highlight, is.list))) {
        highlightLoopDepth <- sum(sapply(highlight, is.list))
      } else {
        highlightLoopDepth <- 1
      }
      for (j in 1:highlightLoopDepth) {
        if (any(sapply(highlight, is.list))) {
          thisHL <- highlight[[j]]
        } else {
          thisHL <- highlight
        }
        if ('tier' %in% names(thisHL)) {
          if (tier == thisHL$tier | tg[[tier]]$name == thisHL$tier) {
            match <- which(grepl(thisHL$label, lab))
            lab_highlight <- lab[match]
            t1_highlight <- t1[match]
            t2_highlight <- t2[match]
            if ('background' %in% names(thisHL)) {
              graphics::rect(t1_highlight,
                             -10,
                             t2_highlight,
                             10,
                             col = thisHL$background, border = NA)
            }
            if (alignment[[i]]=='central') graphics::text(
              t1_highlight+(t2_highlight-t1_highlight)/2, 5, lab_highlight,
              col=thisHL$color, adj=c(0.5,0.5))
            if (alignment[[i]]=='left') graphics::text(
              t1_highlight, 5, lab_highlight, pos=4, col=thisHL$color,
              adj=c(0.5,0.5))
            if (alignment[[i]]=='right') graphics::text(
              t2_highlight, 5, lab_highlight, pos=2, col=thisHL$color,
              adj=c(0.5, 0.5))
            graphics::abline(v=line_vec)
          }
        }
      }
    }

    graphics::box()
  }
}

#' Annotate praatpicture plot component
#'
#' Helper function for annotating plot components made with
#' praatpicture. Do not use directly, instead use [praatpicture] with the
#' `annotate` argument.
#'
#' @param plot_component String giving the name of the plot component to
#' annotate.
#' @param args A list of vectors giving arguments used for annotating.
#' See [praatpicture] documentation.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, annotate=c('spectrogram', 0.25, 1500,
#' 'An annotation'))
make_annot <- function(plot_component, args) {
  r <- which(sapply(args, '[[', 1) == plot_component)
  for (i in r) {
    params <- args[[i]]
    params <- as.list(params[2:length(params)])
    for (p in 1:2) params[[p]] <- as.numeric(params[[p]])
    for (p in c('adj', 'pos', 'offset', 'cex', 'font')) {
      params[which(names(params) == p)] <-
        as.numeric(params[which(names(params) == p)])
    }
    do.call(eval(parse(text='graphics::text')), params)
  }
}

options(shiny.maxRequestSize=50*1024^2)

server <- function(input, output, session) {

  curDev <- grDevices::dev.cur()
  onSessionEnded(function() grDevices::dev.set(curDev))

  observeEvent(input$frames, {
    if ('sound' %in% input$frames) {
      shinyjs::show('wave_color')
      shinyjs::show('wave_channels')
      shinyjs::show('wave_channelNames')
      shinyjs::show('wave_lineWidth')
      shinyjs::show('wave_axisDigits')
      shinyjs::show('pitch_plotOnWave')
      shinyjs::show('intensity_plotOnWave')
    } else {
      shinyjs::hide('wave_color')
      shinyjs::hide('wave_channels')
      shinyjs::hide('wave_channelNames')
      shinyjs::hide('wave_lineWidth')
      shinyjs::hide('wave_axisDigits')
      shinyjs::hide('pitch_plotOnWave')
      shinyjs::hide('intensity_plotOnWave')
    }

    if ('TextGrid' %in% input$frames) {
      shinyjs::show('tg_file')
    } else {
      shinyjs::hide('tg_file')
    }

    if ('spectrogram' %in% input$frames) {
      shinyjs::show('spec_freqRangeMin')
      shinyjs::show('spec_freqRangeMax')
      shinyjs::show('spec_windowLength')
      shinyjs::show('spec_dynamicRange')
      shinyjs::show('spec_timeStep')
      shinyjs::show('spec_windowShape')
      shinyjs::show('spec_colors')
      shinyjs::show('spec_axisLabel')
      shinyjs::show('spec_channel')
      shinyjs::show('pitch_plotOnSpec')
      shinyjs::show('formant_plotOnSpec')
      shinyjs::show('intensity_plotOnSpec')
    } else {
      shinyjs::hide('spec_freqRangeMin')
      shinyjs::hide('spec_freqRangeMax')
      shinyjs::hide('spec_windowLength')
      shinyjs::hide('spec_dynamicRange')
      shinyjs::hide('spec_timeStep')
      shinyjs::hide('spec_windowShape')
      shinyjs::hide('spec_colors')
      shinyjs::hide('spec_axisLabel')
      shinyjs::hide('spec_channel')
      shinyjs::hide('pitch_plotOnSpec')
      shinyjs::hide('formant_plotOnSpec')
      shinyjs::hide('intensity_plotOnSpec')
    }

    if ('TextGrid' %in% input$frames) {
      shinyjs::show('tg_tiers')
      shinyjs::show('tg_focusTier')
      shinyjs::show('tg_focusTierColor')
      shinyjs::show('tg_focusTierLineType')
      shinyjs::show('tg_tierNames')
      shinyjs::show('tg_alignment')
      shinyjs::show('tg_specialChar')
      shinyjs::show('tg_color')
    } else {
      shinyjs::hide('tg_tiers')
      shinyjs::hide('tg_focusTier')
      shinyjs::hide('tg_focusTierColor')
      shinyjs::hide('tg_focusTierLineType')
      shinyjs::hide('tg_tierNames')
      shinyjs::hide('tg_alignment')
      shinyjs::hide('tg_specialChar')
      shinyjs::hide('tg_color')
    }

    if ('pitch' %in% input$frames) {
      shinyjs::show('pitch_plotType')
      shinyjs::show('pitch_scale')
      shinyjs::show('pitch_freqRangeMin')
      shinyjs::show('pitch_freqRangeMax')
      shinyjs::show('pitch_timeStep')
      shinyjs::show('pitch_floor')
      shinyjs::show('pitch_ceiling')
      shinyjs::show('pitch_color')
      shinyjs::show('pitch_axisLabel')
    } else {
      shinyjs::hide('pitch_plotType')
      shinyjs::hide('pitch_scale')
      shinyjs::hide('pitch_freqRangeMin')
      shinyjs::hide('pitch_freqRangeMax')
      shinyjs::hide('pitch_timeStep')
      shinyjs::hide('pitch_floor')
      shinyjs::hide('pitch_ceiling')
      shinyjs::hide('pitch_color')
      shinyjs::hide('pitch_axisLabel')
    }

    if ('formant' %in% input$frames) {
      shinyjs::show('formant_plotType')
      shinyjs::show('formant_freqRangeMin')
      shinyjs::show('formant_freqRangeMax')
      shinyjs::show('formant_dynamicRange')
      shinyjs::show('formant_timeStep')
      shinyjs::show('formant_windowLength')
      shinyjs::show('formant_maxN')
      shinyjs::show('formant_color')
      shinyjs::show('formant_axisLabel')
      shinyjs::show('formant_number')
    } else {
      shinyjs::hide('formant_plotType')
      shinyjs::hide('formant_freqRangeMin')
      shinyjs::hide('formant_freqRangeMax')
      shinyjs::hide('formant_dynamicRange')
      shinyjs::hide('formant_timeStep')
      shinyjs::hide('formant_windowLength')
      shinyjs::hide('formant_maxN')
      shinyjs::hide('formant_color')
      shinyjs::hide('formant_axisLabel')
      shinyjs::hide('formant_number')
    }

    if ('intensity' %in% input$frames) {
      shinyjs::show('intensity_rangeMin')
      shinyjs::show('intensity_rangeMax')
      shinyjs::show('intensity_minPitch')
      shinyjs::show('intensity_color')
      shinyjs::show('intensity_axisLabel')
    } else {
      shinyjs::hide('intensity_rangeMin')
      shinyjs::hide('intensity_rangeMax')
      shinyjs::hide('intensity_minPitch')
      shinyjs::hide('intensity_color')
      shinyjs::hide('intensity_axisLabel')
    }

    updateTextInput(inputId = 'proportion',
                    value = paste(rep(50, length(input$frames)),
                                  collapse=','))
  })

  observeEvent(input$pitch_plotOnSpec, {
    if (input$pitch_plotOnSpec) {
      shinyjs::show('pitch_plotType')
      shinyjs::show('pitch_scale')
      shinyjs::show('pitch_freqRangeMin')
      shinyjs::show('pitch_freqRangeMax')
      shinyjs::show('pitch_timeStep')
      shinyjs::show('pitch_floor')
      shinyjs::show('pitch_ceiling')
      shinyjs::show('pitch_color')
      shinyjs::show('pitch_axisLabel')
    } else {
      shinyjs::hide('pitch_plotType')
      shinyjs::hide('pitch_scale')
      shinyjs::hide('pitch_freqRangeMin')
      shinyjs::hide('pitch_freqRangeMax')
      shinyjs::hide('pitch_timeStep')
      shinyjs::hide('pitch_floor')
      shinyjs::hide('pitch_ceiling')
      shinyjs::hide('pitch_color')
      shinyjs::hide('pitch_axisLabel')
    }
  })

  observeEvent(input$pitch_plotOnWave, {
    if (input$pitch_plotOnWave) {
      shinyjs::show('pitch_plotType')
      shinyjs::show('pitch_scale')
      shinyjs::show('pitch_freqRangeMin')
      shinyjs::show('pitch_freqRangeMax')
      shinyjs::show('pitch_timeStep')
      shinyjs::show('pitch_floor')
      shinyjs::show('pitch_ceiling')
      shinyjs::show('pitch_color')
      shinyjs::show('pitch_axisLabel')
    } else {
      shinyjs::hide('pitch_plotType')
      shinyjs::hide('pitch_scale')
      shinyjs::hide('pitch_freqRangeMin')
      shinyjs::hide('pitch_freqRangeMax')
      shinyjs::hide('pitch_timeStep')
      shinyjs::hide('pitch_floor')
      shinyjs::hide('pitch_ceiling')
      shinyjs::hide('pitch_color')
      shinyjs::hide('pitch_axisLabel')
    }
  })


  observeEvent(input$formant_plotOnSpec, {
    if (input$formant_plotOnSpec) {
      shinyjs::show('formant_plotType')
      shinyjs::show('formant_freqRangeMin')
      shinyjs::show('formant_freqRangeMax')
      shinyjs::show('formant_dynamicRange')
      shinyjs::show('formant_timeStep')
      shinyjs::show('formant_windowLength')
      shinyjs::show('formant_maxN')
      shinyjs::show('formant_color')
      shinyjs::show('formant_axisLabel')
      shinyjs::show('formant_number')
    } else {
      shinyjs::hide('formant_plotType')
      shinyjs::hide('formant_freqRangeMin')
      shinyjs::hide('formant_freqRangeMax')
      shinyjs::hide('formant_dynamicRange')
      shinyjs::hide('formant_timeStep')
      shinyjs::hide('formant_windowLength')
      shinyjs::hide('formant_maxN')
      shinyjs::hide('formant_color')
      shinyjs::hide('formant_axisLabel')
      shinyjs::hide('formant_number')
    }
  })

  observeEvent(input$intensity_plotOnSpec, {
    if (input$intensity_plotOnSpec) {
      shinyjs::show('intensity_rangeMin')
      shinyjs::show('intensity_rangeMax')
      shinyjs::show('intensity_minPitch')
      shinyjs::show('intensity_color')
      shinyjs::show('intensity_axisLabel')
    } else {
      shinyjs::hide('intensity_rangeMin')
      shinyjs::hide('intensity_rangeMax')
      shinyjs::hide('intensity_minPitch')
      shinyjs::hide('intensity_color')
      shinyjs::hide('intensity_axisLabel')
    }
  })

  observeEvent(input$intensity_plotOnWave, {
    if (input$intensity_plotOnWave) {
      shinyjs::show('intensity_rangeMin')
      shinyjs::show('intensity_rangeMax')
      shinyjs::show('intensity_minPitch')
      shinyjs::show('intensity_color')
      shinyjs::show('intensity_axisLabel')
    } else {
      shinyjs::hide('intensity_rangeMin')
      shinyjs::hide('intensity_rangeMax')
      shinyjs::hide('intensity_minPitch')
      shinyjs::hide('intensity_color')
      shinyjs::hide('intensity_axisLabel')
    }
  })

  wave_channels <- reactive({
    if (input$wave_channels != 'all') {
      as.numeric(unlist(strsplit(input$wave_channels, ',')))
    } else {
      'all'
    }
  })

  tg_tiers <- reactive({
    if (input$tg_tiers == 'all') {
      'all'
    } else if (!grepl(paste(0:9, collapse='|'), input$tg_tiers)) {
      unlist(strsplit(input$tg_tiers, ','))
    } else {
      as.numeric(unlist(strsplit(input$tg_tiers, ',')))
    }
  })

  tg_focusTier <- reactive({
    if (input$tg_focusTier == 'all') {
      'all'
    } else if (input$tg_focusTier == 'none') {
      'none'
    } else if (!grepl(paste(0:9, collapse='|'), input$tg_focusTier)) {
      unlist(strsplit(input$tg_focusTier, ','))
    } else {
      as.numeric(unlist(strsplit(input$tg_focusTier, ',')))
    }
  })

  spec_channel <- reactive({
    if (is.na(input$spec_channel)) {
      NULL
    } else {
      input$spec_channel
    }
  })

  observeEvent(input$wave_channelNames, {
    if (input$wave_channelNames) {
      shinyjs::show('wave_channelNamesStr')
    } else {
      shinyjs::hide('wave_ChannelNamesStr')
    }
  })

  channelNames <- reactive({
    if (input$wave_channelNames) {
      if (!is.na(input$wave_channelNamesStr)) {
        unlist(strsplit(input$wave_channelNamesStr, ','))
      } else {
        TRUE
      }
    } else {
      FALSE
    }
  })

  observeEvent(input$tUnit, {
    if (input$tUnit == 'ms') {
      updateTextInput(inputId = 'time_axisLabel',
                      value = 'Time (ms)')
    } else if (input$tUnit == 's') {
      updateTextInput(inputId = 'time_axisLabel',
                      value = 'Time (s)')
    }
  })

  observeEvent(input$pitch_scale, {
    if (input$pitch_scale == 'semitones') {
      shinyjs::show('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = -12)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 30)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (semitones')
    } else if (input$pitch_scale == 'erb') {
      shinyjs::hide('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = 0)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 10)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (ERB)')
    } else if (input$pitch_scale == 'mel') {
      shinyjs::hide('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = 50)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 500)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (mel)')
    } else if (input$pitch_scale == 'logarithmic') {
      shinyjs::hide('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = 50)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 500)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (log Hz)')
    } else if (input$pitch_scale == 'hz') {
      shinyjs::hide('pitch_semitonesRe')
      updateNumericInput(inputId = 'pitch_freqRangeMin', value = 50)
      updateNumericInput(inputId = 'pitch_freqRangeMax', value = 500)
      updateTextInput(inputId = 'pitch_axisLabel', value = 'Frequency (Hz)')
    }
  })

  observeEvent(input$pitch_floor, {
    updateNumericInput(inputId = 'pitch_timeStep',
                       value = 0.75 / input$pitch_floor)
  })

  observeEvent(input$formant_windowLength, {
    updateNumericInput(inputId = 'formant_timeStep',
                       value = 0.25 * input$formant_windowLength)
  })

  observeEvent(input$formant_maxN, {
    updateNumericInput(inputId = 'formant_number',
                       value = input$formant_maxN,
                       max = input$formant_maxN)
  })

  intensity_range <- reactive({
    if (is.na(input$intensity_rangeMin) & is.na(input$intensity_rangeMax)) {
      NULL
    } else {
      c(as.numeric(input$intensity_rangeMin),
        as.numeric(input$intensity_rangeMax))
    }
  })

  observeEvent(input$globalColor, {
    updateTextInput(inputId = 'wave_color', value = input$globalColor)
    updateTextInput(inputId = 'pitch_color', value = input$globalColor)
    updateTextInput(inputId = 'formant_color', value = input$globalColor)
    updateTextInput(inputId = 'intensity_color', value = input$globalColor)
    updateTextInput(inputId = 'tg_color', value = input$globalColor)
    updateTextInput(inputId = 'tg_focusTierColor', value = input$globalColor)
    updateTextInput(inputId = 'spec_colors',
                    value = paste(input$bgColor, input$globalColor, sep=','))
  })

  observeEvent(input$bgColor, {
    updateTextInput(inputId = 'spec_colors',
                    value = paste(input$bgColor, input$globalColor, sep=','))
  })

  observeEvent(input$highlightBool, {
    if(input$highlightBool) {
      shinyjs::show('highlightComp')
      shinyjs::show('highlightStart')
      shinyjs::show('highlightEnd')
      shinyjs::show('highlightCol')
      shinyjs::show('highlightBG')
      shinyjs::show('highlightTGbool')
      shinyjs::show('highlightDrawSize')
      shinyjs::show('highlightSpeckleSize')
    } else {
      shinyjs::hide('highlightComp')
      shinyjs::hide('highlightStart')
      shinyjs::hide('highlightEnd')
      shinyjs::hide('highlightCol')
      shinyjs::hide('highlightBG')
      shinyjs::hide('highlightTGbool')
      shinyjs::hide('highlightDrawSize')
      shinyjs::hide('highlightSpeckleSize')
    }
  })

  observeEvent(input$highlightTGbool, {
    if (input$highlightTGbool) {
      shinyjs::show('highlightTier')
      shinyjs::show('highlightLabel')
    } else {
      shinyjs::hide('highlightTier')
      shinyjs::hide('highlightLabel')
    }
  })

  observeEvent(input$arrowBool, {
    if(input$arrowBool) {
      shinyjs::show('arrowComp')
      shinyjs::show('arrowx0')
      shinyjs::show('arrowx1')
      shinyjs::show('arrowy0')
      shinyjs::show('arrowy1')
      shinyjs::show('arrowLength')
      shinyjs::show('arrowAngle')
      shinyjs::show('arrowColor')
      shinyjs::show('arrowLineType')
      shinyjs::show('arrowLineWidth')
    } else {
      shinyjs::hide('arrowComp')
      shinyjs::hide('arrowx0')
      shinyjs::hide('arrowx1')
      shinyjs::hide('arrowy0')
      shinyjs::hide('arrowy1')
      shinyjs::hide('arrowLength')
      shinyjs::hide('arrowAngle')
      shinyjs::hide('arrowColor')
      shinyjs::hide('arrowLineType')
      shinyjs::hide('arrowLineWidth')
    }
  })

  observeEvent(input$rectBool, {
    if (input$rectBool) {
      shinyjs::show('rectComp')
      shinyjs::show('rectxleft')
      shinyjs::show('rectybottom')
      shinyjs::show('rectxright')
      shinyjs::show('rectytop')
      shinyjs::show('rectFill')
      shinyjs::show('rectColor')
      shinyjs::show('rectLineType')
      shinyjs::show('rectLineWidth')
    } else {
      shinyjs::hide('rectComp')
      shinyjs::hide('rectxleft')
      shinyjs::hide('rectybottom')
      shinyjs::hide('rectxright')
      shinyjs::hide('rectytop')
      shinyjs::hide('rectFill')
      shinyjs::hide('rectColor')
      shinyjs::hide('rectLineType')
      shinyjs::hide('rectLineWidth')
    }
  })

  observeEvent(input$annotBool, {
    if (input$annotBool) {
      shinyjs::show('annotComp')
      shinyjs::show('annotx')
      shinyjs::show('annoty')
      shinyjs::show('annotLabel')
      shinyjs::show('annotColor')
      shinyjs::show('annotFace')
      shinyjs::show('annotSize')
    } else {
      shinyjs::hide('annotComp')
      shinyjs::hide('annotx')
      shinyjs::hide('annoty')
      shinyjs::hide('annotLabel')
      shinyjs::hide('annotColor')
      shinyjs::hide('annotFace')
      shinyjs::hide('annotSize')
    }
  })

  highlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'all') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      tmp$drawSize <- as.numeric(
        unlist(strsplit(input$highlightDrawSize, ',')))
      tmp$speckleSize <- as.numeric(
        unlist(strsplit(input$highlightSpeckleSize, ',')))
      return(tmp)
    }
  })

  waveHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'wave') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      return(tmp)
    }
  })

  specHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'spectrogram') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$colors <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      return(tmp)
    }
  })

  pitchHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'pitch') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      tmp$drawSize <- as.numeric(
        unlist(strsplit(input$highlightDrawSize, ',')))
      tmp$speckleSize <- as.numeric(
        unlist(strsplit(input$highlightSpeckleSize, ',')))
      return(tmp)
    }
  })

  formantHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'formant') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      tmp$drawSize <- as.numeric(
        unlist(strsplit(input$highlightDrawSize, ',')))
      tmp$speckleSize <- as.numeric(
        unlist(strsplit(input$highlightSpeckleSize, ',')))
      return(tmp)
    }
  })

  intensityHighlight_args <- reactive({
    if (!input$highlightBool | input$highlightComp != 'intensity') {
      return(NULL)
    } else {
      tmp <- list()
      tmp$start <- as.numeric(unlist(strsplit(input$highlightStart, ',')))
      tmp$end <- as.numeric(unlist(strsplit(input$highlightEnd, ',')))
      if (input$highlightTier != '') {
        tmp$tier <- input$highlightTier
        tmp$label <- input$highlightLabel
      }
      tmp$color <- unlist(strsplit(input$highlightCol, ','))
      tmp$background <- input$highlightBG
      tmp$drawSize <- as.numeric(
        unlist(strsplit(input$highlightDrawSize, ',')))
      return(tmp)
    }
  })

  arrow_args <- reactive({
    if (!input$arrowBool) {
      return(NULL)
    } else {
      n_args <- length(unlist(strsplit(input$arrowComp, ',')))
      tmp <- list()
      arrowLength <- as.numeric(unlist(strsplit(input$arrowLength, ',')))
      if (length(arrowLength) != n_args) arrowLength <-
        rep(arrowLength[1], n_args)
      arrowAngle <- as.numeric(unlist(strsplit(input$arrowAngle, ',')))
      if (length(arrowAngle) != n_args) arrowAngle <-
        rep(arrowAngle[1], n_args)
      arrowColor <- unlist(strsplit(input$arrowColor, ','))
      if (length(arrowColor) != n_args) arrowColor <-
        rep(arrowColor[1], n_args)
      arrowLineType <- unlist(strsplit(input$arrowLineType, ','))
      if (length(arrowLineType) != n_args) arrowLineType <-
        rep(arrowLineType[1], n_args)
      arrowLineWidth <- as.numeric(unlist(strsplit(input$arrowLineWidth, ',')))
      if (length(arrowLineWidth) != n_args) arrowLineWidth <-
        rep(arrowLineWidth[1], n_args)
      for (i in 1:n_args) {
        tmp[[i]] <- c(
          unlist(strsplit(input$arrowComp, ','))[i],
          x0 = as.numeric(unlist(strsplit(input$arrowx0, ',')))[i],
          x1 = as.numeric(unlist(strsplit(input$arrowx1, ',')))[i],
          y0 = as.numeric(unlist(strsplit(input$arrowy0, ',')))[i],
          y1 = as.numeric(unlist(strsplit(input$arrowy1, ',')))[i],
          length = arrowLength[i],
          angle = arrowAngle[i],
          col = arrowColor[i],
          lty = arrowLineType[i],
          lwd = arrowLineWidth[i]
        )
      }
      return(tmp)
    }
  })

  rect_args <- reactive({
    if (!input$rectBool) {
      return(NULL)
    } else {
      n_args <- length(unlist(strsplit(input$rectComp, ',')))
      tmp <- list()
      rectLength <- as.numeric(unlist(strsplit(input$arrowLength, ',')))
      rectFill <- unlist(strsplit(input$rectFill, ','))
      if (length(rectFill) != n_args) rectFill <-
        rep(rectFill[1], n_args)
      rectColor <- unlist(strsplit(input$rectColor, ','))
      if (length(rectColor) != n_args) rectColor <-
        rep(rectColor[1], n_args)
      rectLineType <- unlist(strsplit(input$rectLineType, ','))
      if (length(rectLineType) != n_args) rectLineType <-
        rep(rectLineType[1], n_args)
      rectLineWidth <- as.numeric(unlist(strsplit(input$rectLineWidth, ',')))
      if (length(rectLineWidth) != n_args) rectLineWidth <-
        rep(rectLineWidth[1], n_args)
      for (i in 1:n_args) {
        tmp[[i]] <- c(
          unlist(strsplit(input$rectComp, ','))[i],
          xleft = as.numeric(unlist(strsplit(input$rectxleft, ',')))[i],
          ybottom = as.numeric(unlist(strsplit(input$rectybottom, ',')))[i],
          xright = as.numeric(unlist(strsplit(input$rectxright, ',')))[i],
          ytop = as.numeric(unlist(strsplit(input$rectytop, ',')))[i],
          col = rectFill[i],
          border = rectColor[i],
          lwd = rectLineWidth[i],
          lty = rectLineType[i]
        )
      }
      return(tmp)
    }
  })

  annot_args <- reactive({
    if (!input$annotBool) {
      return(NULL)
    } else {
      n_args <- length(unlist(strsplit(input$annotComp, ',')))
      tmp <- list()
      annotLabel <- unlist(strsplit(input$annotLabel, ';'))
      annotColor <- unlist(strsplit(input$annotColor, ','))
      if (length(annotColor) != n_args) annotColor <-
        rep(annotColor, n_args)
      annotFace <- as.numeric(unlist(strsplit(input$annotFace, ',')))
      if (length(annotFace) != n_args) annotFace <-
        rep(annotFace, n_args)
      annotSize <- as.numeric(unlist(strsplit(input$annotSize, ',')))
      if (length(annotSize) != n_args) annotSize <-
        rep(annotSize, n_args)
      for (i in 1:n_args) {
        tmp[[i]] <- c(
          unlist(strsplit(input$annotComp, ','))[i],
          x = as.numeric(unlist(strsplit(input$annotx, ',')))[i],
          y = as.numeric(unlist(strsplit(input$annoty, ',')))[i],
          label = annotLabel[i],
          col = annotColor[i],
          font = annotFace[i],
          cex = annotSize[i]
        )
      }
      return(tmp)
    }
  })

  make_fig <- function(outputFile) {
    req(input$sound)
    if ('TextGrid' %in% input$frames) req(input$tg_file)
    grDevices::png(outputFile,
                   width=as.numeric(input$figWidth),
                   height=as.numeric(input$figHeight),
                   res=as.numeric(input$figDPI),
                   pointsize=input$fontSize, units='cm')
    praatpicture::praatpicture(input$sound$datapath,
                               start = as.numeric(input$start),
                               end = as.numeric(input$end),
                               tfrom0 = input$tfrom0,
                               tUnit = input$tUnit,
                               frames = input$frames,
                               proportion = as.numeric(unlist(strsplit(
                                 input$proportion, ','
                               ))),
                               mainTitle = input$mainTitle,
                               mainTitleAlignment = input$mainTitleAlignment,
                               start_end_only = input$start_end_only,
                               min_max_only = input$min_max_only,
                               time_axisLabel = input$time_axisLabel,
                               speckleSize = input$speckleSize,
                               drawSize = input$drawSize,
                               wave_color = input$wave_color,
                               wave_channels = wave_channels(),
                               wave_channelNames = channelNames(),
                               wave_axisDigits = input$wave_axisDigits,
                               wave_lineWidth = input$wave_lineWidth,
                               wave_highlight = waveHighlight_args(),
                               tg_file = input$tg_file$datapath,
                               tg_tiers = tg_tiers(),
                               tg_focusTier = tg_focusTier(),
                               tg_focusTierColor = unlist(strsplit(
                                 input$tg_focusTierColor, ',')),
                               tg_focusTierLineType = unlist(strsplit(
                                 input$tg_focusTierLineType, ',')),
                               tg_tierNames = input$tg_tierNames,
                               tg_alignment = unlist(strsplit(
                                 input$tg_alignment, ',')),
                               tg_specialChar = input$tg_specialChar,
                               tg_color = unlist(strsplit(
                                 input$tg_color, ',')),
                               spec_freqRange = c(
                                 as.numeric(input$spec_freqRangeMin),
                                 as.numeric(input$spec_freqRangeMax)),
                               spec_windowLength =
                                 as.numeric(input$spec_windowLength),
                               spec_dynamicRange =
                                 as.numeric(input$spec_dynamicRange),
                               spec_timeStep =
                                 as.numeric(input$spec_timeStep),
                               spec_windowShape = input$spec_windowShape,
                               spec_colors = unlist(strsplit(
                                 input$spec_colors, ',')),
                               spec_axisLabel = input$spec_axisLabel,
                               spec_channel = spec_channel(),
                               spec_highlight = specHighlight_args(),
                               pitch_plotType = input$pitch_plotType,
                               pitch_scale = input$pitch_scale,
                               pitch_semitonesRe = input$pitch_semitonesRe,
                               pitch_freqRange = c(
                                 as.numeric(input$pitch_freqRangeMin),
                                 as.numeric(input$pitch_freqRangeMax)),
                               pitch_timeStep = as.numeric(input$pitch_timeStep),
                               pitch_floor = as.numeric(input$pitch_floor),
                               pitch_ceiling = as.numeric(input$pitch_ceiling),
                               pitch_color = unlist(strsplit(
                                 input$pitch_color, ',')),
                               pitch_axisLabel = input$pitch_axisLabel,
                               pitch_plotOnSpec = input$pitch_plotOnSpec,
                               pitch_plotOnWave = input$pitch_plotOnWave,
                               pitch_highlight = pitchHighlight_args(),
                               formant_plotType = input$formant_plotType,
                               formant_freqRange = c(
                                 as.numeric(input$formant_freqRangeMin),
                                 as.numeric(input$formant_freqRangeMax)),
                               formant_dynamicRange = as.numeric(
                                 input$formant_dynamicRange),
                               formant_windowLength = as.numeric(
                                 input$formant_windowLength),
                               formant_timeStep = as.numeric(
                                 input$formant_timeStep),
                               formant_maxN = input$formant_maxN,
                               formant_color = unlist(strsplit(
                                 input$formant_color, ',')),
                               formant_number = input$formant_number,
                               formant_axisLabel = input$formant_axisLabel,
                               formant_plotOnSpec = input$formant_plotOnSpec,
                               formant_highlight = formantHighlight_args(),
                               intensity_range = intensity_range(),
                               intensity_minPitch = as.numeric(
                                 input$intensity_minPitch),
                               intensity_color = unlist(strsplit(
                                 input$intensity_color, ',')),
                               intensity_axisLabel = input$intensity_axisLabel,
                               intensity_plotOnSpec = input$intensity_plotOnSpec,
                               intensity_plotOnWave = input$intensity_plotOnWave,
                               intensity_highlight = intensityHighlight_args(),
                               font = as.numeric(input$fontFace),
                               font.axis = as.numeric(input$fontFace),
                               family = input$fontFamily,
                               col = input$globalColor,
                               col.axis = input$globalColor,
                               bg = input$bgColor,
                               highlight = highlight_args(),
                               draw_arrow = arrow_args(),
                               draw_rectangle = rect_args(),
                               annotate = annot_args(),
                               suppressAxes = input$suppressAxes
    )
    dev.off()
    list(src = outputFile,
         width = as.numeric(input$figWidth) * 40,
         height = as.numeric(input$figHeight) * 40)
  }

  output$plot <- renderImage({make_fig(tempfile(fileext='.png'))}, deleteFile = TRUE)
  output$download <- downloadHandler(
    filename = function() {'output.png'},
    content = function(file) {make_fig(file)}
  )

}

#' Draw rectangle on praatpicture plot component
#'
#' Helper function for drawing rectangles on plot components made with
#' praatpicture. Do not use directly, instead use [praatpicture] with the
#' `draw_rectangle` argument.
#'
#' @param plot_component String giving the name of the plot component to
#' draw on.
#' @param args A list of vectors giving arguments used for drawing
#' rectangles. See [praatpicture] documentation.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, draw_rectangle=c('spectrogram', 0.1, 500, 0.4, 2000))
draw_rectangle <- function(plot_component, args) {
  r <- which(sapply(args, '[[', 1) == plot_component)
  for (i in r) {
    params <- args[[i]]
    params <- as.list(params[2:length(params)])
    do.call(eval(parse(text='graphics::rect')), params)
  }
}

#' Make Praat Picture style plots of acoustic data with embedded audio
#'
#' Generate simple MP4 video files with Praat Picture style plots of acoustic
#' data with time-aligned transcriptions and embedded audio to use in
#' presentations etc.
#'
#' @param sound String giving the file name of a sound file with the .wav
#' extension.
#' @param start Start time (in seconds) of desired plotted area. Default is `0`.
#' @param end End time (in seconds) of desired plotted area. Default is `0`
#' (= the entire file).
#' @param frames String or vector of strings giving the frames that the plot
#' should consist of. Default is `sound`, `spectrogram`, `TextGrid`. This
#' requires a file with the extension `.TextGrid` and the same base name as the
#' sound file. Other options are `pitch`, `formant`, and `intensity`. See
#' details for more information.
#' @param draw_lines Use for drawing straight lines on plot components. Takes
#' an argument of type `list` which should contain a) a string giving the plot
#' component to draw straight lines on, and b) arguments to pass on to
#' [graphics::abline]. Should have a named argument `h` for horizontal lines,
#' or `v` for vertical lines, or `a`,`b` for the intercept and slope of the
#' line otherwise. Alternatively a nested list can be passed if more (sets of)
#' lines should be drawn. If multiple audio
#' channels are plotted and lines should be added to one of these,
#' use the channel identifier instead of a string giving the frame to draw on.
#' The default value is `list('formant', h=seq(0,10000,by=1000), lty='dotted')`.
#' According to Praat defaults, this means that if formants are plotted in a
#' separate frame, horizontal dotted lines (`lty`) are shown at 1000 Hz
#' intervals. To override this behavior, simply pass `draw_lines=NULL`.
#' The argument is ignored if `cursor = TRUE`.
#' @param audio_start Start time (in seconds) of embedded audio. By default
#' it is the same as `start`, i.e. the embedded audio is the portion of the
#' sound file that is being plotted.
#' @param audio_end End time (in seconds) of embedded audio. By default it is
#' the same as `end`, i.e. the embedded audio is the portion of the sound
#' that is being plotted.
#' @param width Number giving the desired width of the resulting animation in
#' pixels; default is `1080`.
#' @param height Number giving the desired height of the resulting animation in
#' pixels; default is `720`.
#' @param pointsize Number; which point size should be used for text in the
#' animation? Default is `25`. See [grDevices::png()] for more details.
#' @param cursor Logical; should a moving cursor show the current position in
#' the sound file? Default is `FALSE`.
#' @param cursor_frameRate Number giving the desired number of frames per second
#' for moving cursor. Default is `50`.
#' @param cursor_color String giving the color of the moving cursor. Default is
#' `black`.
#' @param cursor_lwd Number giving the line width of the moving cursor. Default
#' is `2`.
#' @param verbose Logical; should status messages be printed in the console as
#' figures are being generated? Default is `TRUE`.
#' @param outputFile String giving the desired file name. Default is
#' `praatvid.mp4`.
#' @param useViewer Logical; should the video be shown in the Viewer pane in
#' RStudio? Default is `TRUE`; if true, the video is oSnly saved in a
#' temporary directory, but can be downloaded from a browser.
#' @param ... Further arguments passed to `praatpicture`.
#'
#' @seealso This function is a wrapper for [av::av_capture_graphics()] used to
#' produce plots similar to those made with [praatpicture()] with embedded
#' audio. For more detail on your
#' options, see the `praatpicture()` help file.
#' @return No return value, produces a video file.
#' @export
#'
#' @examples
#' \dontrun{
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' talking_praatpicture(soundFile)
#' }
talking_praatpicture <- function(sound, start=0, end=0,
                                 frames = c('sound', 'spectrogram', 'TextGrid'),
                                 draw_lines=list('formant',
                                                 h=seq(0,10000,by=1000),
                                                 lty='dotted'),
                                 audio_start=start, audio_end=end,
                                 width=1080, height=720, pointsize=25,
                                 cursor=FALSE, cursor_frameRate=50,
                                 cursor_color = 'black', cursor_lwd = 2,
                                 verbose=TRUE,
                                 outputFile='praatvid.mp4', useViewer=TRUE,
                                 ...) {

  aend <- audio_end
  if (aend == 0) aend <- Inf

  if (audio_start != 0 | audio_end != 0) {
    snd <- tuneR::readWave(sound, from=audio_start, to=aend, units='seconds')
    sr <- snd@samp.rate
    nsamp <- length(snd@left)
    dur <- nsamp / sr
    tuneR::writeWave(snd, filename='tmp.wav', extensible=FALSE)
    audioFile <- 'tmp.wav'
  } else {
    audioFile <- sound
    snd <- tuneR::readWave(sound, from=0, units='seconds', toWaveMC=T)
    sr <- snd@samp.rate
    nsamp <- snd@dim[1]
    dur <- nsamp / sr
  }

  if (useViewer) {
    tempDir <- tempfile()
    dir.create(tempDir)
    htmlFile <- file.path(tempDir, 'index.html')
    outputFile <- file.path(tempDir, 'output.mp4')
  }

  if (cursor) {
    frameShift <- 1 / cursor_frameRate
    numFrames <- dur / frameShift
    cursorPositions <- seq(audio_start, audio_start + dur,
                           length.out = numFrames)
    av::av_capture_graphics(
      for (i in 1:numFrames) {
        cursorLines <- list()
        for (frame in frames) cursorLines[[frame]] <- list(
          frame, v = cursorPositions[i], col = cursor_color, lwd = cursor_lwd)
        praatpicture(sound, start, end, frames = frames,
                     draw_lines = cursorLines, tfrom0 = FALSE,
                     ...)
        if (verbose) print(paste('Frame', i, 'generated'))
        },
      output=outputFile, audio=audioFile, verbose=FALSE,
      width=width, height=height, pointsize=pointsize,
      framerate=cursor_frameRate)
  } else {
    av::av_capture_graphics(praatpicture(sound, start, end, frames = frames,
                                         draw_lines = draw_lines, ...),
                            output=outputFile, audio=audioFile, verbose=FALSE,
                            width=width, height=height, pointsize=pointsize,
                            framerate=1/dur)
  }

  if (audio_start != start | audio_end != end) unlink('tmp.wav', force=TRUE)

  if (useViewer) {
    writeLines(text='<video width="550" height="400" controls>\n<source src="output.mp4">\n</video>', con=htmlFile)
    rstudioapi::viewer(htmlFile)
  }

}

#' Plot intensity object
#'
#' Function for plotting intensity objects called by [praatpicture]. Instead of
#' using this function directly, just use
#' `praatpicture('my_sound_file', frames='intensity')`.
#'
#' @param it IntensityTier object loaded using [rPraat::it.read] or other object
#' formatted in a similar way, i.e. a `list` object containing the elements
#' `t` (a vector of time values) and `i` (a vector of intensity values) of
#' identical length.
#' @param start Start time (in seconds) of desired plotted area.
#' @param end End time (in seconds) of desired plotted area.
#' @param tfrom0 Logical; should time on the x-axis run from 0 or from the
#' original time? Default is `TRUE`.
#' @param tgbool Logical; should dotted lines be plotted corresponding to
#' locations in a TextGrid? Default is `FALSE`.
#' @param lines Numeric vector giving locations in seconds of locations from
#' a TextGrid to be plotted with dotted lines. Default is `NULL`.
#' @param focusTierColor String or vector of strings giving the color(s) to
#' use for plotting focus tier lines. If multiple tiers are focused, a vector
#' of the same length can be passed, and the nth tier will be plotted in the
#' nth color. Default is `'black'`.
#' @param focusTierLineType String or vector of strings giving the line
#' type(s) for plotting focus tier lines. If multiple tiers are focused, a
#' vector of the same length can be passed, and the nth tier will be plotted in
#' the nth line type. Default is `'dotted'`.
#' @param range Vector of two integers giving the intensity range to be
#' used for producing intensity plots. Default is `NULL`, in which case the
#' range is simply the minimum and maximum levels in the curve.
#' @param color String giving the name of the color to be used for
#' plotting intensity. Default is `'black'`.
#' @param ind Integer indexing current plot frame relative to other plot
#' components. Default is `NULL`.
#' @param min_max_only Logical; should only minimum and maximum values be given
#' on the y-axis? Default is `TRUE`. Can also be a logical vector if some but
#' not all plot components should have minimum and maximum values on the y-axis.
#' Ignored for TextGrid component.
#' @param highlight Named list giving parameters for differential
#' highlighting of the intensity contour based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `color`) and
#' `drawSize` (integer), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param axisLabel String giving the name of the label to print along
#' the y-axis when plotting intensity. Default is `Intensity (dB)`.
#' @param drawSize Number indicating the line width of the intensity contour.
#' Default is `1`. Controls the `lwd` argument of [graphics::lines].
#' @param suppressAxes Should all axes and the box around plots be suppressed?
#' Default is `FALSE`. Generally not recommended.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, frames='intensity')
intensityplot <- function(it, start, end, tfrom0=TRUE, tgbool=FALSE, lines=NULL,
                          focusTierColor='black', focusTierLineType='dotted',
                          range=NULL, color='black', ind=NULL, min_max_only=TRUE,
                          highlight=NULL,
                          axisLabel='Intensity (dB)', drawSize=1,
                          suppressAxes=FALSE) {

  if (!min_max_only[ind]) {
    if (ind == 1) {
      yax <- 's'
    } else {
      ytix <- grDevices::axisTicks(range, log=F)
      ytix <- ytix[-length(ytix)]
      yax <- 'n'
    }
  } else {
    yax <- 'n'
    ytix <- round(range, 0)
  }

  if (tfrom0) {
    it$t <- it$t - start
    start <- 0
  }

  create_highlighting <- function(highlight, it) {
    hl_t <- c()
    hl_i <- c()
    for (int in 1:length(highlight$start)) {
      times <- which(it$t > highlight$start[int] & it$t < highlight$end[int])
      if (highlight$start[int] < start) {
        extrema <- zoo::na.approx(c(NA, it$i),
                                  c(highlight$end[int], it$t))
        hl_t <- c(hl_t, it$t[times], highlight$end[int],
                  max(it$t[times] + 0.0001))
        hl_i <- c(hl_i, it$i[times], extrema[1], NA)
      } else {
        extrema <- zoo::na.approx(c(NA, NA, it$i),
                                  c(highlight$start[int], highlight$end[int],
                                    it$t))[1:2]
        hl_t <- c(hl_t, highlight$start[int], it$t[times], highlight$end[int],
                  max(it$t[times] + 0.0001))
        hl_i <- c(hl_i, extrema[1], it$i[times], extrema[2], NA)
      }
    }
    if (any(highlight$start < start)) {
      highlight$start[which(highlight$start < start)] <- start
    }
    if (any(highlight$end > end)) {
      highlight$end[which(highlight$end > end)] <- end
    }
    if (!'color' %in% names(highlight)) highlight$color <- color
    if (!'drawSize' %in% names(highlight)) highlight$drawSize <- drawSize
    return(list(params = highlight, hl_t = hl_t, hl_i = hl_i))
  }

  plot(it$t, it$i, xlim=c(start, end+start), xaxt='n',
       ylim=range, yaxt=yax, type='l', col=color, lwd=drawSize)
  if (!suppressAxes) {
    if (!min_max_only[ind] & ind != 1) graphics::axis(2, at=ytix)
    if (min_max_only[ind]) graphics::axis(2, at=ytix, padj=c(0,1), las=2,
                                          tick=F)
  }

  if (!is.null(highlight)) {
    if (any(sapply(highlight, is.list))) {
      highlightLoopDepth <- sum(sapply(highlight, is.list))
    } else {
      highlightLoopDepth <- 1
    }
    for (i in 1:highlightLoopDepth) {
      if (any(sapply(highlight, is.list))) {
        thisHL <- create_highlighting(highlight[[i]], it)
      } else {
        thisHL <- create_highlighting(highlight, it)
      }
      if ('background' %in% names(thisHL$params)) {
        graphics::rect(thisHL$params$start,
                       range[1] - range[2] * 2,
                       thisHL$params$end,
                       range[2] + range[2] * 2,
                       col = thisHL$params$background, border = NA)
      }
      graphics::lines(thisHL$hl_t, thisHL$hl_i,
                      col=thisHL$params$color,
                      lwd=thisHL$params$drawSize)
    }
  }

  if (tgbool) {
    for (i in 1:length(lines)) {
      graphics::abline(v=lines[[i]], col=focusTierColor[i],
                       lty=focusTierLineType[i])
    }
  }

  if (!suppressAxes) graphics::mtext(axisLabel, side=2, line=3.5, cex=0.8)

}

#' Plot pitch object
#'
#' Function for plotting pitch objects called by [praatpicture]. Instead of
#' using this function directly, just use
#' `praatpicture('my_sound_file', frames='pitch')`.
#'
#' @param pt PitchTier object loaded using [rPraat::pt.read] or other object
#' formatted in a similar way, i.e. a `list` object containing the elements
#' `t` (a vector of time values) and `f` (a vector of frequency values) of
#' identical length.
#' @param start Start time (in seconds) of desired plotted area.
#' @param end End time (in seconds) of desired plotted area.
#' @param tfrom0 Logical; should time on the x-axis run from 0 or from the
#' original time? Default is `TRUE`.
#' @param tgbool Logical; should dotted lines be plotted corresponding to
#' locations in a TextGrid? Default is `FALSE`.
#' @param lines Numeric vector giving locations in seconds of locations from
#' a TextGrid to be plotted with dotted lines. Default is `NULL`.
#' @param focusTierColor String or vector of strings giving the color(s) to
#' use for plotting focus tier lines. If multiple tiers are focused, a vector
#' of the same length can be passed, and the nth tier will be plotted in the
#' nth color. Default is `'black'`.
#' @param focusTierLineType String or vector of strings giving the line
#' type(s) for plotting focus tier lines. If multiple tiers are focused, a
#' vector of the same length can be passed, and the nth tier will be plotted in
#' the nth line type. Default is `'dotted'`.
#' @param plotType String giving the type of pitch plot to produce; default
#' is `draw` (a line plot), the only other option is `speckle` (a point plot).
#' Alternatively a vector `c('draw','speckle')` can be passed, in which case
#' both are used.
#' @param scale String giving the frequency scale to use when producing
#' pitch plots. Default is `hz`; other options are `logarithmic` (also in Hz),
#' `semitones`, `erb`, and `mel`.
#' @param freqRange Vector of two integers giving the frequency range to be
#' used for producing pitch plots. Default is `NULL`, in which case the pitch
#' range is automatically reset to `c(-12,30)` for the `semitones` scale,
#' `c(0,10)` for the `erb` scale, and `c(50,500)` for the Hz-based scales,
#' following Praat defaults.
#' @param semitonesRe Frequency in Hz giving the reference level for converting
#' pitch frequency to semitones. Default is `100`.
#' @param color String giving the name of the color to be used for
#' plotting pitch. Default is `'black'`.
#' @param ind Integer indexing pitch relative to other plot components.
#' Default is `NULL`.
#' @param min_max_only Logical; should only minimum and maximum values be given
#' on the y-axis? Default is `TRUE`. Can also be a logical vector if some but
#' not all plot components should have minimum and maximum values on the y-axis.
#' Ignored for TextGrid component.
#' @param highlight Named list giving parameters for differential
#' highlighting of pitch based on the time domain. This list
#' should contain information about which parts of the plot to highlight, either
#' done with the `start` and `end` arguments which must be numbers or numeric
#' vectors, or using the `tier` and `label` arguments to highlight based on
#' information in a plotted TextGrid. Further contains the optional arguments
#' `color` (string or vector of strings, see `color`),
#' `drawSize` or `speckleSize` (both numeric), and `background`
#' (a string specifying a background color). Can be nested lists to highlight
#' different parts of a figure in different ways.
#' @param axisLabel String giving the name of the label to print along the
#' y-axis when printing a pitch track. Default is `NULL`, in which case the
#' axis label will depend on the scale.
#' @param drawSize Number indicating the line width if
#' `plotType` is `'draw'`. Default is `1`. Controls the `lwd` argument of
#' [graphics::lines].
#' @param speckleSize Number indicating the point size of if `_plotType` is
#' `'speckle'`. Default is `1`. Controls the `cex` arguments of
#' [graphics::points].
#' @param suppressAxes Should all axes and the box around plots be suppressed?
#' Default is `FALSE`. Generally not recommended.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, frames='pitch')
pitchplot <- function(pt, start, end, tfrom0=TRUE, tgbool=FALSE, lines=NULL,
                      focusTierColor='black', focusTierLineType='dotted',
                      plotType='draw', scale='hz', freqRange=NULL,
                      semitonesRe=100, color='black', ind=NULL,
                      min_max_only=TRUE, highlight=NULL,
                      axisLabel=NULL, drawSize=1, speckleSize=1,
                      suppressAxes=FALSE) {

  if (scale == 'logarithmic') {
    logsc <- 'y'
  } else {
    logsc <- ''
  }

  if (!min_max_only[ind]) {
    if (ind == 1 | scale == 'logarithmic') {
      yax <- 's'
    } else {
      ytix <- grDevices::axisTicks(freqRange, log=F)
      ytix <- ytix[-length(ytix)]
      yax <- 'n'
    }
  } else {
    yax <- 'n'
    ytix <- freqRange
  }

  if (tfrom0) {
    pt$t <- pt$t - start
    start <- 0
  }

  create_highlighting <- function(highlight, pt) {
    tSpeckle <- c()
    fSpeckle <- c()
    tDraw <- c()
    fDraw <- c()
    for (int in 1:length(highlight$start)) {
      times <- which(pt$t > highlight$start[int] & pt$t < highlight$end[int])
      tSpeckle <- c(tSpeckle, pt$t[times], max(pt$t[times] + 0.0001))
      fSpeckle <- c(fSpeckle, pt$f[times], NA)
      if (highlight$start[int] < start) {
        extrema <- zoo::na.approx(c(NA, pt$f),
                                  c(highlight$end[int], pt$t))
        tDraw <- c(tDraw, pt$t[times], highlight$end[int],
                   max(pt$t[times] + 0.0001))
        fDraw <- c(fDraw, pt$f[times], extrema[1], NA)
      } else {
        extrema <- zoo::na.approx(c(NA, NA, pt$f),
                                  c(highlight$start[int], highlight$end[int],
                                    pt$t))[1:2]
        tDraw <- c(tDraw, highlight$start[int], pt$t[times], highlight$end[int],
                   max(pt$t[times] + 0.0001))
        fDraw <- c(fDraw, extrema[1], pt$f[times], extrema[2], NA)
      }
      if (any(highlight$start < start)) {
        highlight$start[which(highlight$start < start)] <- start
      }
      if (any(highlight$end > end)) {
        highlight$end[which(highlight$end > end)] <- end
      }
    }

    if (!'color' %in% names(highlight)) highlight$color <- color
    if (!'drawSize' %in% names(highlight)) highlight$drawSize <- drawSize
    if (!'speckleSize' %in% names(highlight)) highlight$speckleSize <-
      speckleSize

    return(list(params = highlight, tSpeckle = tSpeckle, fSpeckle = fSpeckle,
                tDraw = tDraw, fDraw = fDraw))
  }

  if ('draw' %in% plotType) {
    plot(pt$t, pt$f, xlim=c(start, end+start), xaxt='n', ylim=freqRange,
         yaxt=yax, type='l', log=logsc, col=color, lwd=drawSize)
    if (!suppressAxes) {
      if (ind != 1 & scale != 'logarithmic' & !min_max_only[ind]) {
        graphics::axis(2, at=ytix)
      }
      if (min_max_only[ind]) graphics::axis(2, at=ytix, las=2, padj=c(0,1),
                                            tick=F)
    }

    if ('speckle' %in% plotType) {
      graphics::points(pt$t, pt$f, pch=20, col=color, cex=speckleSize)
    }

    if (!is.null(highlight)) {
      if (any(sapply(highlight, is.list))) {
        highlightLoopDepth <- sum(sapply(highlight, is.list))
      } else {
        highlightLoopDepth <- 1
      }
      for (i in 1:highlightLoopDepth) {
        if (any(sapply(highlight, is.list))) {
          thisHL <- create_highlighting(highlight[[i]], pt)
        } else {
          thisHL <- create_highlighting(highlight, pt)
        }
        if ('background' %in% names(thisHL$params)) {
          graphics::rect(thisHL$params$start,
                         freqRange[1] - freqRange[2] * 2,
                         thisHL$params$end,
                         freqRange[2] + freqRange[2] * 2,
                         col = thisHL$params$background, border = NA)
        }
        graphics::lines(thisHL$tDraw, thisHL$fDraw,
                        col=thisHL$params$color,
                        lwd=thisHL$params$drawSize)
        if ('speckle' %in% plotType) {
          graphics::points(thisHL$tSpeckle, thisHL$fSpeckle, pch=20,
                           col=thisHL$params$color,
                           cex=thisHL$params$speckleSize)
        }
      }
    }

    if (tgbool) {
      for (i in 1:length(lines)) {
        graphics::abline(v=lines[[i]], col=focusTierColor[i],
                         lty=focusTierLineType[i])
      }
    }
  }

  if (length(plotType) == 1) {
    if (plotType == 'speckle') {
      plot(pt$t, pt$f, xlim=c(start, end+start), xaxt='n', ylim=freqRange,
           yaxt=yax, type='p', pch=20, log=logsc, col=color, cex=speckleSize)
      if (!suppressAxes) {
        if (ind != 1 & scale != 'logarithmic' & !min_max_only[ind]) {
          graphics::axis(2, at=ytix)
        }
        if (min_max_only[ind]) graphics::axis(2, at=ytix, las=2, padj=c(0,1),
                                              tick=F)
      }

      if (!is.null(highlight)) {
        if (any(sapply(highlight, is.list))) {
          highlightLoopDepth <- sum(sapply(highlight, is.list))
        } else {
          highlightLoopDepth <- 1
        }
        for (i in 1:highlightLoopDepth) {
          if (any(sapply(highlight, is.list))) {
            thisHL <- create_highlighting(highlight[[i]], pt)
          } else {
            thisHL <- create_highlighting(highlight, pt)
          }
          if ('background' %in% names(thisHL$params)) {
            graphics::rect(thisHL$params$start,
                           freqRange[1] - freqRange[2] * 2,
                           thisHL$params$end,
                           freqRange[2] + freqRange[2] * 2,
                           col = thisHL$params$background, border = NA)
          }
          graphics::points(thisHL$tSpeckle, thisHL$fSpeckle,
                           col=thisHL$params$color, pch=20,
                           cex=thisHL$params$speckleSize)
          if ('draw' %in% plotType) {
            graphics::lines(thisHL$tDraw, thisHL$fDraw, col=thisHL$params$color,
                            lwd=thisHL$params$drawSize)
          }
        }
      }

      if (tgbool) {
        for (i in 1:length(lines)) {
          graphics::abline(v=lines[[i]], col=focusTierColor[i],
                           lty=focusTierLineType[i])
        }
      }
    }
  }

  if (!suppressAxes) graphics::mtext(axisLabel, side=2, line=3.5, cex=0.8)

}

#' Interactively create a TextGrid tier
#'
#' Function for creating TextGrid tiers called by [make_TextGrid]. Instead of
#' using this function directly, use [make_TextGrid].
#'
#' @param sound String giving the file name of a sound file with the .wav
#' extension.
#' @param tierName String giving the name of the tier.
#' @param start Start time (in seconds) of desired plotted area. Default is `0`.
#' @param end End time (in seconds) of desired plotted area. Default is `0`
#' (= the entire file).
#' @param show String giving the type of plot to show. Default is `wave`,
#' another option is `spectrogram`. Note that spectrogram plotting is relatively
#' slow within this function.
#' @param channel Number indicating which audio channel to show. Default is `1`.
#' @param sampa2ipa Logical; should SAMPA transcriptions be converted to IPA?
#' Default is `FALSE`.
#'
#' @return A list object identical to a single tier created by
#' [rPraat::tg.read()] when
#' loading TextGrid objects into R.
#' @export
#'
#' @examples
#' \dontrun{
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/2.wav')
#' tg <- make_TextGrid(soundFile, tierNames='Mary')
#' # Follow the steps shown in the console
#'
#' praatpicture(soundFile, tg_obj=tg)
#' }
tg_createTier <- function(sound, tierName, start=0, end=0, show='wave',
                          channel=1, sampa2ipa=FALSE) {

  if (end == 0) end <- Inf
  snd <- tuneR::readWave(sound, from=start, to=end, units='seconds', toWaveMC=T)
  sig <- snd@.Data[,channel]
  sr <- snd@samp.rate

  grDevices::X11()
  dur <- length(sig) / sr

  if (show == 'wave') {
    plot(y=sig, x=seq(0, dur, length.out=length(sig)), type='l',
         ylim=c(min(sig), max(sig)*1.1),
         xlab='Time (s)', ylab='',
         main='Click Stop -> Stop locator when you are done')
  }

  if (show == 'spectrogram') {
    phonTools::spectrogram(sig, sr, colors=FALSE, window='hamming')
    dur <- dur * 1000
  }

  message(crayon::blue(paste('Navigate to the graphics device window and click',
              'in the',
              'positions where you want TextGrid boundaries for tier',
              tierName)))
  l <- graphics::locator()
  t0 <- l$x
  graphics::abline(v=t0, lty='dotted')
  tier <- list()
  tier$name <- tierName
  intbool <- readline('Is this an interval tier? [y/n] ')
  if (intbool == 'y') {
    tier$type <- 'interval'
    tier$t1 <- c(0, t0)
    tier$t2 <- c(t0, dur)
    for (i in 1:(length(tier$t1)-2)) {
      tier$label[i] <- readline(paste0('Label ', i, ' of ',
                                       length(tier$t1)-2, ': '))
    }
    tier$label <- c('', tier$label, '')
    if (sampa2ipa) tier$label <- ipa::sampa(tier$label)
    if (show == 'wave') graphics::text(tier$t1+(tier$t2-tier$t1)/2, max(sig)*1.1,
                                       tier$label, font=2)
    if (show == 'spectrogram') graphics::text(tier$t1+(tier$t2-tier$t1)/2, 4500,
                                              tier$label, font=2, col='blue')
    if (show == 'spectrogram') {
      tier$t1 <- tier$t1 / 1000
      tier$t2 <- tier$t2 / 1000
    }
    tier$t1 <- start + tier$t1
    tier$t2 <- start + tier$t2
  } else {
    tier$type <- 'point'
    tier$t <- t0
    for (i in 1:length(tier$t)) {
      tier$label[i] <- readline(paste0('Label ', i, ' of ',
                                       length(tier$t), ': '))
    }
    if (sampa2ipa) tier$label <- ipa::sampa(tier$label)
    if (show == 'wave') graphics::text(tier$t, max(sig)*1.1, tier$label, font=2)
    if (show == 'spectrogram') graphics::text(tier$t, 4500, tier$label, font=2,
                                              col='blue')
    if (show == 'spectrogram') tier$t <- tier$t / 1000
    tier$t <- start + tier$t
  }
  dummy <- readline('Check the resulting annotation and type enter! ')
  grDevices::dev.off()
  return(tier)
}

#' Draw straight lines on plot component
#'
#' Helper function for adding straight lines to plot components made with
#' praatpicture. Do not use directly, instead use [praatpicture] with the
#' `draw_lines` argument.
#'
#' @param plot_component String giving the name of the plot component to
#' draw on.
#' @param args A list of vectors giving arguments used for drawing
#' straight lines. See [praatpicture] documentation.
#'
#' @return No return values, called internally by [praatpicture] and sibling
#' functions.
#' @export
#'
#' @examples
#' # Don't use directly
#' datapath <- system.file('extdata', package='praatpicture')
#' soundFile <- paste0(datapath, '/1.wav')
#' praatpicture(soundFile, draw_lines=c('spectrogram',
#' h=seq(0,5000,by=1000), lty='dashed'))
draw_lines <- function(plot_component, args) {
  r <- which(sapply(args, '[[', 1) == plot_component)
  for (i in r) {
    params <- args[[i]]
    params <- as.list(params[2:length(params)])
    do.call(eval(parse(text='graphics::abline')), params)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/talking_praatpicture.R
\name{talking_praatpicture}
\alias{talking_praatpicture}
\title{Make Praat Picture style plots of acoustic data with embedded audio}
\usage{
talking_praatpicture(
  sound,
  start = 0,
  end = 0,
  frames = c("sound", "spectrogram", "TextGrid"),
  draw_lines = list("formant", h = seq(0, 10000, by = 1000), lty = "dotted"),
  audio_start = start,
  audio_end = end,
  width = 1080,
  height = 720,
  pointsize = 25,
  cursor = FALSE,
  cursor_frameRate = 50,
  cursor_color = "black",
  cursor_lwd = 2,
  verbose = TRUE,
  outputFile = "praatvid.mp4",
  useViewer = TRUE,
  ...
)
}
\arguments{
\item{sound}{String giving the file name of a sound file with the .wav
extension.}

\item{start}{Start time (in seconds) of desired plotted area. Default is \code{0}.}

\item{end}{End time (in seconds) of desired plotted area. Default is \code{0}
(= the entire file).}

\item{frames}{String or vector of strings giving the frames that the plot
should consist of. Default is \code{sound}, \code{spectrogram}, \code{TextGrid}. This
requires a file with the extension \code{.TextGrid} and the same base name as the
sound file. Other options are \code{pitch}, \code{formant}, and \code{intensity}. See
details for more information.}

\item{draw_lines}{Use for drawing straight lines on plot components. Takes
an argument of type \code{list} which should contain a) a string giving the plot
component to draw straight lines on, and b) arguments to pass on to
\link[graphics:abline]{graphics::abline}. Should have a named argument \code{h} for horizontal lines,
or \code{v} for vertical lines, or \code{a},\code{b} for the intercept and slope of the
line otherwise. Alternatively a nested list can be passed if more (sets of)
lines should be drawn. If multiple audio
channels are plotted and lines should be added to one of these,
use the channel identifier instead of a string giving the frame to draw on.
The default value is \code{list('formant', h=seq(0,10000,by=1000), lty='dotted')}.
According to Praat defaults, this means that if formants are plotted in a
separate frame, horizontal dotted lines (\code{lty}) are shown at 1000 Hz
intervals. To override this behavior, simply pass \code{draw_lines=NULL}.
The argument is ignored if \code{cursor = TRUE}.}

\item{audio_start}{Start time (in seconds) of embedded audio. By default
it is the same as \code{start}, i.e. the embedded audio is the portion of the
sound file that is being plotted.}

\item{audio_end}{End time (in seconds) of embedded audio. By default it is
the same as \code{end}, i.e. the embedded audio is the portion of the sound
that is being plotted.}

\item{width}{Number giving the desired width of the resulting animation in
pixels; default is \code{1080}.}

\item{height}{Number giving the desired height of the resulting animation in
pixels; default is \code{720}.}

\item{pointsize}{Number; which point size should be used for text in the
animation? Default is \code{25}. See \code{\link[grDevices:png]{grDevices::png()}} for more details.}

\item{cursor}{Logical; should a moving cursor show the current position in
the sound file? Default is \code{FALSE}.}

\item{cursor_frameRate}{Number giving the desired number of frames per second
for moving cursor. Default is \code{50}.}

\item{cursor_color}{String giving the color of the moving cursor. Default is
\code{black}.}

\item{cursor_lwd}{Number giving the line width of the moving cursor. Default
is \code{2}.}

\item{verbose}{Logical; should status messages be printed in the console as
figures are being generated? Default is \code{TRUE}.}

\item{outputFile}{String giving the desired file name. Default is
\code{praatvid.mp4}.}

\item{useViewer}{Logical; should the video be shown in the Viewer pane in
RStudio? Default is \code{TRUE}; if true, the video is oSnly saved in a
temporary directory, but can be downloaded from a browser.}

\item{...}{Further arguments passed to \code{praatpicture}.}
}
\value{
No return value, produces a video file.
}
\description{
Generate simple MP4 video files with Praat Picture style plots of acoustic
data with time-aligned transcriptions and embedded audio to use in
presentations etc.
}
\examples{
\dontrun{
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
talking_praatpicture(soundFile)
}
}
\seealso{
This function is a wrapper for \code{\link[av:capturing]{av::av_capture_graphics()}} used to
produce plots similar to those made with \code{\link[=praatpicture]{praatpicture()}} with embedded
audio. For more detail on your
options, see the \code{praatpicture()} help file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny_praatpicture.R
\name{shiny_praatpicture}
\alias{shiny_praatpicture}
\title{Run praatpicture as Shiny app}
\usage{
shiny_praatpicture()
}
\value{
No return values
}
\description{
Interactive version of \link{praatpicture}
}
\examples{
\dontrun{
shiny_praatpicture()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/praatpicture.R
\name{praatpicture}
\alias{praatpicture}
\title{Make Praat Picture style plots of acoustic data}
\usage{
praatpicture(
  sound,
  start = 0,
  end = 0,
  tfrom0 = TRUE,
  tUnit = "s",
  frames = c("sound", "spectrogram", "TextGrid"),
  proportion = c(30, 50, 20),
  mainTitle = "",
  mainTitleAlignment = 0,
  start_end_only = TRUE,
  min_max_only = TRUE,
  drawSize = 1,
  speckleSize = 1,
  wave_channels = "all",
  wave_channelNames = FALSE,
  wave_energyRange = NULL,
  wave_axisDigits = 3,
  wave_color = "black",
  wave_lineWidth = 1,
  wave_highlight = NULL,
  tg_obj = NULL,
  tg_file = NULL,
  tg_tiers = "all",
  tg_focusTier = tg_tiers[1],
  tg_focusTierColor = "black",
  tg_focusTierLineType = "dotted",
  tg_tierNames = TRUE,
  tg_alignment = "central",
  tg_edgeLabels = "keep",
  tg_specialChar = FALSE,
  tg_color = "black",
  tg_highlight = NULL,
  spec_channel = NULL,
  spec_scale = "hz",
  spec_freqRange = NULL,
  spec_windowLength = 0.005,
  spec_dynamicRange = 50,
  spec_timeStep = 1000,
  spec_windowShape = "Gaussian",
  spec_colors = c("white", "black"),
  spec_axisLabel = NULL,
  spec_highlight = NULL,
  pitch_timeStep = NULL,
  pitch_floor = 75,
  pitch_ceiling = 600,
  pitch_plotType = "draw",
  pitch_scale = "hz",
  pitch_freqRange = NULL,
  pitch_semitonesRe = 100,
  pitch_color = "black",
  pitch_plotOnSpec = FALSE,
  pitch_plotOnWave = FALSE,
  pitch_ssff = NULL,
  pitch_axisLabel = NULL,
  pitch_highlight = NULL,
  formant_timeStep = NULL,
  formant_maxN = 5,
  formant_windowLength = 0.025,
  formant_dynamicRange = 30,
  formant_freqRange = c(50, 5500),
  formant_number = NULL,
  formant_plotType = "speckle",
  formant_color = "black",
  formant_plotOnSpec = FALSE,
  formant_ssff = NULL,
  formant_axisLabel = "Frequency (Hz)",
  formant_highlight = NULL,
  intensity_timeStep = NULL,
  intensity_minPitch = 100,
  intensity_range = NULL,
  intensity_color = "black",
  intensity_plotOnSpec = FALSE,
  intensity_plotOnWave = FALSE,
  intensity_ssff = NULL,
  intensity_axisLabel = "Intensity (dB)",
  intensity_highlight = NULL,
  time_axisLabel = NULL,
  highlight = NULL,
  suppressAxes = FALSE,
  draw_lines = list("formant", h = seq(0, 10000, by = 1000), lty = "dotted"),
  draw_rectangle = NULL,
  draw_arrow = NULL,
  annotate = NULL,
  gender = "u",
  ...
)
}
\arguments{
\item{sound}{String giving the file name of a sound file with the .wav
extension.}

\item{start}{Start time (in seconds) of desired plotted area. Default is \code{0}.}

\item{end}{End time (in seconds) of desired plotted area. Default is \code{0}
(= the entire file).}

\item{tfrom0}{Logical; should time on the x-axis run from 0 or from the
original time? Default is \code{TRUE}.}

\item{tUnit}{String giving the unit of time to print along the x-axis.
Possible options are \code{'s'} (default) for seconds and \code{'ms'} for milliseconds.}

\item{frames}{String or vector of strings giving the frames that the plot
should consist of. Default is \code{sound}, \code{spectrogram}, \code{TextGrid}. This
requires a file with the extension \code{.TextGrid} and the same base name as the
sound file. Other options are \code{pitch}, \code{formant}, and \code{intensity}. See
details for more information.}

\item{proportion}{Integer or vector of integers of the same size as \code{frames}
giving the proportion in percents of the plotting area to be taken up by the
individual frames. Default is \code{c(30,50,20)}. If more or less than three
frames are plotted and no proportions are given, frames will be of equal
size.}

\item{mainTitle}{String giving a title to print at the top of the plot.
The default is an empty string, i.e. no title.}

\item{mainTitleAlignment}{Number indicating the vertical alignment of the
plot title, where \code{0} (default) indicates left-alignment, \code{1} indicates
right-alignment, \code{0.5} indicates central alignment, etc, following the
conventions of the \code{adj} argument of \link[graphics:mtext]{graphics::mtext}.}

\item{start_end_only}{Logical; should there only be ticks on the x-axis
for start and end times? Default is \code{TRUE}.}

\item{min_max_only}{Logical; should only minimum and maximum values be given
on the y-axis? Default is \code{TRUE}. Can also be a logical vector if some but
not all plot components should have minimum and maximum values on the y-axis.
Ignored for TextGrid component.}

\item{drawSize}{Number indicating the line width of plot components where
the \verb{_plotType} is \code{'draw'} (i.e., pitch, formants, or intensity rendered as
line plots). Default is \code{1}. Controls the \code{lwd} argument of
\link[graphics:lines]{graphics::lines}.}

\item{speckleSize}{Number indicating the point size of plot components where
the \verb{_plotType} is \code{'speckle'} (i.e. pitch or formants rendered as point
plots). Default is \code{1}. Controls the \code{cex} arguments of \link[graphics:points]{graphics::points}.}

\item{wave_channels}{Vector of numbers or strings giving either numeric
identifiers of audio channels to plot of the names of audio channels to plot.
Also understands \code{'all'}, which plots all channels and is the default.}

\item{wave_channelNames}{Should names of audio channels be printed
on the y-axis? If \code{TRUE}, names will be grabbed from the audio metadata if
available. Alternatively, if two channels are available, they will be named
\code{left} and \code{right}. If more or less than two channels are available,
channels are named \code{Cn}, where \code{n} is the number of the channel. Alternatvely,
a vector of strings can be provided with channel names. Default is \code{FALSE}.}

\item{wave_energyRange}{Numeric vector giving the desired energy
range (y-axis range) of waveform(s). Default is \code{NULL}, in which case the
y-axis range is set to the lowest and highest value for each wave.
If a vector of two values is passed for a multi-wave plot, these are
recycled for each wave. Separate ranges can be set for each waveform by
passing a vector of a length corresponding to twice the number of waves
plotted.}

\item{wave_axisDigits}{Numeric giving the number of digits to print for
values along the y-axis of the waveform. Default is \code{3}. If \code{0} is passed,
the y-axis is suppressed. Alternatively a vector of numbers, if different
numbers should be used for different channels. Note that this only
applies when \code{min_max_only = TRUE}, as otherwise the look of the y-axis is
determined entirely using \code{grDevices::axisTicks()}.}

\item{wave_color}{String giving the name of the color to be used for plotting
the waveform. Default is \code{'black'}. Alternatively a vector of strings, if
different colors should be used for different channels.}

\item{wave_lineWidth}{Number giving the line width to use for plotting
the waveform. Default is \code{1}. Alternatively a vector of numbers, if
different line widths should be used for different channels.}

\item{wave_highlight}{Named list giving parameters for differential
highlighting of the waveform based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the argument
\code{color} (string, see \code{wave_color}), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{tg_obj}{A TextGrid object returned by the \code{\link[=make_TextGrid]{make_TextGrid()}} function.}

\item{tg_file}{Path of file to be used for plotting TextGrid. Default is
\code{NULL}, in which case the function searches for a TextGrid sharing the same
base name as \code{sound} with the \code{.TextGrid} extension.}

\item{tg_tiers}{Vector of numbers or strings giving either numeric identifiers
of TextGrid tiers to plot or the names of TextGrid tiers to plot. Also
understands \code{'all'}, which plots all tiers and is the default.}

\item{tg_focusTier}{For which tier(s) should lines be shown on all
acoustic plots giving the locations of boundaries? Vector of number or
strings giving either numeric identifiers
of TextGrid tiers or the names of TextGrid tiers to plot. Default is
\code{tg_tiers[1]}, i.e. the first tier given in the \code{tg_tiers} argument.
Additionally accepts the string \code{none},
in which case no lines are shown on acoustic plots, and \code{all}, in
which case lines from all tiers are shown on acoustic plots.}

\item{tg_focusTierColor}{String or vector of strings giving the color(s) to
use for plotting focus tier lines. If multiple tiers are focused, a vector
of the same length can be passed, and the nth tier will be plotted in the
nth color. Default is \code{'black'}.}

\item{tg_focusTierLineType}{String or vector of strings giving the line
type(s) for plotting focus tier lines. If multiple tiers are focused, a
vector of the same length can be passed, and the nth tier will be plotted in
the nth line type. Default is \code{'dotted'}.}

\item{tg_tierNames}{Logical; should TextGrid tier names be printed along the
y-axis? Default is \code{TRUE}.}

\item{tg_alignment}{String giving the desired alignment of text in the
TextGrids. Default is \code{central}; other options are \code{left} and \code{right}.
Alternatively, a vector of strings if different alignments are needed.}

\item{tg_edgeLabels}{String specifying how to handle TextGrid labels in
interval tiers that fall partially before \code{start} or partially after \code{end}.
Default is \code{'keep'}, where labels are kept at the center of the interval.
Other options are \code{'center'}, where labels are recentered to the visible
part of the interval, or \code{'discard'}, where such labels are ignored.}

\item{tg_specialChar}{Logical; should Praat typesetting for special font types
such as italic, bold, and small caps be converted into corresponding
R-readable special font types. Default is \code{FALSE}, since special characters
can create unfortunate text alignment artefacts.
See https://www.fon.hum.uva.nl/praat/manual/Text_styles.html.}

\item{tg_color}{String or vector of strings giving the name of the color(s)
to be used for the text in TextGrids. Default is \code{'black'}. If a vector is
provided, different colors are used for different tiers.}

\item{tg_highlight}{Named list giving parameters for differential
highlighting of TextGrid intervals. This list
should contain information about which intervals to highlight, using the
\code{tier} and \code{label}. Further contains the argument
\code{color}, and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{spec_channel}{Numeric giving the channel that should be used to
generate the spectrogram. Default is \code{1}. Generating spectrograms from
multiple channels is not currently possible with \code{praatpicture}.}

\item{spec_scale}{String giving the frequency scale to use for plotting
spectrograms. Default is \code{hz}. Alternatives are \code{erb} (for the equivalent
rectangular bandwidth scale), \code{mel} (for the Mel scale, using 1000 Hz
as the corner frequency, following Fant 1968), or \code{logarithmic} (for
log Hz).}

\item{spec_freqRange}{Vector of two integers giving the frequency range to be
used for plotting spectrograms. Default is \code{NULL}, in which case suitable
ranges are chosen based on the frequency scale. For \code{hz}, the default is
\code{c(0,5000)}; for \code{erb}, the default is \code{c(0.8,42)}; for \code{mel}, the default is
\code{c(30,4400)}. The latter two correspond roughly to the human auditory
frequency response.}

\item{spec_windowLength}{Window length in seconds for generating spectrograms.
Default is \code{0.005}.}

\item{spec_dynamicRange}{Dynamic range in dB for generating spectrograms. The
maximum intensity minus \code{spec_dynamicRange} will all be printed in white.
Default is \code{50}.}

\item{spec_timeStep}{How many time steps should be calculated for spectrograms?
Default is \code{1000}.}

\item{spec_windowShape}{String giving the name of the window shape to be
applied to the signal when generating spectrograms. Default is \code{Gaussian};
other options are \code{square}, \code{Hamming}, \code{Bartlett}, \code{Hanning}, or \code{Blackman}.
Note that the Gaussian window function provided by the \code{phonTools} package
and used in \code{praatpicture()} does not have the same properties as the
Gaussian window function used for spectral estimation in Praat; plotting
a simple sine wave with high dynamic range will produce sidelobes in
\code{praatpicture()} but not in Praat. It's recommended to use Blackman windows
instead if you have this problem.}

\item{spec_colors}{Vector of strings giving the names of colors to be used
for plotting the spectrogram; default is \code{c('white', 'black')}. The first
value is used for plotting the lowest visible amplitude, and the last for
plotting the highest visible amplitude. Vectors with more than two color
names can be used for plotting values in between in different colors.}

\item{spec_axisLabel}{String giving the name of the label to print along the
y-axis when plotting a spectrogram. Default is \code{NULL}, in which case the
axis label will depend on the scale.}

\item{spec_highlight}{Named list giving parameters for differential
highlighting of the spectrogram based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the argument
\code{colors} (vector of strings, see \code{spec_colors}). Can be nested lists to
highlight different parts of a figure in different ways.}

\item{pitch_timeStep}{Measurement interval in seconds for tracking pitch.
Default is \code{NULL}, in which case the measurement interval is equal to
0.75 / \code{pitch_floor}.}

\item{pitch_floor}{Frequency in Hz; no pitch candidates considered below
this frequency. Default is \code{75}.}

\item{pitch_ceiling}{Frequency in Hz; no pitch candidates considered above
this frequency. Default is \code{600}.}

\item{pitch_plotType}{String giving the type of pitch plot to produce; default
is \code{draw} (a line plot), the only other option is \code{speckle} (a point plot).
Alternatively a vector \code{c('draw','speckle')} can be passed, in which case
both are used.}

\item{pitch_scale}{String giving the frequency scale to use when producing
pitch plots. Default is \code{hz}; other options are \code{logarithmic} (also in Hz),
\code{semitones}, \code{erb}, and \code{mel}.}

\item{pitch_freqRange}{Vector of two integers giving the frequency range to be
used for producing pitch plots. Default is \code{NULL}, in which case the pitch
range is automatically reset to \code{c(-12,30)} for the \code{semitones} scale,
\code{c(0,10)} for the \code{erb} scale, and \code{c(50,500)} for the Hz-based scales,
following Praat defaults.}

\item{pitch_semitonesRe}{Frequency in Hz giving the reference level for
converting pitch frequency to semitones. Default is \code{100}.}

\item{pitch_color}{String giving the name of the color to be used for
plotting pitch. Default is \code{'black'}. If \code{pitch_plotOnSpec=TRUE}, axes will
follow the same color scheme. Also if \code{pitch_plotOnSpec=TRUE}, a vector of
two strings can be passed, in which case the second color is used for
background highlighting.}

\item{pitch_plotOnSpec}{Boolean; should pitch be plotted on top of
spectrogram? Default is \code{FALSE}.}

\item{pitch_plotOnWave}{Boolean; should pitch be plotted on top of
waveform? Default is \code{FALSE}.}

\item{pitch_ssff}{An object of class \code{AsspDataObj} containing a pitch track.
Default is \code{NULL}.}

\item{pitch_axisLabel}{String giving the name of the label to print along the
y-axis when printing a pitch track. Default is \code{NULL}, in which case the
axis label will depend on the scale. If \code{pitch_plotOnSpec=TRUE}, this label
will be printed on the right-hand y-axis label.}

\item{pitch_highlight}{Named list giving parameters for differential
highlighting of pitch based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{pitch_color}),
\code{drawSize} or \code{speckleSize} (both numeric), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{formant_timeStep}{Measurement interval in seconds for tracking formants.
Default is \code{NULL}, in which case the measurement interval is equal to
\code{formant_windowLength} / 4.}

\item{formant_maxN}{Integer giving the maximum number of formants to track.
Default is \code{5}.}

\item{formant_windowLength}{The effective duration of the analysis window
used for tracking formants in seconds; the actual duration of the
analysis window is twice this value.}

\item{formant_dynamicRange}{Dynamic range in dB for producing formant plots.
When a formant plot of \code{formant_plotType='speckle'} is drawn, no formants are
shown in frames with intensity level \code{formant_dynamicRange} below the maximum
intensity. Default is \code{30}. If set to \code{0}, all formants are shown.}

\item{formant_freqRange}{Vector of two integers giving the frequency range to
be used for producing formant plots. Default is \code{c(0,5500)}.}

\item{formant_number}{Number of formants to plot. Default is \code{NULL}, in
which case all available formants are plotted.}

\item{formant_plotType}{String giving the type of formant plot to produce;
default is \code{speckle} (a point plot), the only other option is \code{draw} (a line
plot). Alternatively a vector \code{c('draw','speckle')} can be passed, in which
case both are used.}

\item{formant_color}{String or vector of strings giving the name(s) of
colors to be used for plotting formants. If one color is provided, all
formants will be plotted in this color. If multiple colors are provided,
different formants will be shown in different colors. Default is \code{'black'}.
If \code{formant_plotOnSpec=TRUE} and the length of this vector twice the number
of formants plotted, the first
half of strings will be used for the formants' primary colors and the second
half will be used for background highlighting. If the length of this vector
is one more than the number of formants plotted, the last string will
be used for background highlighting.}

\item{formant_plotOnSpec}{Boolean; should formants be plotted on top of
spectrogram? Default is \code{FALSE}.}

\item{formant_ssff}{An object of class \code{AsspDataObj} containing formant tracks.
Default is \code{NULL}.}

\item{formant_axisLabel}{String giving the name of the label to print along the
y-axis when plotting formants. Default is \code{Frequency (Hz)}.}

\item{formant_highlight}{Named list giving parameters for differential
highlighting of formants based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{formant_color}),
\code{drawSize} or \code{speckleSize} (both numeric), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{intensity_timeStep}{Measurement interval in seconds for tracking
intensity. Default is \code{NULL}, in which case the measurement interval is
equal to 0.8 * \code{intensity_minPitch}.}

\item{intensity_minPitch}{Lowest pitch in Hz used when calculating
intensity; default is \code{100}}

\item{intensity_range}{Vector of two integers giving the intensity range to be
used for producing intensity plots. Default is \code{NULL}, in which case the
range is simply the minimum and maximum levels in the curve.}

\item{intensity_color}{String giving the name of the color to be used for
plotting intensity. Default is \code{'black'}. If \code{intensity_plotOnSpec=TRUE},
axes will follow the same color scheme. Also if \code{intensity_plotOnSpec=TRUE},
a vector of two strings can be passed, in which case the second color is used
for background highlighting.}

\item{intensity_plotOnSpec}{Boolean; should intensity be plotted on top of
spectrogram? Default is \code{FALSE}.}

\item{intensity_plotOnWave}{Boolean; should intensity be plotted on top of
waveform? Default is \code{FALSE}.}

\item{intensity_ssff}{An object of class \code{AsspDataObj} containing intensity
tracks. Default is \code{NULL}.}

\item{intensity_axisLabel}{String giving the name of the label to print along
the y-axis when plotting intensity. Default is \code{Intensity (dB)}.
If \code{intensity_plotOnSpec=TRUE}, this label
will be printed on the right-hand y-axis label.}

\item{intensity_highlight}{Named list giving parameters for differential
highlighting of the intensity contour based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{intensity_color}) and
\code{drawSize} (integer), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{time_axisLabel}{String giving the name of the label to print along
the x-axis. Default is \code{NULL}, in which case \code{Time (s)} is printed if
\code{tUnit = 's'} and \code{Time (ms)} is printed if \code{tUnit = 'ms'}.}

\item{highlight}{Named list giving parameters for differential highlighting
of part of the plot based on the time domain. This list should contain
information about which parts of the plot to highlight, either done with the
\code{start} and \code{end} arguments which must be numbers or numeric vectors, or
using the \code{tier} and \code{label} arguments to highlight based on information in
a plotted TextGrid. Further contains the optional arguments \code{color}
(a string), \code{drawSize} and \code{speckleSize} (both numeric), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways. This argument is
used to highlight all plot components, use the \verb{*_highlight} arguments for
highlighting individuals plot components.}

\item{suppressAxes}{Should all axes and the box around plots be suppressed?
Default is \code{FALSE}. Generally not recommended.}

\item{draw_lines}{Use for drawing straight lines on plot components. Takes
an argument of type \code{list} which should contain a) a string giving the plot
component to draw straight lines on, and b) arguments to pass on to
\link[graphics:abline]{graphics::abline}. Should have a named argument \code{h} for horizontal lines,
or \code{v} for vertical lines, or \code{a},\code{b} for the intercept and slope of the
line otherwise. Alternatively a nested list can be passed if more (sets of)
lines should be drawn. If multiple audio
channels are plotted and lines should be added to one of these,
use the channel identifier instead of a string giving the frame to draw on.
The default value is \code{list('formant', h=seq(0,10000,by=1000), lty='dotted')}.
According to Praat defaults, this means that if formants are plotted in a
separate frame, horizontal dotted lines (\code{lty}) are shown at 1000 Hz
intervals. To override this behavior, simply pass \code{draw_lines=NULL}.}

\item{draw_rectangle}{Use for drawing rectangles on plot components. A
vector containing a) a string giving the plot component to draw a rectangle
on, and b) arguments to pass on to \link[graphics:rect]{graphics::rect}. Alternatively a list
of such vectors, if more rectangles should be drawn. If multiple audio
channels are plotted and a rectangle should be added to one of these,
use the channel identifier instead of a string giving the frame to draw on.}

\item{draw_arrow}{Use for drawing arrows on plot components. A vector
containing a) a string giving the plot component to draw an arrow on, and
b) arguments to pass on to \link[graphics:arrows]{graphics::arrows}. Alternatively a list of
such vectors, if more arrows should be drawn. If multiple audio
channels are plotted and an arrow should be added to one of these,
use the channel identifier instead of a string giving the frame to draw on.}

\item{annotate}{Use for annotating plot components. A vector containing
a) a string giving the plot component to annotate, and
b) arguments to pass on to \link[graphics:text]{graphics::text}. Alternatively a list of
such vectors, if more annotations should be made. If multiple audio
channels are plotted and annotations should be added to one of these,
use the channel identifier instead of a string giving the frame to draw on.}

\item{gender}{String indicating the gender of the speaker; default is
\code{u} for unknown, other legal values are \code{m} and \code{f}. Used to tweak pitch
and formant tracking parameters.}

\item{...}{Further global plotting arguments passed on to \code{par()}.}
}
\value{
No return value, produces a figure.
}
\description{
Generate plots of acoustic data aligned with transcriptions similar
to those made with Praat Picture. The default is to produce a plot with a
relatively small waveform, somewhat larger spectrogram, and the first tier
of a TextGrid.
}
\details{
When available, pitch, formant, and intensity tracks are loaded
from Praat files with the same base name as \code{sound}; i.e., if your sound
file is called \verb{1.wav} and there is a Praat file called \verb{1.Formant} in the
same directory, this file is used for plotting formants. Pitch files should
have either the \code{PitchTier} or \code{Pitch} extension, and intensity files should
have the \code{IntensityTier} extension.

If no such files are available, the signal processing tools in the \code{wrassp}
package are used; pitch is tracked with the function \link[wrassp:ksvF0]{wrassp::ksvF0},
formants are tracked with \link[wrassp:forest]{wrassp::forest}, and intensity is tracked with
\link[wrassp:rmsana]{wrassp::rmsana}. Parameters are set to mimic Praat as closely as possible,
e.g. using a Gaussian-like window shape \code{KAISER2_0}, but results will differ
from Praat simply because the tracking algorithms differ; as far as I know,
the Burg algorithm used by Praat for tracking formants isn't implemented in R,
nor is the autocorrelation method for tracking pitch.

Spectrograms are generated with the function \link[phonTools:spectrogram]{phonTools::spectrogram}. The
code portion that actually adds the spectrogram to a plot is based on
\link[phonTools:spectrogram]{phonTools::plot.spectrogram} but rewritten to use a bitmap raster for
rendering the image if the graphics device allows for it, which
significantly speeds up rendering the spectrogram.
}
\examples{
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/waveplot.R
\name{waveplot}
\alias{waveplot}
\title{Plot waveform}
\usage{
waveplot(
  sig,
  bit,
  t,
  start,
  tfrom0 = TRUE,
  nchan = 1,
  energyRange = NULL,
  color = "black",
  pitch_plotOnWave = FALSE,
  pt = NULL,
  pitch_plotType = "draw",
  pitch_scale = "hz",
  pitch_freqRange = NULL,
  pitch_axisLabel = NULL,
  pitch_color = "black",
  pitch_highlight = NULL,
  intensity_plotOnWave = FALSE,
  it = NULL,
  intensity_range = NULL,
  intensity_axisLabel = "Intensity (dB)",
  intensity_color = "black",
  intensity_highlight = NULL,
  tgbool = FALSE,
  lines = NULL,
  focusTierColor = "black",
  focusTierLineType = "dotted",
  ind = NULL,
  line_comp = NULL,
  rect_comp = NULL,
  arr_comp = NULL,
  annot_comp = NULL,
  draw_lines = NULL,
  draw_rectangle = NULL,
  draw_arrow = NULL,
  annotate = NULL,
  channelNames = FALSE,
  axisDigits = 3,
  lineWidth = 1,
  cn = NULL,
  min_max_only = TRUE,
  highlight = NULL,
  drawSize = 1,
  speckleSize = 1,
  suppressAxes = FALSE
)
}
\arguments{
\item{sig}{Numeric vector corresponding to a sound signal.}

\item{bit}{Numeric; will generally be grabbed from a loaded \code{WaveMC} object.}

\item{t}{Numeric vector giving times corresponding to the signal.}

\item{start}{Start time (in seconds) of desired plotted area.}

\item{tfrom0}{Logical; should time on the x-axis run from 0 or from the
original time? Default is \code{TRUE}.}

\item{nchan}{Numeric; how many channels will be plotted? Default is \code{1}.}

\item{energyRange}{Numeric vector giving the desired energy
range (y-axis range) of waveform(s). Default is \code{NULL}, in which case the
y-axis range is set to the lowest and highest value for each wave.
If a vector of two values is passed for a multi-wave plot, these are
recycled for each wave. Separate ranges can be set for each waveform by
passing a vector of a length corresponding to twice the number of waves
plotted.}

\item{color}{String giving the name of the color to be used for plotting
the waveform. Default is \code{'black'}. Alternatively, a vector of colors, if
different channels should be plotted with different colors.}

\item{pitch_plotOnWave}{Boolean; should pitch be plotted on top of
waveform? Default is \code{FALSE}.}

\item{pt}{Pitch object loaded using \link[rPraat:pt.read]{rPraat::pt.read} or similar object.}

\item{pitch_plotType}{String giving the type of pitch plot to produce; default
is \code{draw} (a line plot), the only other option is \code{speckle} (a point plot).
Alternatively a vector \code{c('draw','speckle')} can be passed, in which case
both are used.}

\item{pitch_scale}{String giving the frequency scale to use when producing
pitch plots. Default is \code{hz}; other options are \code{logarithmic} (also in Hz),
\code{semitones}, \code{erb}, and \code{mel}.}

\item{pitch_freqRange}{Vector of two integers giving the frequency range to be
used for producing pitch plots. Default is \code{NULL}, in which case the pitch
range is automatically reset to \code{c(-12,30)} for the \code{semitones} scale,
\code{c(0,10)} for the \code{erb} scale, and \code{c(50,500)} for the Hz-based scales,
following Praat defaults.}

\item{pitch_axisLabel}{String giving the name of the label to print along the
y-axis when printing a pitch track. Default is \code{NULL}, in which case the
axis label will depend on the scale.}

\item{pitch_color}{String or vector of strings giving the name of the color
to be used for plotting pitch. Default is \code{'black'}. If a vector of two
strings is passed, the second color will be used for background highlighting.}

\item{pitch_highlight}{Named list giving parameters for differential
highlighting of pitch based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{pitch_color}),
\code{drawSize} or \code{speckleSize} (both numeric).}

\item{intensity_plotOnWave}{Boolean; should intensity be plotted on top of
waveform? Default is \code{FALSE}.}

\item{it}{Intensity object loaded using \link[rPraat:it.read]{rPraat::it.read} or similar object.}

\item{intensity_range}{Vector of two integers giving the intensity range to be
used for producing intensity plots. Default is \code{NULL}, in which case the
range is simply the minimum and maximum levels in the curve.}

\item{intensity_axisLabel}{String giving the name of the label to print along
the y-axis when plotting intensity. Default is \code{Intensity (dB)}.}

\item{intensity_color}{String or vector of strings giving the name of the
color to be used for plotting intensity. Default is \code{'black'}. If a vector of
two strings is passed, the second color will be used for background
highlighting.}

\item{intensity_highlight}{Named list giving parameters for differential
highlighting of the intensity contour based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{intensity_color}) and
\code{drawSize} (integer).}

\item{tgbool}{Logical; should dotted lines be plotted corresponding to
locations in a TextGrid? Default is \code{FALSE}.}

\item{lines}{Numeric vector giving locations in seconds of locations from
a TextGrid to be plotted with dotted lines. Default is \code{NULL}.}

\item{focusTierColor}{String or vector of strings giving the color(s) to
use for plotting focus tier lines. If multiple tiers are focused, a vector
of the same length can be passed, and the nth tier will be plotted in the
nth color. Default is \code{'black'}.}

\item{focusTierLineType}{String or vector of strings giving the line
type(s) for plotting focus tier lines. If multiple tiers are focused, a
vector of the same length can be passed, and the nth tier will be plotted in
the nth line type. Default is \code{'dotted'}.}

\item{ind}{Integer indexing waveform relative to other plot components.
Default is \code{NULL}.}

\item{line_comp}{Vector of strings or numbers giving plot components to draw
straight lines on. Default is \code{NULL}.}

\item{rect_comp}{Vector of strings or numbers giving plot components to draw
rectangles on. Default is \code{NULL}.}

\item{arr_comp}{Vector of strings of numbers giving plot components to draw
arrows on. Default is \code{NULL}.}

\item{annot_comp}{Vector of strings of numbers giving plot components to
annotate. Default is \code{NULL}.}

\item{draw_lines}{List of arguments for drawing straight lines passed from
\code{praatpicture()}. Default is \code{NULL}.}

\item{draw_rectangle}{List of arguments for drawing rectangles passed from
\code{praatpicture()}. Default is \code{NULL}.}

\item{draw_arrow}{List of arguments for drawing arrows passed from
\code{praatpicture()}. Default is \code{NULL}.}

\item{annotate}{List of arguments for annotating passed from
\code{praatpicture()}. Default is \code{NULL}.}

\item{channelNames}{Logical; should names of audio channels be printed on
the y-axis? Default is \code{FALSE}.}

\item{axisDigits}{Numeric giving the number of digits to print for
values along the y-axis of the waveform. Default is \code{3}. If \code{0} is passed,
the y-axis is suppressed. Alternatively a vector of numbers, if different
numbers should be used for different channels. Note that this only
applies when \code{min_max_only = TRUE}, as otherwise the look of the y-axis is
determined entirely using \code{grDevices::axisTicks()}.}

\item{lineWidth}{Number giving the line width to use for plotting
the waveform. Default is \code{1}. Alternatively a vector of numbers, if
different line widths should be used for different channels.}

\item{cn}{Vector of strings with channel names to be printed on the y-axis
if \code{channelNames} is \code{TRUE}.}

\item{min_max_only}{Logical; should only minimum and maximum values be given
on the y-axis? Default is \code{TRUE}. Can also be a logical vector if some but
not all plot components should have minimum and maximum values on the y-axis.
Ignored for TextGrid component.}

\item{highlight}{Named list giving parameters for differential
highlighting of the waveform based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the argument
\code{color} (string, see \code{color}), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{drawSize}{Number indicating the line width of plot components where
the \verb{_plotType} is \code{'draw'} (i.e., pitch, formants, or intensity rendered as
line plots). Default is \code{1}. Controls the \code{lwd} argument of
\link[graphics:lines]{graphics::lines}.}

\item{speckleSize}{Number indicating the point size of plot components where
the \verb{_plotType} is \code{'speckle'} (i.e. pitch or formants rendered as point
plots). Default is \code{1}. Controls the \code{cex} arguments of \link[graphics:points]{graphics::points}.}

\item{suppressAxes}{Should all axes and the box around plots be suppressed?
Default is \code{FALSE}. Generally not recommended.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Function for plotting waveforms called by \link{praatpicture}. Instead of using
this function directly, just use
\code{praatpicture('my_sound_file', frames='sound')}.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, frames='sound')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_annot.R
\name{make_annot}
\alias{make_annot}
\title{Annotate praatpicture plot component}
\usage{
make_annot(plot_component, args)
}
\arguments{
\item{plot_component}{String giving the name of the plot component to
annotate.}

\item{args}{A list of vectors giving arguments used for annotating.
See \link{praatpicture} documentation.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Helper function for annotating plot components made with
praatpicture. Do not use directly, instead use \link{praatpicture} with the
\code{annotate} argument.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, annotate=c('spectrogram', 0.25, 1500,
'An annotation'))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tg_stylize.R
\name{tg_stylize}
\alias{tg_stylize}
\title{Convert Praat font styles to R font styles}
\usage{
tg_stylize(lab)
}
\arguments{
\item{lab}{A string or vector of strings with labels from a TextGrid.}
}
\value{
A list with elements of class \code{expression}.
}
\description{
Helper function for converting Praat font styles such as italics, bold,
and small caps into expressions that can be read by base R plots.
Instead of using
this function directly, just use
\code{praatpicture('my_sound_file', frames='TextGrid', tg_specialChar=TRUE)}.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')

# With stylized text
praatpicture(soundFile, frames='TextGrid')

# Without stylized text
praatpicture(soundFile, frames='TextGrid', tg_specialChar=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_TextGrid.R
\name{make_TextGrid}
\alias{make_TextGrid}
\title{Interactively create a TextGrid object}
\usage{
make_TextGrid(
  sound,
  tierNames,
  start = 0,
  end = 0,
  audioInViewer = TRUE,
  show = "wave",
  channel = 1,
  sampa2ipa = FALSE
)
}
\arguments{
\item{sound}{String giving the file name of a sound file with the .wav
extension.}

\item{tierNames}{String or vector of strings giving the name(s) of tiers in
the new TextGrid object.}

\item{start}{Start time (in seconds) of desired plotted area. Default is \code{0}.}

\item{end}{End time (in seconds) of desired plotted area. Default is \code{0}
(= the entire file).}

\item{audioInViewer}{Logical; should audio be playable from the Viewer pane
in RStudio?}

\item{show}{String giving the type of plot to show. Default is \code{wave},
another option is \code{spectrogram}. Note that spectrogram plotting is relatively
slow within this function.}

\item{channel}{Number indicating which audio channel to show. Default is \code{1}.}

\item{sampa2ipa}{Logical; should SAMPA transcriptions be converted to IPA?
Default is \code{FALSE}.}
}
\value{
A list object identical to those created by \code{\link[rPraat:tg.read]{rPraat::tg.read()}} when
loading TextGrid objects into R. This object can be passed to the \code{tg_obj}
argument when using \code{praatpicture}.
}
\description{
Annotate a sound file by interacting with waveform or spectrogram plots,
resulting in a TextGrid object which can be used for creating various
acoustic plots with time-aligned annotations with \code{\link[=praatpicture]{praatpicture()}}.
}
\details{
Running this function will show either a waveform or a spectrogram
in a separate \code{X11} graphics device window. Click on this figure in the
locations where you want to add boundaries to your TextGrid objects. This
should be done sequentially, starting with the first boundary along the
time axis and ending with the last. It does not matter where on the y-axis
you click.

Once you have indicated all the desired boundaries, you will be prompted
in the R console to say whether the tier is an interval tier or a point tier
by typing \code{y} (for interval tier) or \code{n} (for point tier). Subsequently
you will be prompted in the console to write labels corresponding to each
interval or point.

If you are creating a TextGrid with multiple tiers (i.e., if \code{tierNames}
is longer than 1), this process will be repeated for all tiers.
}
\examples{
\dontrun{
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/2.wav')
tg <- make_TextGrid(soundFile, tierNames=c('Mary', 'John', 'Bell'))
# Follow the steps shown in the console

praatpicture(soundFile, tg_obj=tg)
}
}
\seealso{
\code{make_TextGrid()} is largely a wrapper around the function
\code{\link[=tg_createTier]{tg_createTier()}} which does most of the work.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/emupicture.R
\name{emupicture}
\alias{emupicture}
\title{Make Praat Picture style plots in EMU}
\usage{
emupicture(
  db_handle,
  session = "0000",
  bundle,
  pitch_ssffExt = NULL,
  formant_ssffExt = NULL,
  intensity_ssffExt = NULL,
  talking = FALSE,
  ...
)
}
\arguments{
\item{db_handle}{The handle of an EMU database loaded into R.}

\item{session}{String giving the name of the session where the sound file
to plot is located.
Default is \code{0000}.}

\item{bundle}{String giving the name of the bundle with the sound file to plot.}

\item{pitch_ssffExt}{String giving the file extension for an SSFF track with
pitch data to plot. Default is \code{NULL}.}

\item{formant_ssffExt}{String giving the file extension for an SSFF track
with formant data to plot. Default is \code{NULL}.}

\item{intensity_ssffExt}{String giving the file extension for an SSFF track with intensity data to
plot. Default is \code{NULL}.}

\item{talking}{Boolean; should a video be created with embedded audio, as
when using \link{talking_praatpicture}? Default is \code{FALSE}.}

\item{...}{Further arguments passed to \link{praatpicture} (or
\link{talking_praatpicture} if \code{talking = TRUE}).}
}
\value{
No return value, produces a plot or a video.
}
\description{
Generate plots in the style of Praat Pictures from sound files and annotations
in an EMU database.
}
\examples{
\donttest{
# Create demo data and load demo database
emuR::create_emuRdemoData(tempdir())
db_path <- paste0(tempdir(), '/emuR_demoData/ae_emuDB')
db <- emuR::load_emuDB(db_path)

emuR::list_bundles(db)
emupicture(db, bundle='msajc003', tg_tiers=c('Text', 'Tone'))

# Plot SSFF track data

emuR::list_ssffTrackDefinitions(db)
emupicture(db, bundle='msajc003', frames=c('sound', 'formant'),
proportion=c(30,70), formant_ssffExt='fms', formant_number=4)
}
}
\seealso{
See \link{praatpicture} for more details on how to customize plots and
\link{talking_praatpicture} for more details on how to customize videos.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pitchplot.R
\name{pitchplot}
\alias{pitchplot}
\title{Plot pitch object}
\usage{
pitchplot(
  pt,
  start,
  end,
  tfrom0 = TRUE,
  tgbool = FALSE,
  lines = NULL,
  focusTierColor = "black",
  focusTierLineType = "dotted",
  plotType = "draw",
  scale = "hz",
  freqRange = NULL,
  semitonesRe = 100,
  color = "black",
  ind = NULL,
  min_max_only = TRUE,
  highlight = NULL,
  axisLabel = NULL,
  drawSize = 1,
  speckleSize = 1,
  suppressAxes = FALSE
)
}
\arguments{
\item{pt}{PitchTier object loaded using \link[rPraat:pt.read]{rPraat::pt.read} or other object
formatted in a similar way, i.e. a \code{list} object containing the elements
\code{t} (a vector of time values) and \code{f} (a vector of frequency values) of
identical length.}

\item{start}{Start time (in seconds) of desired plotted area.}

\item{end}{End time (in seconds) of desired plotted area.}

\item{tfrom0}{Logical; should time on the x-axis run from 0 or from the
original time? Default is \code{TRUE}.}

\item{tgbool}{Logical; should dotted lines be plotted corresponding to
locations in a TextGrid? Default is \code{FALSE}.}

\item{lines}{Numeric vector giving locations in seconds of locations from
a TextGrid to be plotted with dotted lines. Default is \code{NULL}.}

\item{focusTierColor}{String or vector of strings giving the color(s) to
use for plotting focus tier lines. If multiple tiers are focused, a vector
of the same length can be passed, and the nth tier will be plotted in the
nth color. Default is \code{'black'}.}

\item{focusTierLineType}{String or vector of strings giving the line
type(s) for plotting focus tier lines. If multiple tiers are focused, a
vector of the same length can be passed, and the nth tier will be plotted in
the nth line type. Default is \code{'dotted'}.}

\item{plotType}{String giving the type of pitch plot to produce; default
is \code{draw} (a line plot), the only other option is \code{speckle} (a point plot).
Alternatively a vector \code{c('draw','speckle')} can be passed, in which case
both are used.}

\item{scale}{String giving the frequency scale to use when producing
pitch plots. Default is \code{hz}; other options are \code{logarithmic} (also in Hz),
\code{semitones}, \code{erb}, and \code{mel}.}

\item{freqRange}{Vector of two integers giving the frequency range to be
used for producing pitch plots. Default is \code{NULL}, in which case the pitch
range is automatically reset to \code{c(-12,30)} for the \code{semitones} scale,
\code{c(0,10)} for the \code{erb} scale, and \code{c(50,500)} for the Hz-based scales,
following Praat defaults.}

\item{semitonesRe}{Frequency in Hz giving the reference level for converting
pitch frequency to semitones. Default is \code{100}.}

\item{color}{String giving the name of the color to be used for
plotting pitch. Default is \code{'black'}.}

\item{ind}{Integer indexing pitch relative to other plot components.
Default is \code{NULL}.}

\item{min_max_only}{Logical; should only minimum and maximum values be given
on the y-axis? Default is \code{TRUE}. Can also be a logical vector if some but
not all plot components should have minimum and maximum values on the y-axis.
Ignored for TextGrid component.}

\item{highlight}{Named list giving parameters for differential
highlighting of pitch based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{color}),
\code{drawSize} or \code{speckleSize} (both numeric), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{axisLabel}{String giving the name of the label to print along the
y-axis when printing a pitch track. Default is \code{NULL}, in which case the
axis label will depend on the scale.}

\item{drawSize}{Number indicating the line width if
\code{plotType} is \code{'draw'}. Default is \code{1}. Controls the \code{lwd} argument of
\link[graphics:lines]{graphics::lines}.}

\item{speckleSize}{Number indicating the point size of if \verb{_plotType} is
\code{'speckle'}. Default is \code{1}. Controls the \code{cex} arguments of
\link[graphics:points]{graphics::points}.}

\item{suppressAxes}{Should all axes and the box around plots be suppressed?
Default is \code{FALSE}. Generally not recommended.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Function for plotting pitch objects called by \link{praatpicture}. Instead of
using this function directly, just use
\code{praatpicture('my_sound_file', frames='pitch')}.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, frames='pitch')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intensityplot.R
\name{intensityplot}
\alias{intensityplot}
\title{Plot intensity object}
\usage{
intensityplot(
  it,
  start,
  end,
  tfrom0 = TRUE,
  tgbool = FALSE,
  lines = NULL,
  focusTierColor = "black",
  focusTierLineType = "dotted",
  range = NULL,
  color = "black",
  ind = NULL,
  min_max_only = TRUE,
  highlight = NULL,
  axisLabel = "Intensity (dB)",
  drawSize = 1,
  suppressAxes = FALSE
)
}
\arguments{
\item{it}{IntensityTier object loaded using \link[rPraat:it.read]{rPraat::it.read} or other object
formatted in a similar way, i.e. a \code{list} object containing the elements
\code{t} (a vector of time values) and \code{i} (a vector of intensity values) of
identical length.}

\item{start}{Start time (in seconds) of desired plotted area.}

\item{end}{End time (in seconds) of desired plotted area.}

\item{tfrom0}{Logical; should time on the x-axis run from 0 or from the
original time? Default is \code{TRUE}.}

\item{tgbool}{Logical; should dotted lines be plotted corresponding to
locations in a TextGrid? Default is \code{FALSE}.}

\item{lines}{Numeric vector giving locations in seconds of locations from
a TextGrid to be plotted with dotted lines. Default is \code{NULL}.}

\item{focusTierColor}{String or vector of strings giving the color(s) to
use for plotting focus tier lines. If multiple tiers are focused, a vector
of the same length can be passed, and the nth tier will be plotted in the
nth color. Default is \code{'black'}.}

\item{focusTierLineType}{String or vector of strings giving the line
type(s) for plotting focus tier lines. If multiple tiers are focused, a
vector of the same length can be passed, and the nth tier will be plotted in
the nth line type. Default is \code{'dotted'}.}

\item{range}{Vector of two integers giving the intensity range to be
used for producing intensity plots. Default is \code{NULL}, in which case the
range is simply the minimum and maximum levels in the curve.}

\item{color}{String giving the name of the color to be used for
plotting intensity. Default is \code{'black'}.}

\item{ind}{Integer indexing current plot frame relative to other plot
components. Default is \code{NULL}.}

\item{min_max_only}{Logical; should only minimum and maximum values be given
on the y-axis? Default is \code{TRUE}. Can also be a logical vector if some but
not all plot components should have minimum and maximum values on the y-axis.
Ignored for TextGrid component.}

\item{highlight}{Named list giving parameters for differential
highlighting of the intensity contour based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{color}) and
\code{drawSize} (integer), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{axisLabel}{String giving the name of the label to print along
the y-axis when plotting intensity. Default is \code{Intensity (dB)}.}

\item{drawSize}{Number indicating the line width of the intensity contour.
Default is \code{1}. Controls the \code{lwd} argument of \link[graphics:lines]{graphics::lines}.}

\item{suppressAxes}{Should all axes and the box around plots be suppressed?
Default is \code{FALSE}. Generally not recommended.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Function for plotting intensity objects called by \link{praatpicture}. Instead of
using this function directly, just use
\code{praatpicture('my_sound_file', frames='intensity')}.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, frames='intensity')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tg_createTier.R
\name{tg_createTier}
\alias{tg_createTier}
\title{Interactively create a TextGrid tier}
\usage{
tg_createTier(
  sound,
  tierName,
  start = 0,
  end = 0,
  show = "wave",
  channel = 1,
  sampa2ipa = FALSE
)
}
\arguments{
\item{sound}{String giving the file name of a sound file with the .wav
extension.}

\item{tierName}{String giving the name of the tier.}

\item{start}{Start time (in seconds) of desired plotted area. Default is \code{0}.}

\item{end}{End time (in seconds) of desired plotted area. Default is \code{0}
(= the entire file).}

\item{show}{String giving the type of plot to show. Default is \code{wave},
another option is \code{spectrogram}. Note that spectrogram plotting is relatively
slow within this function.}

\item{channel}{Number indicating which audio channel to show. Default is \code{1}.}

\item{sampa2ipa}{Logical; should SAMPA transcriptions be converted to IPA?
Default is \code{FALSE}.}
}
\value{
A list object identical to a single tier created by
\code{\link[rPraat:tg.read]{rPraat::tg.read()}} when
loading TextGrid objects into R.
}
\description{
Function for creating TextGrid tiers called by \link{make_TextGrid}. Instead of
using this function directly, use \link{make_TextGrid}.
}
\examples{
\dontrun{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/2.wav')
tg <- make_TextGrid(soundFile, tierNames='Mary')
# Follow the steps shown in the console

praatpicture(soundFile, tg_obj=tg)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/specplot.R
\name{specplot}
\alias{specplot}
\title{Plot spectrogram}
\usage{
specplot(
  sig,
  sr,
  t,
  start,
  end,
  tfrom0 = TRUE,
  scale = "hz",
  freqRange = NULL,
  windowLength = 0.005,
  dynamicRange = 60,
  timeStep = 1000,
  windowShape = "Gaussian",
  colors = c("white", "black"),
  pitch_plotOnSpec = FALSE,
  pt = NULL,
  pitch_plotType = "draw",
  pitch_scale = "hz",
  pitch_freqRange = NULL,
  pitch_axisLabel = NULL,
  pitch_color = "black",
  pitch_highlight = NULL,
  formant_plotOnSpec = FALSE,
  fm = NULL,
  formant_plotType = "speckle",
  formant_dynamicRange = 30,
  formant_color = "black",
  formant_highlight = NULL,
  intensity_plotOnSpec = FALSE,
  it = NULL,
  intensity_range = NULL,
  intensity_axisLabel = "Intensity (dB)",
  intensity_color = "black",
  intensity_highlight = NULL,
  tgbool = FALSE,
  lines = NULL,
  focusTierColor = "black",
  focusTierLineType = "dotted",
  ind = NULL,
  min_max_only = TRUE,
  highlight = NULL,
  axisLabel = NULL,
  drawSize = 1,
  speckleSize = 1,
  suppressAxes = FALSE
)
}
\arguments{
\item{sig}{Numeric vector corresponding to a sound signal.}

\item{sr}{Integer giving the sampling rate of the signal.}

\item{t}{Numeric vector giving times corresponding to the signal.}

\item{start}{Start time (in seconds) of desired plotted area.}

\item{end}{End time (in seconds) of desired plotted area.}

\item{tfrom0}{Logical; should time on the x-axis run from 0 or from the
original time? Default is \code{TRUE}.}

\item{scale}{String giving the frequency scale to use for plotting
spectrograms. Default is \code{hz}. Alternatives are \code{erb} (for the equivalent
rectangular bandwidth scale), \code{mel} (for the Mel scale, using 1000 Hz
as the corner frequency, following Fant 1968), or \code{logarithmic} (for
log Hz).}

\item{freqRange}{Vector of two integers giving the frequency range to be
used for plotting spectrograms. Default is \code{NULL}, in which case suitable
ranges are chosen based on the frequency scale. For \code{hz}, the default is
\code{c(0,5000)}; for \code{erb}, the default is \code{c(0.8,42)}; for \code{mel}, the default is
\code{c(30,4400)}. The latter two correspond roughly to the human auditory
frequency response.}

\item{windowLength}{Window length in seconds for generating spectrograms.
Default is \code{0.005}.}

\item{dynamicRange}{Dynamic range in dB for generating spectrograms. The
maximum intensity minus \code{dynamicRange} will all be printed in white. Default
is \code{50}.}

\item{timeStep}{How many time steps should be calculated for spectrograms?
Default is \code{1000}. Note that this takes a while to plot, so for fiddling with
plotting parameters it is a good idea to choose a smaller value.}

\item{windowShape}{String giving the name of the window shape to be applied
to the signal when generating spectrograms. Default is \code{Gaussian}; other
options are \code{square}, \code{Hamming}, \code{Bartlett}, \code{Hanning}, or \code{Blackman}.
Note that the Gaussian window function provided by the \code{phonTools} package
and used in \code{praatpicture()} does not have the same properties as the
Gaussian window function used for spectral estimation in Praat; plotting
a simple sine wave with high dynamic range will produce sidelobes in
\code{praatpicture()} but not in Praat. It's recommended to use Blackman windows
instead if you have this problem.}

\item{colors}{Vector of strings giving the names of colors to be used
for plotting the spectrogram; default is \code{c('white', 'black')}. The first
value is used for plotting the lowest visible amplitude, and the last for
plotting the highest visible amplitude. Vectors with more than two color
names can be used for plotting values in between in different colors.}

\item{pitch_plotOnSpec}{Boolean; should pitch be plotted on top of
spectrogram? Default is \code{FALSE}.}

\item{pt}{Pitch object loaded using \link[rPraat:pt.read]{rPraat::pt.read} or similar object.}

\item{pitch_plotType}{String giving the type of pitch plot to produce; default
is \code{draw} (a line plot), the only other option is \code{speckle} (a point plot).
Alternatively a vector \code{c('draw','speckle')} can be passed, in which case
both are used.}

\item{pitch_scale}{String giving the frequency scale to use when producing
pitch plots. Default is \code{hz}; other options are \code{logarithmic} (also in Hz),
\code{semitones}, \code{erb}, and \code{mel}.}

\item{pitch_freqRange}{Vector of two integers giving the frequency range to be
used for producing pitch plots. Default is \code{NULL}, in which case the pitch
range is automatically reset to \code{c(-12,30)} for the \code{semitones} scale,
\code{c(0,10)} for the \code{erb} scale, and \code{c(50,500)} for the Hz-based scales,
following Praat defaults.}

\item{pitch_axisLabel}{String giving the name of the label to print along the
y-axis when printing a pitch track. Default is \code{NULL}, in which case the
axis label will depend on the scale.}

\item{pitch_color}{String or vector of strings giving the name of the color
to be used for plotting pitch. Default is \code{'black'}. If a vector of two
strings is passed, the second color will be used for background highlighting.}

\item{pitch_highlight}{Named list giving parameters for differential
highlighting of pitch based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{pitch_color}),
\code{drawSize} or \code{speckleSize} (both numeric).}

\item{formant_plotOnSpec}{Boolean; should formants be plotted on top of
spectrogram? Default is \code{FALSE}.}

\item{fm}{Formant object loaded using \link[rPraat:formant.read]{rPraat::formant.read} or similar
object.}

\item{formant_plotType}{String giving the type of formant plot to produce;
default is \code{speckle} (a point plot), the only other option is \code{draw} (a line
plot). Alternatively a vector \code{c('draw','speckle')} can be passed, in which
case both are used.}

\item{formant_dynamicRange}{Dynamic range in dB for producing formant plots.
When a formant plot of \code{formant_plotType='speckle'} is drawn, no formants are
shown in frames with intensity level \code{formant_dynamicRange} below the maximum
intensity. Default is \code{30}. If set to \code{0}, all formants are shown.}

\item{formant_color}{String or vector of strings giving the name(s) of
colors to be used for plotting formants. If one color is provided, all
formants will be plotted in this color. If multiple colors are provided,
different formants will be shown in different colors. Default is \code{'black'}.
If the length of this vector twice the number of formants plotted, the first
half of strings will be used for the formants' primary colors and the second
half will be used for background highlighting. If the length of this vector
is one more than the number of formants plotted, the last string will
be used for background highlighting.}

\item{formant_highlight}{Named list giving parameters for differential
highlighting of formants based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{formant_color}),
\code{drawSize} or \code{speckleSize} (both numeric).}

\item{intensity_plotOnSpec}{Boolean; should intensity be plotted on top of
spectrogram? Default is \code{FALSE}.}

\item{it}{Intensity object loaded using \link[rPraat:it.read]{rPraat::it.read} or similar object.}

\item{intensity_range}{Vector of two integers giving the intensity range to be
used for producing intensity plots. Default is \code{NULL}, in which case the
range is simply the minimum and maximum levels in the curve.}

\item{intensity_axisLabel}{String giving the name of the label to print along
the y-axis when plotting intensity. Default is \code{Intensity (dB)}.}

\item{intensity_color}{String or vector of strings giving the name of the
color to be used for plotting intensity. Default is \code{'black'}. If a vector of
two strings is passed, the second color will be used for background
highlighting.}

\item{intensity_highlight}{Named list giving parameters for differential
highlighting of the intensity contour based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{intensity_color}) and
\code{drawSize} (integer).}

\item{tgbool}{Logical; should dotted lines be plotted corresponding to
locations in a TextGrid? Default is \code{FALSE}.}

\item{lines}{Numeric vector giving locations in seconds of locations from
a TextGrid to be plotted with dotted lines. Default is \code{NULL}.}

\item{focusTierColor}{String or vector of strings giving the color(s) to
use for plotting focus tier lines. If multiple tiers are focused, a vector
of the same length can be passed, and the nth tier will be plotted in the
nth color. Default is \code{'black'}.}

\item{focusTierLineType}{String or vector of strings giving the line
type(s) for plotting focus tier lines. If multiple tiers are focused, a
vector of the same length can be passed, and the nth tier will be plotted in
the nth line type. Default is \code{'dotted'}.}

\item{ind}{Integer indexing waveform relative to other plot components.
Default is \code{NULL}.}

\item{min_max_only}{Logical; should only minimum and maximum values be given
on the y-axis? Default is \code{TRUE}. Can also be a logical vector if some but
not all plot components should have minimum and maximum values on the y-axis.
Ignored for TextGrid component.}

\item{highlight}{Named list giving parameters for differential
highlighting of the spectrogram based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the argument
\code{colors} (vector of strings, see \code{colors}). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{axisLabel}{String giving the name of the label to print along the
y-axis when plotting a spectrogram. Default is \code{NULL}, in which case the
axis label will depend on the scale.}

\item{drawSize}{Number indicating the line width of plot components where
the \verb{_plotType} is \code{'draw'} (i.e., pitch, formants, or intensity rendered as
line plots). Default is \code{1}. Controls the \code{lwd} argument of
\link[graphics:lines]{graphics::lines}.}

\item{speckleSize}{Number indicating the point size of plot components where
the \verb{_plotType} is \code{'speckle'} (i.e. pitch or formants rendered as point
plots). Default is \code{1}. Controls the \code{cex} arguments of \link[graphics:points]{graphics::points}.}

\item{suppressAxes}{Should all axes and the box around plots be suppressed?
Default is \code{FALSE}. Generally not recommended.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Function for plotting spectrograms called by \link{praatpicture}. Instead of using
this function directly, just use
\code{praatpicture('my_sound_file', frames='spectrogram')}.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, frames='spectrogram')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conv2sc.R
\name{conv2sc}
\alias{conv2sc}
\title{Convert capital letters to Unicode small caps}
\usage{
conv2sc(x)
}
\arguments{
\item{x}{A string where all capital letters should be converted to small
caps.}
}
\value{
A string where all capital letters have been converted to small caps.
}
\description{
Helper function to convert capital letters into Unicode small caps.
May not work for all font families. Note that there's no Unicode small cap
'X', so 'X' will just be converted to 'x'.
}
\examples{
my_string <- 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
conv2sc(my_string)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tgplot.R
\name{tgplot}
\alias{tgplot}
\title{Plot TextGrid}
\usage{
tgplot(
  tg,
  t,
  sr,
  start,
  end,
  tiers = 1,
  tfrom0 = TRUE,
  tierNames = TRUE,
  alignment = "central",
  edgeLabels = "keep",
  specialChar = FALSE,
  color = "black",
  highlight = NULL
)
}
\arguments{
\item{tg}{TextGrid object loaded using \link[rPraat:tg.read]{rPraat::tg.read}}

\item{t}{Numeric vector giving times corresponding to the signal.}

\item{sr}{Integer giving the sampling rate of the signal.}

\item{start}{Start time (in seconds) of desired plotted area.}

\item{end}{End time (in seconds) of desired plotted area.}

\item{tiers}{Vector of number or strings giving either numeric identifiers
of TextGrid tiers to plot or the names of TextGrid tiers to plot. Default is
\code{1}, which plots just the first tier.}

\item{tfrom0}{Logical; should time on the x-axis run from 0 or from the
original time? Default is \code{TRUE}.}

\item{tierNames}{Logical; should TextGrid tier names be printed along the
y-axis? Default is \code{TRUE}.}

\item{alignment}{String giving the desired alignment of text in the
TextGrids. Default is \code{central}; other options are \code{left} and \code{right}.
Alternatively, a vector of strings if different alignments are needed.}

\item{edgeLabels}{String specifying how to handle TextGrid labels in
interval tiers that fall partially before \code{start} or partially after \code{end}.
Default is \code{'keep'}, where labels are kept at the center of the interval.
Other options are \code{'center'}, where labels are recentered to the visible
part of the interval, or \code{'discard'}, where such labels are ignored.}

\item{specialChar}{Logical; should Praat typesetting for special font types
such as italic, bold, and small caps be converted into corresponding
R-readable special font types. Default is \code{FALSE}, since special characters
can create unfortunate text alignment artefacts.
See https://www.fon.hum.uva.nl/praat/manual/Text_styles.html.}

\item{color}{String or vector of strings giving the name of the color(s)
to be used for the text in TextGrids. Default is \code{'black'}. If a vector is
provided, different colors are used for different tiers.}

\item{highlight}{Named list giving parameters for differential
highlighting of TextGrid intervals. This list
should contain information about which intervals to highlight, using the
\code{tier} and \code{label}. Further contains the argument
\code{color}, and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Function for plotting TextGrids called by \link{praatpicture}. Instead of using
this function directly, just use
\code{praatpicture('my_sound_file', frames='TextGrid')}.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, frames='TextGrid')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/draw_spectralslice.R
\name{draw_spectralslice}
\alias{draw_spectralslice}
\title{Draw spectral slice}
\usage{
draw_spectralslice(
  sound,
  time,
  channel = 1,
  freqRange = NULL,
  energyRange = 60,
  scale = "hz",
  method = "fft",
  multitaper_args = NULL,
  windowLength = 0.005,
  windowShape = "Gaussian",
  color = "black",
  lineWidth = 1,
  freq_axisLabel = NULL,
  energy_axisLabel = "Sound pressure level (dB/Hz)",
  mainTitle = "",
  mainTitleAlignment = 0,
  highlight = NULL,
  draw_lines = NULL,
  draw_rectangle = NULL,
  draw_arrow = NULL,
  annotate = NULL,
  ...
)
}
\arguments{
\item{sound}{String giving the file name of a sound file with the .wav
extension.}

\item{time}{Time (in seconds) specifying the center of the window from which
to estimate spectrum.}

\item{channel}{Numeric giving the channel that should be used to
generate the spectrogram. Default is \code{1}.}

\item{freqRange}{Vector of two numbers giving the frequency range to be
used for plotting spectrograms. Default is \code{NULL}, in which case the whole
spectrum is plotted using the Nyquist frequency for the upper limit.}

\item{energyRange}{Either numeric or vector of two numbers used to specify
the y-axis range in units of dB/Hz. If a single number is passed, this is
interpreted as the desired dynamic range of the y-axis. If a vector of
two numbers is passed, these are used to exactly delimit the y-axis.
Default is \code{60}, i.e. a dynamic range of 60 dB/Hz relative to the global
maximum.}

\item{scale}{String giving the frequency scale to use. Default is \code{hz}.
Alternatives are \code{erb} (for the equivalent
rectangular bandwidth scale), \code{mel} (for the Mel scale, using 1000 Hz
as the corner frequency, following Fant 1968), or \code{logarithmic} (for
log Hz).}

\item{method}{String specifying the spectral estimation. Default is \code{fft}
(for the fast discrete Fourier transform). The only other option is
\code{multitaper}, in which case \link[multitaper:spec.mtm]{multitaper::spec.mtm} is used to generate the
spectrum.}

\item{multitaper_args}{Optional named list of arguments passed on to
\link[multitaper:spec.mtm]{multitaper::spec.mtm} if \code{method='multitaper'}.}

\item{windowLength}{Window length in seconds for generating spectra.
Default is \code{0.005}.}

\item{windowShape}{String giving the name of the window shape to be
applied to the signal when generating spectrograms. Default is \code{Gaussian};
other options are \code{square}, \code{Hamming}, \code{Bartlett}, \code{Hanning}, \code{Blackman}, or
\code{Kaiser}.Note that the Gaussian window function provided by the \code{phonTools}
package and used for this function does not have the same properties as the
Gaussian window function used for spectral estimation in Praat.}

\item{color}{String giving the name of the color to be used for plotting
the spectrum. Default is \code{'black'}.}

\item{lineWidth}{Number giving the line width to use for plotting
the spectrum. Default is \code{1}.}

\item{freq_axisLabel}{String giving the name of the label to print along the
y-axis when plotting a spectrogram. Default is \code{NULL}, in which case the
axis label will depend on the scale.}

\item{energy_axisLabel}{String giving the name of the label to print along
the y-axis. Default is \verb{Sound pressure level (dB/Hz)}.}

\item{mainTitle}{String giving a title to print at the top of the plot.
The default is an empty string, i.e. no title.}

\item{mainTitleAlignment}{Number indicating the vertical alignment of the
plot title, where \code{0} (default) indicates left-alignment, \code{1} indicates
right-alignment, \code{0.5} indicates central alignment, etc, following the
conventions of the \code{adj} argument of \link[graphics:mtext]{graphics::mtext}.}

\item{highlight}{Named list giving parameters for differential highlighting
of part of the plot based on the frequency domain. This list should contain
information about which parts of the plot to highlight, done with the
\code{start} and \code{end} arguments which must be numbers or numeric vectors.
Further contains the optional arguments \code{color}
(a string), \code{drawSize} (numeric), and \code{background}
(a string specifying a background color).}

\item{draw_lines}{Use for drawing straight lines. Takes
an argument of type \code{list} which should contain arguments to pass on to
\link[graphics:abline]{graphics::abline}. Should have a named argument \code{h} for horizontal lines,
or \code{v} for vertical lines, or \code{a},\code{b} for the intercept and slope of the
line otherwise. Alternatively a nested list can be passed if more (sets of)
lines should be drawn. Default is \code{NULL}.}

\item{draw_rectangle}{Use for drawing rectangles. Should be a named
list containing arguments to pass on to \link[graphics:rect]{graphics::rect}. Can also be
multiple nested lists, if more rectangles should be drawn.}

\item{draw_arrow}{Use for drawing arrows on plot components. Should be a
named list containing arguments to pass on to \link[graphics:arrows]{graphics::arrows}.
Can also be multiple nested lists, if more rectangles should be drawn.}

\item{annotate}{Use for annotating plot components. Should be a
named list containing arguments to pass on to\link[graphics:text]{graphics::text}.
Can also be multiple nested lists, if more annotations should be added.}

\item{...}{Further global plotting arguments passed on to \code{par()}.}
}
\value{
No return value, produces a figure.
}
\description{
Generate and plot spectral slice from window around a specified time point
of a sound file.
}
\examples{
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
draw_spectralslice(soundFile, time = 0.75)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pitch_overlay.R
\name{pitch_overlay}
\alias{pitch_overlay}
\title{Overlay pitch on plot frame}
\usage{
pitch_overlay(
  pt,
  bottomRange,
  topRange,
  start,
  org_start = 0,
  tfrom0 = TRUE,
  freqRange = NULL,
  plotType = "draw",
  scale = "hz",
  color = "black",
  ind = NULL,
  drawSize = 1,
  speckleSize = 1,
  axisLabel = NULL,
  min_max_only = TRUE,
  highlight = NULL,
  intensity_overlay = FALSE,
  suppressAxes = FALSE
)
}
\arguments{
\item{pt}{PitchTier object loaded using \link[rPraat:pt.read]{rPraat::pt.read} or other object
formatted in a similar way, i.e. a \code{list} object containing the elements
\code{t} (a vector of time values) and \code{f} (a vector of frequency values) of
identical length.}

\item{bottomRange}{Bottom y-axis range of the plot frame that pitch
is plotted on.}

\item{topRange}{Top y-axis range of the plot frame that pitch is plotted on.}

\item{start}{Start time (in seconds) of desired plotted area.}

\item{org_start}{Start time (in seconds) of desired plotted area in the
original sound file.}

\item{tfrom0}{Logical; should time on the x-axis run from 0 or from the
original time? Default is \code{TRUE}.}

\item{freqRange}{Vector of two integers giving the frequency range to be
used for producing pitch plots. Default is \code{NULL}, in which case the pitch
range is automatically reset to \code{c(-12,30)} for the \code{semitones} scale,
\code{c(0,10)} for the \code{erb} scale, and \code{c(50,500)} for the Hz-based scales,
following Praat defaults.}

\item{plotType}{String giving the type of pitch plot to produce; default
is \code{draw} (a line plot), the only other option is \code{speckle} (a point plot).
Alternatively a vector \code{c('draw','speckle')} can be passed, in which case
both are used.}

\item{scale}{String giving the frequency scale to use when producing
pitch plots. Default is \code{hz}; other options are \code{logarithmic} (also in Hz),
\code{semitones}, \code{erb}, and \code{mel}.}

\item{color}{String giving the name of the color to be used for
plotting pitch. Default is \code{'black'}.}

\item{ind}{Integer indexing current plot frame relative to other plot
components. Default is \code{NULL}.}

\item{drawSize}{Number indicating the line width if
\code{plotType} is \code{'draw'}. Default is \code{1}. Controls the \code{lwd} argument of
\link[graphics:lines]{graphics::lines}.}

\item{speckleSize}{Number indicating the point size of if \verb{_plotType} is
\code{'speckle'}. Default is \code{1}. Controls the \code{cex} arguments of
\link[graphics:points]{graphics::points}.}

\item{axisLabel}{String giving the name of the label to print along the
y-axis when printing a pitch track. Default is \code{NULL}, in which case the
axis label will depend on the scale.}

\item{min_max_only}{Logical; should only minimum and maximum values be given
on the y-axis? Default is \code{TRUE}. Can also be a logical vector if some but
not all plot components should have minimum and maximum values on the y-axis.
Ignored for TextGrid component.}

\item{highlight}{Named list giving parameters for differential
highlighting of pitch based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{color}),
\code{drawSize} or \code{speckleSize} (both numeric), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{intensity_overlay}{Logical; is intensity also overlaid on the same
plot frame? Default is \code{FALSE}.}

\item{suppressAxes}{Should all axes and the box around plots be suppressed?
Default is \code{FALSE}. Generally not recommended.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Function for overlaying pitch contour on another plot frame, viz. the
waveform or spectrogram. Instead of using this function directly, just use
\code{praatpicture('my_sound_file')} with \code{pitch_plotOnSpec} or \code{pitch_plotOnWave}
set to \code{TRUE}.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, frames = 'spectrogram', pitch_plotOnSpec = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/draw_arrow.R
\name{draw_arrow}
\alias{draw_arrow}
\title{Draw arrow on praatpicture plot component}
\usage{
draw_arrow(plot_component, args)
}
\arguments{
\item{plot_component}{String giving the name of the plot component to
draw on.}

\item{args}{A list of vectors giving arguments used for drawing
arrows. See \link{praatpicture} documentation.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Helper function for drawing arrows on plot components made with
praatpicture. Do not use directly, instead use \link{praatpicture} with the
\code{draw_arrow} argument.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, draw_arrow=c('spectrogram', 0.1, 500, 0.4, 2000))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/draw_lines.R
\name{draw_lines}
\alias{draw_lines}
\title{Draw straight lines on plot component}
\usage{
draw_lines(plot_component, args)
}
\arguments{
\item{plot_component}{String giving the name of the plot component to
draw on.}

\item{args}{A list of vectors giving arguments used for drawing
straight lines. See \link{praatpicture} documentation.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Helper function for adding straight lines to plot components made with
praatpicture. Do not use directly, instead use \link{praatpicture} with the
\code{draw_lines} argument.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, draw_lines=c('spectrogram',
h=seq(0,5000,by=1000), lty='dashed'))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/praatanimation.R
\name{praatanimation}
\alias{praatanimation}
\title{Make animations from Praat Picture-style plots of acoustic data}
\usage{
praatanimation(
  sound,
  width = 1080,
  height = 720,
  frameRate = 24,
  n_frames = 50,
  loop = TRUE,
  outputFile = NULL,
  outputFormat = "gif",
  useViewer = TRUE,
  verbose = TRUE,
  pointsize = 25,
  start = 0,
  end = 0,
  spec_freqRange = c(0, 5000),
  spec_windowLength = 0.005,
  spec_dynamicRange = 50,
  spec_timeStep = 1000,
  pitch_timeStep = NULL,
  pitch_floor = 50,
  pitch_ceiling = 600,
  pitch_freqRange = c(50, 500),
  pitch_semitonesRe = 100,
  formant_timeStep = NULL,
  formant_windowLength = 0.025,
  formant_dynamicRange = 30,
  formant_freqRange = c(50, 5500),
  intensity_timeStep = NULL,
  intensity_minPitch = 100,
  intensity_range = NULL,
  ...
)
}
\arguments{
\item{sound}{String giving the file name of a sound file with the .wav
extension.}

\item{width}{Number giving the desired width of the resulting animation in
pixels; default is \code{1080}.}

\item{height}{Number giving the desired height of the resulting animation in
pixels; default is \code{720}.}

\item{frameRate}{Number giving the desired frame rate of the resulting
animation in Hz; default is \code{24}, i.e. 24 frames per second.}

\item{n_frames}{Number giving the desired number of frames of the resulting
animation; default is \code{50}.}

\item{loop}{Logical; should the animation be looped? Default is \code{TRUE}.
Ignored when \code{outputType} is \code{mp4}.}

\item{outputFile}{String giving the desired file name of the animation.
Default is \code{NULL}, in which case GIF files are named \code{praatgif.gif} and
MP4 files are named \code{praatvid.mp4}. If you choose a different name, make
sure that the file extension matches the selected \code{outputType}.}

\item{outputFormat}{String giving the desired file type; default is \code{gif},
the only other option is \code{mp4}.S}

\item{useViewer}{Logical; should the animation be shown in the Viewer pane in
RStudio? Default is \code{TRUE}; if true, the animation is only saved in a
temporary directory, but can be downloaded from a browser.}

\item{verbose}{Logical; should status messages be printed in the console as
figures are being generated? Default is \code{TRUE}.}

\item{pointsize}{Number; which point size should be used for text in the
animation? Default is \code{25}. See \code{\link[grDevices:png]{grDevices::png()}} for more details.}

\item{start}{Start time (in seconds) of desired plotted area. Default is \code{0}.
Alternatively, a vector giving the first and last start time in the
animation.}

\item{end}{End time (in seconds) of desired plotted area. Default is \code{0}
(= the entire file). Alternatively, a vector giving the first and last
end time in the animation.}

\item{spec_freqRange}{Vector of two integers giving the frequency range to be
used for plotting spectrograms. Default is \code{c(0,5000)}. Alternatively, a
vector of four integers giving the first and last lowest frequency, followed
by the first and last highest frequency in the animation; i.e.,
\code{c(0,0,5000,10000)} will produce an animation where the upper frequency
boundary gradually increases from 5000 Hz to 10,000 Hz.}

\item{spec_windowLength}{Window length in seconds for generating spectrograms.
Default is \code{0.005}. Alternatively, a vector giving the first and last
window lengths in the animation.}

\item{spec_dynamicRange}{Dynamic range in dB for generating spectrograms. The
maximum intensity minus \code{spec_dynamicRange} will all be printed in white.
Default is \code{50}. Alternatively, a vector giving the first and last
dynamic range values in the animation.}

\item{spec_timeStep}{How many time steps should be calculated for spectrograms?
Default is \code{1000}. Alternatively, a vector giving the first and last
time step values in the animation.}

\item{pitch_timeStep}{Measurement interval in seconds for tracking pitch.
Default is \code{NULL}, in which case the measurement interval is equal to
0.75 / \code{pitch_floor}. Alternatively, a vector giving the first and last
measurement intervals in the animation.}

\item{pitch_floor}{Frequency in Hz; no pitch candidates considered below
this frequency. Default is \code{75}. Alternatively, a vector giving the first and
last pitch floors to be used in the animation.}

\item{pitch_ceiling}{Frequency in Hz; no pitch candidates considered above
this frequency. Default is \code{600}. Alternatively, a vector giving the first and
last pitch ceilings to be used in the animation.}

\item{pitch_freqRange}{Vector of two integers giving the frequency range to be
used for producing pitch plots. Default is \code{c(50,500)}. If the frequency
scales \code{semitones} or \code{erb} are used, the pitch range is automatically reset
to the Praat defaults for these scales (\code{c(-12,30)} and \code{c(0,10)},
respectively). Alternatively, a
vector of four integers giving the first and last lowest frequency, followed
by the first and last highest frequency in the animation
(see \code{spec_freqRange} for usage details).}

\item{pitch_semitonesRe}{Frequency in Hz giving the reference level for
converting pitch frequency to semitones. Default is \code{100}.
Alternatively, a vector giving the first and
last semitone reference levels to be used in the animation.}

\item{formant_timeStep}{Measurement interval in seconds for tracking formants.
Default is \code{NULL}, in which case the measurement interval is equal to
\code{formant_windowLength} / 4. Alternatively, a vector giving the first and
last measurement intervals to be used in the animation.}

\item{formant_windowLength}{The effective duration of the analysis window
used for tracking formants in seconds; the actual duration of the
analysis window is twice this value. Alternatively, a vector giving the first
and last window lengths to be used in the animation.}

\item{formant_dynamicRange}{Dynamic range in dB for producing formant plots.
When a formant plot of \code{formant_plotType='speckle'} is drawn, no formants are
shown in frames with intensity level \code{formant_dynamicRange} below the maximum
intensity. Default is \code{30}. If set to \code{0}, all formants are shown.
Alternatively, a vector giving the first
and last dynamic range levels to be used in the animation.}

\item{formant_freqRange}{Vector of two integers giving the frequency range to
be used for producing formant plots. Default is \code{c(0,5500)}. Alternatively, a
vector of four integers giving the first and last lowest frequency, followed
by the first and last highest frequency in the animation
(see \code{spec_freqRange} for usage details).}

\item{intensity_timeStep}{Measurement interval in seconds for tracking
intensity. Default is \code{NULL}, in which case the measurement interval is
equal to 0.8 * \code{intensity_minPitch}. Alternatively, a vector giving the first
and last measurement intervals to be used in the animation.}

\item{intensity_minPitch}{Lowest pitch in Hz used when calculating
intensity; default is \code{100}. Alternatively, a vector giving the first
and last minimum pitch levels to be used in the animation.}

\item{intensity_range}{Vector of two integers giving the intensity range to be
used for producing intensity plots. Default is \code{NULL}, in which case the
range is simply the minimum and maximum levels in the curve. Alternatively, a
vector of four integers giving the first and last lowest level, followed
by the first and last highest level in the animation
(see \code{spec_freqRange} for usage details).}

\item{...}{Further arguments passed to \code{praatpicture}.}
}
\value{
No return value, produces an animated figure.
}
\description{
Animate some aspect of a Praat Picture-style plot
of acoustic data, potentially aligned with transcriptions.
}
\examples{
\dontrun{
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')

# Show increasing frequency range
praatanimation(soundFile, spec_freqRange=c(0,0,4000,12000))

# Transition from narrowband to broadband spectrogram
praatanimation(soundFile, spec_windowLength=c(0.005,0.03))

# Etc.
}
}
\seealso{
This function is a wrapper for either \code{gifski::save_gif()}
or \code{av::av_capture_graphics()} used to
produce animations based on \code{praatpicture()}. For more detail on your
options, see the \code{praatpicture()} help file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formantplot.R
\name{formantplot}
\alias{formantplot}
\title{Plot formant object}
\usage{
formantplot(
  fm,
  start,
  end,
  tfrom0 = TRUE,
  tgbool = FALSE,
  lines = NULL,
  focusTierColor = "black",
  focusTierLineType = "dotted",
  dynamicRange = 30,
  freqRange = c(0, 5500),
  plotType = "speckle",
  color = "black",
  ind = NULL,
  min_max_only = FALSE,
  highlight = NULL,
  axisLabel = "Frequency (Hz)",
  drawSize = 1,
  speckleSize = 1,
  suppressAxes = FALSE
)
}
\arguments{
\item{fm}{Formant object loaded using \link[rPraat:formant.read]{rPraat::formant.read} or similar
object, i.e. a \code{list} object containing the elements \code{t} (time),
\code{frequencyArray} (array or formant frequencies with number of rows
corresponding to the length of \code{t}), \code{maxnFormants} (integer giving the
number of formants), \code{intensityVector} (vector of intensity values of
the same length as \code{t}), and \code{conv2db} (logical).}

\item{start}{Start time (in seconds) of desired plotted area.}

\item{end}{End time (in seconds) of desired plotted area.}

\item{tfrom0}{Logical; should time on the x-axis run from 0 or from the
original time? Default is \code{TRUE}.}

\item{tgbool}{Logical; should dotted lines be plotted corresponding to
locations in a TextGrid? Default is \code{FALSE}.}

\item{lines}{Numeric vector giving locations in seconds of locations from
a TextGrid to be plotted with dotted lines. Default is \code{NULL}.}

\item{focusTierColor}{String or vector of strings giving the color(s) to
use for plotting focus tier lines. If multiple tiers are focused, a vector
of the same length can be passed, and the nth tier will be plotted in the
nth color. Default is \code{'black'}.}

\item{focusTierLineType}{String or vector of strings giving the line
type(s) for plotting focus tier lines. If multiple tiers are focused, a
vector of the same length can be passed, and the nth tier will be plotted in
the nth line type. Default is \code{'dotted'}.}

\item{dynamicRange}{Dynamic range in dB for producing formant plots.
When a formant plot of \code{plotType='speckle'} is drawn, no formants are
shown in frames with intensity level \code{dynamicRange} below the maximum
intensity. Default is \code{30}. If set to \code{0}, all formants are shown.}

\item{freqRange}{Vector of two integers giving the frequency range to be
used for producing formant plots. Default is \code{c(0,5500)}.}

\item{plotType}{String giving the type of formant plot to produce;
default is \code{speckle} (a point plot), the only other option is \code{draw} (a line
plot). Alternatively a vector \code{c('draw','speckle')} can be passed, in which
case both are used.}

\item{color}{String or vector of strings giving the name(s) of
colors to be used for plotting formants. If one color is provided, all
formants will be plotted in this color. If multiple colors are provided,
different formants will be shown in different colors. Default is \code{'black'}.}

\item{ind}{Integer indexing formants relative to other plot components.
Default is \code{NULL}.}

\item{min_max_only}{Logical; should only minimum and maximum values be given
on the y-axis? Default is \code{FALSE}. Can also be a logical vector if some but
not all plot components should have minimum and maximum values on the y-axis.
Ignored for TextGrid component.}

\item{highlight}{Named list giving parameters for differential
highlighting of formants based on the time domain. This list
should contain information about which parts of the plot to highlight, either
done with the \code{start} and \code{end} arguments which must be numbers or numeric
vectors, or using the \code{tier} and \code{label} arguments to highlight based on
information in a plotted TextGrid. Further contains the optional arguments
\code{color} (string or vector of strings, see \code{color}),
\code{drawSize} or \code{speckleSize} (both numeric), and \code{background}
(a string specifying a background color). Can be nested lists to highlight
different parts of a figure in different ways.}

\item{axisLabel}{String giving the name of the label to print along the
y-axis when plotting formants. Default is \code{Frequency (Hz)}.}

\item{drawSize}{Number indicating the line width if
\code{plotType} is \code{'draw'}. Default is \code{1}. Controls the \code{lwd} argument of
\link[graphics:lines]{graphics::lines}.}

\item{speckleSize}{Number indicating the point size of if \verb{_plotType} is
\code{'speckle'}. Default is \code{1}. Controls the \code{cex} arguments of
\link[graphics:points]{graphics::points}.}

\item{suppressAxes}{Should all axes and the box around plots be suppressed?
Default is \code{FALSE}. Generally not recommended.}
}
\value{
No return values, called internally by \link{praatpicture} and sibling
functions.
}
\description{
Function for plotting formant objects called by \link{praatpicture}. Instead of
using this function directly, just use
\code{praatpicture('my_sound_file', frames='formant')}.
}
\examples{
# Don't use directly
datapath <- system.file('extdata', package='praatpicture')
soundFile <- paste0(datapath, '/1.wav')
praatpicture(soundFile, frames='formant')
}
